Package: BIEN
Title: Tools for Accessing the Botanical Information and Ecology
        Network Database
Version: 1.2.8
Authors@R: person("Brian", "Maitner", email = "bmaitner@gmail.com", role = c("aut", "cre"))
Description: Provides Tools for Accessing the Botanical Information and Ecology Network Database.  The BIEN database contains cleaned and standardized botanical data including occurrence, trait, plot and taxonomic data (See <https://bien.nceas.ucsb.edu/bien/> for more Information).  This package provides functions that query the BIEN database by constructing and executing optimized SQL queries.
Depends: R (>= 4.1.0), RPostgreSQL
License: MIT + file LICENSE
Imports: DBI, ape, sf, terra, doParallel, parallel, foreach
Suggests: knitr, rmarkdown, testthat, curl, maps
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-28 22:08:14 UTC; bmaitner
Author: Brian Maitner [aut, cre]
Maintainer: Brian Maitner <bmaitner@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-30 07:40:16 UTC
Built: R 4.5.2; ; 2026-04-30 11:04:07 UTC; unix

Package: BIEN
Title: Tools for Accessing the Botanical Information and Ecology
        Network Database
Version: 1.2.6
Authors@R: person("Brian", "Maitner", email = "bmaitner@gmail.com", role = c("aut", "cre"))
Description: Provides Tools for Accessing the Botanical Information and Ecology Network Database.  The BIEN database contains cleaned and standardized botanical data including occurrence, trait, plot and taxonomic data (See <https://bien.nceas.ucsb.edu/bien/> for more Information).  This package provides functions that query the BIEN database by constructing and executing optimized SQL queries.
Depends: R (>= 3.2.1), RPostgreSQL
License: MIT + file LICENSE
Imports: DBI, ape, sf, fasterize, raster, terra, doParallel, parallel,
        foreach
Suggests: knitr, rmarkdown, testthat, maps
VignetteBuilder: knitr
RoxygenNote: 7.2.2
NeedsCompilation: no
Packaged: 2023-01-06 03:04:57 UTC; Brian Maitner
Author: Brian Maitner [aut, cre]
Maintainer: Brian Maitner <bmaitner@gmail.com>
Repository: CRAN
Date/Publication: 2023-01-06 07:50:10 UTC
Built: R 4.2.0; ; 2023-01-11 03:25:25 UTC; unix

# Generated by roxygen2: do not edit by hand

export(BIEN_list_all)
export(BIEN_list_country)
export(BIEN_list_county)
export(BIEN_list_sf)
export(BIEN_list_state)
export(BIEN_metadata_citation)
export(BIEN_metadata_database_version)
export(BIEN_metadata_list_political_names)
export(BIEN_metadata_match_data)
export(BIEN_occurrence_box)
export(BIEN_occurrence_country)
export(BIEN_occurrence_county)
export(BIEN_occurrence_family)
export(BIEN_occurrence_genus)
export(BIEN_occurrence_records_per_species)
export(BIEN_occurrence_sf)
export(BIEN_occurrence_species)
export(BIEN_occurrence_state)
export(BIEN_phylogeny_complete)
export(BIEN_phylogeny_conservative)
export(BIEN_phylogeny_label_nodes)
export(BIEN_plot_country)
export(BIEN_plot_dataset)
export(BIEN_plot_datasource)
export(BIEN_plot_list_datasource)
export(BIEN_plot_list_sampling_protocols)
export(BIEN_plot_metadata)
export(BIEN_plot_name)
export(BIEN_plot_sampling_protocol)
export(BIEN_plot_sf)
export(BIEN_plot_state)
export(BIEN_ranges_box)
export(BIEN_ranges_genus)
export(BIEN_ranges_intersect_species)
export(BIEN_ranges_list)
export(BIEN_ranges_load_species)
export(BIEN_ranges_sf)
export(BIEN_ranges_shapefile_to_skinny)
export(BIEN_ranges_skinny_ranges_to_richness_raster)
export(BIEN_ranges_species)
export(BIEN_ranges_species_bulk)
export(BIEN_stem_datasource)
export(BIEN_stem_family)
export(BIEN_stem_genus)
export(BIEN_stem_sampling_protocol)
export(BIEN_stem_species)
export(BIEN_taxonomy_family)
export(BIEN_taxonomy_genus)
export(BIEN_taxonomy_species)
export(BIEN_trait_country)
export(BIEN_trait_family)
export(BIEN_trait_genus)
export(BIEN_trait_list)
export(BIEN_trait_mean)
export(BIEN_trait_species)
export(BIEN_trait_trait)
export(BIEN_trait_traitbyfamily)
export(BIEN_trait_traitbygenus)
export(BIEN_trait_traitbyspecies)
export(BIEN_trait_traits_per_species)
import(RPostgreSQL)
import(doParallel)
import(foreach)
importFrom(DBI,dbDriver)
importFrom(DBI,dbFetch)
importFrom(ape,getMRCA)
importFrom(ape,read.tree)
importFrom(sf,read_sf)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_text)
importFrom(sf,st_bbox)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_transform)
importFrom(sf,st_write)
importFrom(terra,rast)
importFrom(terra,rasterize)
importFrom(terra,values)

# RBIEN
Tools for accessing the Botanical Information and Ecology Network (BIEN) database

# News:
New license: We've updated the license on the BIEN data to a CC-BY license.  If you are using the Github version of the R package, this change will be reflected when you use the function BIEN_metadata_citation().  A forthcoming publication will also detail this change (and other updates to associated data products and software).

## Installing
To install the development version of BIEN from Github:

```{r}
devtools::install_github("bmaitner/RBIEN")

```

<a href="https://buymeacoffee.com/maitner" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/v2/default-blue.png" alt="Buy Me A Coffee" style="height: 60px !important;width: 217px !important;" ></a>

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----load-packages, message = F,warning = FALSE, results = 'hide'-------------
library(BIEN)
library(ape) #Package for working with phylogenies in R
library(maps) #Useful for making quick maps of occurrences
library(sf) # A package for spatial data

## ----load-vignette, eval = FALSE----------------------------------------------
#  vignette("BIEN")

## ----xs-occurrences-----------------------------------------------------------

Xanthium_strumarium <- BIEN_occurrence_species(species = "Xanthium strumarium")


## ----view-xs-occurrences------------------------------------------------------

str(Xanthium_strumarium)

head(Xanthium_strumarium)


## ----xs-occurrences-pt-2, eval = FALSE----------------------------------------
#  
#  Xanthium_strumarium_full <- BIEN_occurrence_species(species = "Xanthium strumarium",
#                                                      cultivated = TRUE,
#                                                      all.taxonomy = TRUE,
#                                                      native.status = TRUE,
#                                                      observation.type = TRUE,
#                                                      political.boundaries = TRUE)
#  
#  

## ----map-xs, eval = FALSE-----------------------------------------------------
#  # Make a quick map to plot our points on
#  
#    map('world', fill = TRUE, col= "grey", bg = "light blue")
#  
#  # Plot the points from the full query in red
#  
#    points(cbind(Xanthium_strumarium_full$longitude,
#                 Xanthium_strumarium_full$latitude),
#           col = "red",
#           pch = 20,
#           cex = 1)
#  
#  # Plot the points from the default query in blue
#  
#    points(cbind(Xanthium_strumarium$longitude,
#                 Xanthium_strumarium$latitude),
#           col = "blue",
#           pch = 20,
#           cex = 1)
#  

## ----occs-bahamas, eval = FALSE-----------------------------------------------
#  
#  Bahamas <- BIEN_occurrence_country(country =  "Bahamas")
#  
#  #Let's see how many species we have
#  
#    length(unique(Bahamas$scrubbed_species_binomial))
#    #About 400 species with valid occurrence records.
#  
#  #Now, let's take a look at where those occurrences are:
#  
#    map(regions = "Bahamas" ,
#        fill = TRUE ,
#        col= "grey",
#        bg = "light blue")
#  
#    points(cbind(Bahamas$longitude,Bahamas$latitude),
#           col = "blue",
#           pch = 20,
#           cex = 1)
#  
#  #Looks like some islands are considerably better sampled than others.
#  

## ----xs-range-----------------------------------------------------------------

Xanthium_strumarium_range <- BIEN_ranges_load_species(species = "Xanthium strumarium")


## ----xs-range-map-------------------------------------------------------------

#First, let's add a base map so that our range has some context:

  map('world', fill = TRUE ,
      col= "grey",
      bg = "light blue",
      xlim = c(-180, -20),
      ylim = c(-60, 80))

#Now, we can add the range map:
  plot(Xanthium_strumarium_range[1],
       col = "green",
       add = TRUE)




## ----xs-range-and-points------------------------------------------------------

map('world',
    fill = TRUE ,
    col = "grey",
    bg = "light blue",
    xlim = c(-180, -20),
    ylim = c(-60, 80))

plot(Xanthium_strumarium_range[1],
     col = "green",
     add = TRUE)

points(cbind(Xanthium_strumarium$longitude,Xanthium_strumarium$latitude),
       col = "blue",
       pch = 20,
       cex = 1)



## ----luq-occs, eval = FALSE---------------------------------------------------
#  
#  LUQUILLO <- BIEN_plot_name(plot.name = "LUQUILLO")
#  
#  head(LUQUILLO)
#  

## ---- eval = FALSE------------------------------------------------------------
#  
#  LUQUILLO_full <- BIEN_plot_name(plot.name = "LUQUILLO",
#                                  cultivated = TRUE,
#                                  all.taxonomy = TRUE,
#                                  native.status = TRUE,
#                                  political.boundaries = TRUE,
#                                  all.metadata = TRUE)
#  

## ----salix-traits, eval = FALSE-----------------------------------------------
#  
#  Salix_traits <- BIEN_trait_genus(genus = "Salix")
#  

## ----trait-list, eval = FALSE-------------------------------------------------
#  
#  BIEN_trait_list()
#  

## ----leaf-area, eval = FALSE--------------------------------------------------
#  
#  leaf_area <- BIEN_trait_trait(trait = "leaf area")
#  

## ----taxonomy-----------------------------------------------------------------

Asclepias_taxonomy <- BIEN_taxonomy_genus(genus = "Asclepias")

#We see that the genus Asclepias falls within the family Apocynaceae and the order Gentianales.

#You'll also notice that a given species may appear more than once (due to multiple circumscriptions, some of which may be illegitimate).

#If we'd just like to know all the speciess that aren't illegitimate:

Asclepias_species <- unique(Asclepias_taxonomy$scrubbed_species_binomial[Asclepias_taxonomy$scrubbed_taxonomic_status %in% c("accepted",  "no opinion")])



## ----phylogeny, eval = FALSE--------------------------------------------------
#  
#  phylo <- BIEN_phylogeny_conservative()
#  
#  #Let's make sure it looks alright
#  
#    plot.phylo(x = phylo, show.tip.label =  FALSE)
#  
#  #If we just want to see which species are included
#  
#    phylo_species <- phylo$tip.label
#  
#  

## ----stems, eval = FALSE------------------------------------------------------
#  
#  Cupressus_arizonica_stems <- BIEN_stem_species("Cupressus arizonica")
#  

## ----lists, eval = FALSE------------------------------------------------------
#  
#  Bahamas_species_list <- BIEN_list_country(country = "Bahamas")
#  
#  #Notice that we find many more species listed than we found occurrence records for.  What happened?  There are many records coming from the Bahamas that lack coordinates.  These records are used used in the "_list_" functions, but not the occurrence functions.
#  

## ----lists-multi-country, eval = FALSE----------------------------------------
#  
#  country_vector <- c("Haiti","Dominican Republic")
#  
#  Haiti_DR <- BIEN_list_country(country = country_vector)
#  

## ----lists-pol-divs-----------------------------------------------------------
#To see all of the political division names, and associated codes, we can use this function:

political_names <- BIEN_metadata_list_political_names()

#Let's take a look at what the dataframe contains:

head(political_names)

#In addition to the standardized country, state (state_province_ascii) and county (county_parish_ascii) names, we have the associated codes that can be used in BIEN functions.
#Note that 'state' refers to any primary political division (e.g. province), and 'county' refers to any secondary political division (e.g. parish).

#Looking at the political_names dataframe, we see that the Dominican Republic has country code "DO", and Haiti has country code "HT"

Haiti_DR_from_codes <- BIEN_list_country(country.code = c("HT","DO"))



## ----md-1---------------------------------------------------------------------

BIEN_metadata_database_version()


## ----selaginella-occs, eval = FALSE-------------------------------------------
#  
#  Selaginella_selaginoides_occurrences <- BIEN_occurrence_species("Selaginella selaginoides", new.world = NULL)
#  

## ----selaginella-md, eval = FALSE---------------------------------------------
#  
#  citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences)
#  

## ----md-2, eval = FALSE-------------------------------------------------------
#  
#  temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a temporary working directory
#  
#  
#  citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
#                                       bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
#                                        acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))
#  
#  

## ----md-3, eval = FALSE-------------------------------------------------------
#  #First, let's get some trait data:
#  selaginella_selaginoides_traits <- BIEN_trait_species(species = "Selaginella selaginoides")
#  
#  #Now, we just need to modify our previous bit of code to include the trait data as well:
#  
#  temp_dir <- file.path(tempdir(), "BIEN_temp")
#  
#  citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
#                                        trait.dataframe = selaginella_selaginoides_traits,
#                                        bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
#                                        acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))
#  

## ----setup, include = FALSE---------------------------------------------------
library(BIEN)

bien_vignette_online <- !is.null(
  tryCatch(expr = BIEN:::.BIEN_sql("SELECT 1;"),
           error = function(e){NULL})
)

knitr::opts_chunk$set(echo = TRUE,
                      eval = bien_vignette_online)

if(!bien_vignette_online){
  message("BIEN database is unavailable or refusing connections; remote-query chunks were not evaluated while building this vignette.")
}

## ----load-packages, message = F,warning = FALSE, results = 'hide'-------------
library(ape) #Package for working with phylogenies in R
library(maps) #Useful for making quick maps of occurrences
library(sf) # A package for spatial data

## ----load-vignette, eval = FALSE----------------------------------------------
# vignette("BIEN")

## ----xs-occurrences-----------------------------------------------------------

Xanthium_strumarium <- BIEN_occurrence_species(species = "Xanthium strumarium")


## ----view-xs-occurrences------------------------------------------------------

str(Xanthium_strumarium)

head(Xanthium_strumarium)


## ----xs-occurrences-pt-2, eval = FALSE----------------------------------------
# 
# Xanthium_strumarium_full <- BIEN_occurrence_species(species = "Xanthium strumarium",
#                                                     cultivated = TRUE,
#                                                     all.taxonomy = TRUE,
#                                                     native.status = TRUE,
#                                                     observation.type = TRUE,
#                                                     political.boundaries = TRUE)
# 
# 

## ----map-xs, eval = FALSE-----------------------------------------------------
# # Make a quick map to plot our points on
# 
#   map('world', fill = TRUE, col= "grey", bg = "light blue")
# 
# # Plot the points from the full query in red
# 
#   points(cbind(Xanthium_strumarium_full$longitude,
#                Xanthium_strumarium_full$latitude),
#          col = "red",
#          pch = 20,
#          cex = 1)
# 
# # Plot the points from the default query in blue
# 
#   points(cbind(Xanthium_strumarium$longitude,
#                Xanthium_strumarium$latitude),
#          col = "blue",
#          pch = 20,
#          cex = 1)
# 

## ----occs-bahamas, eval = FALSE-----------------------------------------------
# 
# Bahamas <- BIEN_occurrence_country(country =  "Bahamas")
# 
# #Let's see how many species we have
# 
#   length(unique(Bahamas$scrubbed_species_binomial))
#   #About 400 species with valid occurrence records.
# 
# #Now, let's take a look at where those occurrences are:
# 
#   map(regions = "Bahamas" ,
#       fill = TRUE ,
#       col= "grey",
#       bg = "light blue")
# 
#   points(cbind(Bahamas$longitude,Bahamas$latitude),
#          col = "blue",
#          pch = 20,
#          cex = 1)
# 
# #Looks like some islands are considerably better sampled than others.
# 

## ----xs-range-----------------------------------------------------------------

Xanthium_strumarium_range <- BIEN_ranges_load_species(species = "Xanthium strumarium")


## ----xs-range-map-------------------------------------------------------------

#First, let's add a base map so that our range has some context:

  map('world', fill = TRUE ,
      col= "grey",
      bg = "light blue",
      xlim = c(-180, -20),
      ylim = c(-60, 80))

#Now, we can add the range map:
  plot(Xanthium_strumarium_range[1],
       col = "green",
       add = TRUE)




## ----xs-range-and-points------------------------------------------------------

map('world',
    fill = TRUE ,
    col = "grey",
    bg = "light blue",
    xlim = c(-180, -20),
    ylim = c(-60, 80))

plot(Xanthium_strumarium_range[1],
     col = "green",
     add = TRUE)

points(cbind(Xanthium_strumarium$longitude,Xanthium_strumarium$latitude),
       col = "blue",
       pch = 20,
       cex = 1)



## ----luq-occs, eval = FALSE---------------------------------------------------
# 
# LUQUILLO <- BIEN_plot_name(plot.name = "LUQUILLO")
# 
# head(LUQUILLO)
# 

## ----eval = FALSE-------------------------------------------------------------
# 
# LUQUILLO_full <- BIEN_plot_name(plot.name = "LUQUILLO",
#                                 cultivated = TRUE,
#                                 all.taxonomy = TRUE,
#                                 native.status = TRUE,
#                                 political.boundaries = TRUE,
#                                 all.metadata = TRUE)
# 

## ----salix-traits, eval = FALSE-----------------------------------------------
# 
# Salix_traits <- BIEN_trait_genus(genus = "Salix")
# 

## ----trait-list, eval = FALSE-------------------------------------------------
# 
# BIEN_trait_list()
# 

## ----leaf-area, eval = FALSE--------------------------------------------------
# 
# leaf_area <- BIEN_trait_trait(trait = "leaf area")
# 

## ----taxonomy-----------------------------------------------------------------

Asclepias_taxonomy <- BIEN_taxonomy_genus(genus = "Asclepias")

#We see that the genus Asclepias falls within the family Apocynaceae and the order Gentianales.

#You'll also notice that a given species may appear more than once (due to multiple circumscriptions, some of which may be illegitimate).

#If we'd just like to know all the speciess that aren't illegitimate:

Asclepias_species <- unique(Asclepias_taxonomy$scrubbed_species_binomial[Asclepias_taxonomy$scrubbed_taxonomic_status %in% c("accepted",  "no opinion")])



## ----phylogeny, eval = FALSE--------------------------------------------------
# 
# phylo <- BIEN_phylogeny_conservative()
# 
# #Let's make sure it looks alright
# 
#   plot.phylo(x = phylo, show.tip.label =  FALSE)
# 
# #If we just want to see which species are included
# 
#   phylo_species <- phylo$tip.label
# 
# 

## ----stems, eval = FALSE------------------------------------------------------
# 
# Cupressus_arizonica_stems <- BIEN_stem_species("Cupressus arizonica")
# 

## ----lists, eval = FALSE------------------------------------------------------
# 
# Bahamas_species_list <- BIEN_list_country(country = "Bahamas")
# 
# #Notice that we find many more species listed than we found occurrence records for.  What happened?  There are many records coming from the Bahamas that lack coordinates.  These records are used used in the "_list_" functions, but not the occurrence functions.
# 

## ----lists-multi-country, eval = FALSE----------------------------------------
# 
# country_vector <- c("Haiti","Dominican Republic")
# 
# Haiti_DR <- BIEN_list_country(country = country_vector)
# 

## ----lists-pol-divs-----------------------------------------------------------
#To see all of the political division names, and associated codes, we can use this function:

political_names <- BIEN_metadata_list_political_names()

#Let's take a look at what the dataframe contains:

head(political_names)

#In addition to the standardized country, state (state_province_ascii) and county (county_parish_ascii) names, we have the associated codes that can be used in BIEN functions.
#Note that 'state' refers to any primary political division (e.g. province), and 'county' refers to any secondary political division (e.g. parish).

#Looking at the political_names dataframe, we see that the Dominican Republic has country code "DO", and Haiti has country code "HT"

Haiti_DR_from_codes <- BIEN_list_country(country.code = c("HT","DO"))



## ----md-1---------------------------------------------------------------------

BIEN_metadata_database_version()


## ----selaginella-occs, eval = FALSE-------------------------------------------
# 
# Selaginella_selaginoides_occurrences <- BIEN_occurrence_species("Selaginella selaginoides", new.world = NULL)
# 

## ----selaginella-md, eval = FALSE---------------------------------------------
# 
# citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences)
# 

## ----md-2, eval = FALSE-------------------------------------------------------
# 
# temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a temporary working directory
# 
# 
# citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
#                                      bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
#                                       acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))
# 
# 

## ----md-3, eval = FALSE-------------------------------------------------------
# #First, let's get some trait data:
# selaginella_selaginoides_traits <- BIEN_trait_species(species = "Selaginella selaginoides")
# 
# #Now, we just need to modify our previous bit of code to include the trait data as well:
# 
# temp_dir <- file.path(tempdir(), "BIEN_temp")
# 
# citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
#                                       trait.dataframe = selaginella_selaginoides_traits,
#                                       bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
#                                       acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))
# 

#####################

#'Extract occurrence data for specified species from BIEN
#'
#'BIEN_occurrence_species downloads occurrence records for specific species from the BIEN database.
#' @param species A single species, or a vector of species.  Genus and species should be separated by a space. Genus should be capitalized.
#' @param only.geovalid Should the returned records be limited to those with validated coordinates?  Default is TRUE
#' @template occurrence
#' @return Dataframe containing occurrence records for the specified species.
#' @examples \dontrun{
#' BIEN_occurrence_species("Abies amabilis")
#' species_vector<-c("Abies amabilis", "Acer nigrum")
#' BIEN_occurrence_species(species_vector)
#' BIEN_occurrence_species(species_vector,all.taxonomy = TRUE)}
#' @family occurrence functions
#' @export
BIEN_occurrence_species <- function(species,
                                  cultivated = FALSE,
                                  new.world = NULL,
                                  all.taxonomy = FALSE,
                                  native.status = FALSE,
                                  natives.only = TRUE,
                                  observation.type = FALSE,
                                  political.boundaries = FALSE,
                                  collection.info = FALSE,
                                  only.geovalid = TRUE,
                                  ...){
  
  #Test input
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(species)
  .is_log(native.status)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(natives.only)
  .is_log(collection.info)
  .is_log(only.geovalid)
  
  #set conditions for query
  cultivated_<-.cultivated_check(cultivated)  
  newworld_<-.newworld_check(new.world)
  taxonomy_<-.taxonomy_check(all.taxonomy)  
  native_<-.native_check(native.status)
  observation_<-.observation_check(observation.type)
  political_<-.political_check(political.boundaries)  
  natives_<-.natives_check(natives.only)
  collection_<-.collection_check(collection.info)
  geovalid_<-.geovalid_check(only.geovalid)
  
  
  # set the query
  query <- paste("SELECT scrubbed_species_binomial",taxonomy_$select,
                  native_$select,political_$select," ,latitude, longitude,date_collected,
                 datasource,dataset,dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",
                 collection_$select,
                 cultivated_$select,
                 newworld_$select,
                 observation_$select,
                 geovalid_$select,"
                 FROM view_full_occurrence_individual 
                 WHERE scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ")",
                 cultivated_$query,
                 newworld_$query,
                 natives_$query,
                 observation_$query,
                 geovalid_$query, "
                 AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                 AND (georef_protocol is NULL OR georef_protocol<>'county centroid') 
                 AND (is_centroid IS NULL OR is_centroid=0) 
                 AND scrubbed_species_binomial IS NOT NULL ;")
  
  return(.BIEN_sql(query, ...))
  
}

##############

#'Extract occurrence data for specified sf polygon
#'
#'BIEN_occurrence_sf downloads occurrence records falling within a user-specified sf polygon
#' @param sf An object of class sf. Note that the projection must be WGS84.
#' @param only.geovalid Should the returned records be limited to those with validated coordinates?  Default is TRUE
#' @template occurrence
#' @return Dataframe containing occurrence records falling within the polygon.
#' @examples \dontrun{
#' library(sf)  
#' 
#' # first, we download an example shapefile to use (a species range)
#' 
#' BIEN_ranges_species("Carnegiea gigantea")#saves range to the current working directory
#' 
#' # load the range map as an sf object
#' 
#' sf <- st_read(dsn = ".",layer = "Carnegiea_gigantea")
#' 
#' # get the occurrences that occur within the polygon.
#' 
#' species_occurrences <- BIEN_occurrence_sf(sf = sf)
#' }
#' @family occurrence functions
#' @importFrom sf st_geometry st_as_text st_bbox
#' @export
BIEN_occurrence_sf <- function(sf,
                               cultivated = FALSE,
                               new.world = NULL,
                               all.taxonomy = FALSE,
                               native.status = FALSE,
                               natives.only = TRUE,
                               observation.type = FALSE,
                               political.boundaries = FALSE,
                               collection.info = FALSE,
                               only.geovalid = TRUE,
                               ...){
  
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(natives.only)
  .is_log(collection.info)
  .is_log(only.geovalid)
  
  
  # Convert the sf to wkt (needed for sql query)  
  wkt <- sf |>
    st_geometry() |>
    st_as_text()
  
  # Get bounding box of sf (used as a sort of index to make query a bit faster)
  sf_bbox <-
    sf |>
    st_bbox()
  
  long_min <- sf_bbox["xmin"]
  long_max <- sf_bbox["xmax"]
  lat_min <- sf_bbox["ymin"]
  lat_max <- sf_bbox["ymax"]
  
  
  #set conditions for query
  
  cultivated_ <- .cultivated_check(cultivated)  
  newworld_ <- .newworld_check(new.world)
  taxonomy_ <- .taxonomy_check(all.taxonomy)  
  native_ <- .native_check(native.status)
  observation_ <- .observation_check(observation.type)
  political_ <- .political_check(political.boundaries)  
  natives_ <- .natives_check(natives.only)
  collection_ <- .collection_check(collection.info)
  geovalid_<-.geovalid_check(only.geovalid)
  
  
  # set the query
  query <- paste("SELECT scrubbed_species_binomial",
                 taxonomy_$select,
                 native_$select,
                 political_$select," ,
                  latitude, longitude, date_collected,datasource,dataset,
                 dataowner,custodial_institution_codes,collection_code,
                 a.datasource_id",collection_$select,cultivated_$select,
                 newworld_$select,
                 observation_$select,
                 geovalid_$select,"
                    FROM 
                          (SELECT * FROM view_full_occurrence_individual 
                           WHERE higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                           AND (georef_protocol is NULL OR georef_protocol<>'county centroid')
                           AND latitude BETWEEN ",lat_min," AND ",lat_max,"AND longitude BETWEEN ",long_min," AND ",long_max,") a 
                    WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),a.geom)",
                      cultivated_$query,
                      newworld_$query,
                      natives_$query,
                      observation_$query,
                      geovalid_$query, "
                      AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi')
                      AND (georef_protocol is NULL OR georef_protocol<>'county centroid') 
                      AND (is_centroid IS NULL OR is_centroid=0) 
                      AND scrubbed_species_binomial IS NOT NULL ;")
  
  # create query to retrieve
  df <- .BIEN_sql(query, ...)
  
  
  if(length(df) == 0){
    
    message("No occurrences found")
    return(invisible(NULL))
    
  }else{
    
    return(df)
    
  }
  
}



###############################

#'Extract species list by country
#'
#'BIEN_list_country downloads a list of all species within a country or countries from the BIEN database.
#' @param country A single country or a vector of countries.
#' @param country.code A single country code or a vector of country codes equal in length to the vector of states/province codes.
#' @template list
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @return Dataframe containing species list(s) for the specified country or countries.
#' @examples \dontrun{
#' BIEN_list_country("Canada")
#' country_vector<-c("Canada","United States")
#' BIEN_list_country(country_vector)}
#' @family list functions
#' @export
BIEN_list_country <- function(country = NULL,
                              country.code = NULL,
                              cultivated = FALSE,
                              new.world = NULL,
                              ...){
  
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_char(country)
  .is_char(country.code)
  if(is.null(country) & is.null(country.code)) {
    stop("Please supply either a country name or 2-digit ISO code")
    }
  
  newworld_ <- .newworld_check(new.world)
  
  
  #set base query components
    sql_select <-  paste("SELECT DISTINCT country, scrubbed_species_binomial ")
    
    sql_from <- paste(" FROM species_by_political_division ")
    
    if(is.null(country.code)){
      
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") AND scrubbed_species_binomial IS NOT NULL")
      
    }else{
      
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND scrubbed_species_binomial IS NOT NULL")  
    }
    

  # adjust for optional parameters
  
  if(!cultivated){
    
    # sql_where <- paste(sql_where, " AND (is_cultivated_observation = 0 OR is_cultivated_observation IS NULL) ")
    
  }else{
    
    sql_select  <- paste(sql_select, ",is_cultivated_in_region")
  }
  
  #if(!new.world){
  #  sql_select <- paste(sql_select,",is_new_world")
  #  sql_where <- paste(sql_where, "AND is_new_world = 1 ")  
  #}else{
  #  sql_where <- paste(sql_where, "AND is_new_world = 1 ")
  #}
  
  
  
  
  # form the final query
  query <- paste(sql_select,newworld_$select, sql_from, sql_where,newworld_$query, " ;")
  
  
  return(.BIEN_sql(query, ...))
  #return(.BIEN_sql(query))
  
  }

############################


#'Extract a species list by state/province
#'
#'BIEN_list_state produces a list of all species with geovalidated occurrences falling within specified state(s) or province(s).
#' @param state A state or vector of states (or other primary political divisions, e.g. provinces).
#' @param country A single country or a vector of countries equal in length to the vector of states/provinces.
#' @param state.code A single state/province code, or a vector of states/province codes.
#' @param country.code A single country code or a vector of country codes equal in length to the vector of states/province codes.
#' @template list
#' @return Dataframe containing species list(s) for the specified states/provinces.
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @examples \dontrun{
#' BIEN_list_state("United States","Michigan")
#' state_vector<-c("Michigan","Arizona")
#' BIEN_list_state(country="United States", state= state_vector)}
#' @family list functions
#' @export
BIEN_list_state <- function(country = NULL,
                            country.code = NULL,
                            state = NULL,
                            state.code = NULL,
                            cultivated = FALSE,
                            new.world = NULL,
                            ...){
  .is_char(country)
  .is_char(country.code)
  .is_char(state)
  .is_char(state.code)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  
  if(is.null(country)& is.null(country.code)) {
    
    stop("Please supply either a country name or 2-digit ISO code")
    
    }  
  
  # set base query components
  
    sql_select <-  paste("SELECT DISTINCT country, state_province, scrubbed_species_binomial ")
    sql_from <- paste(" FROM species_by_political_division ")
  
  #if supplying country names  
  if(is.null(country.code) & is.null(state.code)){  
    if(length(country)==1){
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") 
                         AND state_province in (", paste(shQuote(state, type = "sh"),collapse = ', '), ") 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country)==length(state)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i<- paste("(country = ", paste(shQuote(country[i], type = "sh"),collapse = ', '), " AND state_province = ", paste(shQuote(state[i], type = "sh"),collapse = ', '), ")")
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  
        
      }  
      
    }#if length(country>1)
  }else{
    
    if(length(country.code)==1){
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), ")) 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country.code)==length(state.code)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country.code)){
          
          condition_i<- paste("country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code[i], type = "sh"),collapse = ', '), ")) 
                              AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code[i], type = "sh"),collapse = ', '), "))")
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  
        
      }  
      
    }#if length(country>1)

  }  

  # adjust for optional parameters
  if(!cultivated){
    
    #sql_where <- paste(sql_where, " AND (is_cultivated_observation = 0 OR is_cultivated_observation IS NULL) ")
    
  }else{
    
    sql_select  <- paste(sql_select, ",is_cultivated_in_region")

  }
  
  newworld_ <- .newworld_check(new.world)
  
  # form the final query
  query <- paste(sql_select,newworld_$select, sql_from, sql_where,newworld_$query, " ;")
  
  return(.BIEN_sql(query, ...))
  
  }



###########################

#'Extract a species list by county.
#'
#'BIEN_list_county produces a list of all species with geovalidated occurrences falling within specified county or counties.
#' @param country A single country or vector of countries
#' @param state A state or vector of states (or other primary political divisions, e.g. provinces).
#' @param county A single county (or other secondary administrative boundary)or vector of counties.
#' @param state.code A single state/province code, or a vector of states/province codes.
#' @param country.code A single country (or other primary administrative boundary) code or a vector of country codes equal in length to the vector of states/province codes.
#' @param county.code A single county (or other secondary administrative boundary) code or a vector of county codes equal in length to the vectors of states/province codes and country codes.
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @note We recommend using country, state, and county rather than codes, since county names have not been fully standardized.
#' @template list
#' @return Dataframe containing species list(s) for the specified states/provinces.
#' @note This function requires you supply either 1) a single state and country with one or more counties, or 2) vectors of equal length for each political level.
#' @examples \dontrun{
#' BIEN_list_county("United States", "Michigan", "Kent")
#' BIEN_list_county(country = "United States", state = "Michigan", county = "Kent")
#' county_vector<-c("Kent","Kalamazoo")
#' BIEN_list_county(country = "United States", state = "Michigan", county = county_vector)}
#' @family list functions
#' @export
BIEN_list_county <- function(country = NULL,
                             state = NULL,
                             county = NULL,
                             country.code = NULL,
                             state.code = NULL,
                             county.code = NULL,
                             cultivated = FALSE,
                             new.world = NULL,
                             ...){

  .is_char(country.code)
  .is_char(state.code)
  .is_char(county.code)
  .is_char(country)
  .is_char(state)
  .is_char(county)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  
  # set base query components
    sql_select <-  paste("SELECT DISTINCT country, state_province, county,  scrubbed_species_binomial ")
    sql_from <- paste(" FROM species_by_political_division ")
  
  if(is.null(country.code) & is.null(state.code) & is.null(county.code)){  
    
    #sql where
    if(length(country)==1 & length(state)==1){
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") 
                         AND state_province in (", paste(shQuote(state, type = "sh"),collapse = ', '), ") 
                         AND county in (", paste(shQuote(county, type = "sh"),collapse = ', '), ")
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country)==length(state) & length(country)==length(county)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i<- paste("(country = ", paste(shQuote(country[i], type = "sh"),collapse = ', '), " 
                              AND state_province = ", paste(shQuote(state[i], type = "sh"),collapse = ', '), "
                              AND county = ", paste(shQuote(county[i], type = "sh"),collapse = ', '), ")
                              ")
          
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country and/or state the function requires matching vectors of countries, states and counties.")  
        
      }  
      
      
      
    }#if length(country>1)
  }else{ 
    
    #sql where
    if(length(country.code)==1 & length(state.code)==1){
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), "))
                         AND county in (SELECT county_parish_ascii FROM county_parish WHERE admin2code in (", paste(shQuote(county.code, type = "sh"),collapse = ', '), "))
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country)==length(state) & length(country)==length(county)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i<- paste("(country = (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                              AND state_province = (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), ")) 
                              AND county = (SELECT county_parish_ascii FROM county_parish WHERE admin2code in (", paste(shQuote(county.code, type = "sh"),collapse = ', '), "))" )
          
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country and/or state the function requires matching vectors of countries, states and counties.")  
        
      }  
      
      
      
    }#if length(country>1)
    
    
  }
  
  
  # adjust for optional parameters
  if(!cultivated){
    
    #sql_where <- paste(sql_where, " AND (is_cultivated_observation = 0 OR is_cultivated_observation IS NULL) ")
    
  }else{
    
    sql_select  <- paste(sql_select, ",is_cultivated_in_region")
    
  }
  
  #if(!new.world){
  #  sql_select <- paste(sql_select,",is_new_world")
  #}else{
  #  sql_where <- paste(sql_where, "AND is_new_world = 1 ")
  #}
  
  newworld_ <- .newworld_check(new.world)
  
  # form the final query
  query <- paste(sql_select,newworld_$select, sql_from, sql_where,newworld_$query,  " ;")
  
  return(.BIEN_sql(query, ...))
  
  }


###########################

#'Extract a list of all species in the BIEN database.
#'
#'BIEN_list_all produces a list of all species in the BIEN database.
#' @param ... Additional arguments passed to internal functions.
#' @return Dataframe containing a list of all species in the BIEN database.
#' @examples \dontrun{
#' species_list<-BIEN_list_all()}
#' @family list functions
#' @export
BIEN_list_all<-function( ...){
  query <- paste("SELECT species FROM bien_species_all ;")
  
  return(.BIEN_sql(query, ...))
  
}
###########################

#'Extract a list of species within a given sf polygon.
#'
#'BIEN_list_sf produces a list of all species with occurrence records falling within a user-supplied sf object.
#' @param sf An object of class ff.  Note that the object must be in WGS84.
#' @template list
#' @return Dataframe containing a list of all species with occurrences in the supplied sf object.
#' @examples \dontrun{
#' library(sf)
#' 
#' BIEN_ranges_species("Carnegiea gigantea") # saves ranges to the current working directory
#' 
#' sf <- st_read(dsn = ".",
#'               layer = "Carnegiea_gigantea")
#' 
#' species_list <- BIEN_list_sf(sf = sf)
#' }
#' @family list functions
#' @importFrom sf st_geometry st_as_text st_bbox
#' @export
BIEN_list_sf <- function(sf,
                         cultivated = FALSE,
                         new.world = NULL,
                         ...){
  
  .is_log(cultivated)
  .is_log_or_null(new.world)
  
  # Convert the sf to wkt (needed for sql query)  
  wkt <- sf |>
    st_geometry() |>
    st_as_text()
  
  # Get bounding box of sf (used as a sort of index to make query a bit faster)
  sf_bbox <-
    sf |>
    st_bbox()
  
  long_min <- sf_bbox["xmin"]
  long_max <- sf_bbox["xmax"]
  lat_min <- sf_bbox["ymin"]
  lat_max <- sf_bbox["ymax"]  
  
  # adjust for optional parameters
  
  if(!cultivated){
    
    cultivated_query <- "AND (is_cultivated_observation = 0 OR is_cultivated_observation IS NULL)"
    cultivated_select <- ""
  }else{
    
    cultivated_query <- ""
    cultivated_select <- ",is_cultivated_observation,is_cultivated_in_region"
    
  }
  
  newworld_ <- .newworld_check(new.world)
  
  query <- paste("SELECT DISTINCT scrubbed_species_binomial",cultivated_select,newworld_$select ,"
                FROM  
                  (SELECT * FROM view_full_occurrence_individual WHERE higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                  AND is_geovalid = 1 AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) 
                  AND observation_type IN ('plot','specimen','literature','checklist') 
                  AND scrubbed_species_binomial IS NOT NULL 
                  AND latitude BETWEEN ",lat_min," AND ",lat_max,"AND longitude BETWEEN ",long_min," AND ",long_max,") a
               WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),a.geom)",cultivated_query,newworld_$query ," ;")
  
  # create query to retrieve
  
  df <- .BIEN_sql(query, ...)
  #df <- .BIEN_sql(query)
  
  if(length(df) == 0){
    
    message("No species found")
    return(invisible(NULL))
    
  }else{
    
    return(df)
    
  }
  
}



###########################
###########################

#'Extract occurrence data from BIEN for specified genera
#'
#'BIEN_occurrence_genus downloads occurrence records for specific genus/genera from the BIEN database.
#' @param genus A single genus, or a vector of genera. Genera should be capitalized.
#' @template occurrence
#' @return Dataframe containing occurrence records for the specified genera.
#' @examples \dontrun{
#' BIEN_occurrence_genus("Abutilon")
#' genus_vector<-c("Abutilon","Abronia")
#' BIEN_occurrence_genus(genus_vector)
#' BIEN_occurrence_genus(genus = "Abutilon",cultivated = TRUE,new.world = FALSE)}
#' @family occurrence functions
#' @export
BIEN_occurrence_genus <- function(genus,
                                  cultivated = FALSE,
                                  new.world = NULL,
                                  all.taxonomy = FALSE,
                                  native.status = FALSE,
                                  natives.only = TRUE,
                                  observation.type = FALSE,
                                  political.boundaries = FALSE,
                                  collection.info = FALSE,
                                  ...){
  .is_char(genus)
  .is_log(cultivated)
  .is_log(all.taxonomy)
  .is_log_or_null(new.world)
  .is_log(native.status)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(natives.only)
  .is_log(collection.info)
  
  cultivated_<-.cultivated_check(cultivated)  
  newworld_<-.newworld_check(new.world)
  taxonomy_<-.taxonomy_check(all.taxonomy)  
  native_<-.native_check(native.status)
  observation_<-.observation_check(observation.type)
  political_<-.political_check(political.boundaries)  
  natives_<-.natives_check(natives.only)
  collection_<-.collection_check(collection.info)
  
  # set the query
  query <-
    paste("SELECT scrubbed_genus, scrubbed_species_binomial",taxonomy_$select,native_$select,political_$select," ,latitude, longitude,date_collected,datasource,dataset,dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,observation_$select, "
          FROM view_full_occurrence_individual 
          WHERE scrubbed_genus in (", paste(shQuote(genus, type = "sh"),collapse = ', '), ")",cultivated_$query,newworld_$query,natives_$query," 
            AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 
            AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) 
            AND observation_type IN ('plot','specimen','literature','checklist') 
            AND scrubbed_species_binomial IS NOT NULL ;")
  
  return(.BIEN_sql(query, ...))
  
}
############################

#'Extract species occurrences by family.
#'
#'BIEN_occurrence_family extracts all occurrences for a given family (or families) from the BIEN database.
#' @param family A single family or a vector of families.
#' @param only.geovalid Should the returned records be limited to those with validated coordinates?  Default is TRUE
#' @template occurrence
#' @return Dataframe containing occurrence records for the specified family/families.
#' @examples \dontrun{
#' BIEN_occurrence_family("Theaceae")
#' family_vector<-c("Theaceae","Ericaceae")
#' BIEN_occurrence_family(family_vector)}
#' @family occurrence functions
#' @export
BIEN_occurrence_family <- function(family,
                                   cultivated = FALSE,
                                   new.world = NULL,
                                   observation.type = FALSE,
                                   all.taxonomy = FALSE,
                                   native.status = FALSE,
                                   natives.only = TRUE,
                                   political.boundaries = FALSE,
                                   collection.info = FALSE,
                                   only.geovalid = TRUE,
                                   ...){
  
  .is_char(family)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(observation.type)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(only.geovalid)
  
  
  
  #set conditions for query
  cultivated_<-.cultivated_check(cultivated)  
  newworld_<-.newworld_check(new.world)
  taxonomy_<-.taxonomy_check(all.taxonomy)  
  native_<-.native_check(native.status)
  observation_<-.observation_check(observation.type)
  political_<-.political_check(political.boundaries)  
  natives_<-.natives_check(natives.only)
  collection_<-.collection_check(collection.info)
  observation_<-.observation_check(observation.type)
  geovalid_<-.geovalid_check(only.geovalid)
  
  
  
  # set the query
  query <- paste("SELECT scrubbed_family",taxonomy_$select,native_$select,political_$select,",
                    scrubbed_species_binomial, latitude, longitude, date_collected,
                    datasource,dataset, dataowner, custodial_institution_codes,
                    collection_code, view_full_occurrence_individual.datasource_id",
                    collection_$select, cultivated_$select, newworld_$select,
                    observation_$select, geovalid_$select,"
                 FROM view_full_occurrence_individual 
                 WHERE scrubbed_family in (", paste(shQuote(family, type = "sh"),collapse = ', '), ")",
                 cultivated_$query,newworld_$query,natives_$query,observation_$query, geovalid_$query, " 
                    AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi')  
                    AND (georef_protocol is NULL OR georef_protocol<>'county centroid')
                    AND (is_centroid IS NULL OR is_centroid=0) 
                    AND scrubbed_species_binomial IS NOT NULL ;")
  
  return(.BIEN_sql(query, ...))
  
}


#######################

#'Extract species occurrence records by state.
#'
#'BIEN_occurrence_state extracts occurrences records for the specified state(s).
#' @param state A state or vector of states (or other primary political divisions, e.g. provinces).
#' @param country A single country or vector of countries.
#' @param state.code A single state/province code, or a vector of states/province codes.
#' @param country.code A single country (or other primary administrative boundary) code or a vector of country codes equal in length to the vector of states/province codes.
#' @template occurrence
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @note This function requires you supply either 1) a single country with one or more states, or 2) vectors of equal length for each political level.
#' @return Dataframe containing occurrence records for the specified states/provinces.
#' @examples \dontrun{
#' BIEN_occurrence_state("United States","Rhode Island")
#' state_vector<-c("Rhode Island","Maryland")
#' BIEN_occurrence_state(country="United States",state=state_vector)}
#' @family occurrence functions
#' @export
BIEN_occurrence_state <- function(country = NULL,
                                  state = NULL,
                                  country.code = NULL,
                                  state.code = NULL,
                                  cultivated = FALSE,
                                  new.world = NULL,
                                  all.taxonomy = FALSE,
                                  native.status = FALSE,
                                  natives.only = TRUE,
                                  observation.type = FALSE,
                                  political.boundaries = FALSE,
                                  collection.info = FALSE,
                                  ...){
  
  .is_char(country)
  .is_char(state)
  .is_char(country.code)
  .is_char(state.code)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(natives.only)
  .is_log(collection.info)
  
  #set conditions for query
    cultivated_ <- .cultivated_check(cultivated)  
    newworld_ <- .newworld_check(new.world)
    taxonomy_ <- .taxonomy_check(all.taxonomy)  
    native_ <- .native_check(native.status)
    observation_ <- .observation_check(observation.type)
    political_ <- .political_check(political.boundaries)  
    natives_ <- .natives_check(natives.only)
    collection_ <- .collection_check(collection.info)
  
  if(is.null(country.code) & is.null(state.code)){  
    
    ##state where
    if(length(country) == 1){
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") 
                         AND state_province in (", paste(shQuote(state, type = "sh"),collapse = ', '), ") 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country) == length(state)){
        
        sql_where <- "WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i <- paste("(country = ", paste(shQuote(country[i], type = "sh"),collapse = ', '),
                               " AND state_province = ", paste(shQuote(state[i], type = "sh"),collapse = ', '), ")")
          
          if(i != 1){condition_i <- paste("OR ", condition_i)} #stick OR onto the condition where needed
          sql_where <- paste(sql_where, condition_i)
          
        }#for i  
        
        sql_where <- paste(sql_where, ") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        
        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  
        
      }  
      
    }#if length(country>1)
  }else{
    
    ##state where
    if(length(country.code) == 1){
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), ")) 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country.code) == length(state.code)){
        
        sql_where <- "WHERE ("
        
        for(i in 1:length(country.code)){
          
          condition_i <- paste("country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code[i], type = "sh"),collapse = ', '), ")) 
                                AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code[i], type = "sh"),collapse = ', '), "))")
          if(i != 1){condition_i <- paste("OR ",condition_i)}#stick OR onto the condition where needed
          
            sql_where <- paste(sql_where, condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{

        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  

      }  
      
    }#if length(country>1)  

  }  

  # set the query
  query <- paste("SELECT scrubbed_species_binomial" ,taxonomy_$select,political_$select, ", latitude, longitude,date_collected,datasource,
                  dataset,dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",
                  collection_$select,cultivated_$select,newworld_$select,native_$select,observation_$select,"
                 FROM view_full_occurrence_individual ",
                  sql_where,cultivated_$query,newworld_$query,natives_$query," 
                  AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 
                  AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) 
                  AND observation_type IN ('plot','specimen','literature','checklist')  
                  AND scrubbed_species_binomial IS NOT NULL ;")
  
  
  return(.BIEN_sql(query, ...))
  
  }


#############################

#'Extract species occurrence records by country.
#'
#'BIEN_occurrence_country extracts occurrences records for the specified country/countries.
#' @param country A single country or a vector of country.
#' @param country.code A single country code or a vector of country codes equal in length to the vector of states/province codes.
#' @template occurrence
#' @param only.geovalid Should the returned records be limited to those with validated coordinates?  Default is TRUE
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @return Dataframe containing occurrence records for the specified country.
#' @examples \dontrun{
#' BIEN_occurrence_country("Cuba")
#' country_vector<-c("Cuba","Bahamas")
#' BIEN_occurrence_country(country_vector)}
#' @family occurrence functions
#' @export
BIEN_occurrence_country <- function(country = NULL,
                                    country.code = NULL,
                                    cultivated = FALSE,
                                    new.world = NULL,
                                    all.taxonomy = FALSE,
                                    native.status = FALSE,
                                    natives.only = TRUE,
                                    observation.type = FALSE,
                                    political.boundaries = FALSE,
                                    collection.info = FALSE,
                                    only.geovalid = TRUE,
                                    ...){

  .is_char(country)
  .is_char(country.code)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(only.geovalid)
  
  if(is.null(country)& is.null(country.code)) {
    stop("Please supply either a country or 2-digit ISO code")
    }
  
  #set conditions for query
    
    cultivated_ <- .cultivated_check(cultivated)  
    newworld_ <- .newworld_check(new.world)
    taxonomy_ <- .taxonomy_check(all.taxonomy)  
    native_ <- .native_check(native.status)
    observation_ <- .observation_check(observation.type)
    political_ <- .political_check(political.boundaries)  
    natives_ <- .natives_check(natives.only)
    collection_ <- .collection_check(collection.info)
    geovalid_<-.geovalid_check(only.geovalid)
    
  
  
  # set the query
  
  
  if(is.null(country.code)){
      query <- paste("SELECT scrubbed_species_binomial",taxonomy_$select,
                      political_$select, native_$select,", latitude,
                      longitude,date_collected, datasource, dataset ,dataowner,
                      custodial_institution_codes,collection_code,
                      view_full_occurrence_individual.datasource_id",
                      collection_$select, cultivated_$select,
                      newworld_$select, observation_$select, geovalid_$select, "
                    FROM view_full_occurrence_individual
                    WHERE country in
                    (", paste(shQuote(country, type = "sh"),collapse = ', '), ")",
                    cultivated_$query,newworld_$query,natives_$query,
                    geovalid_$query," 
                    AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi')
                    AND (georef_protocol is NULL OR georef_protocol<>'county centroid')
                    AND (is_centroid IS NULL OR is_centroid=0)",
                    observation_$query,";")
  
  }else{
    query <- paste("SELECT scrubbed_species_binomial",taxonomy_$select,political_$select,native_$select,", latitude, longitude, 
                      date_collected,datasource,dataset,dataowner,custodial_institution_codes,collection_code,
                      view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,observation_$select,"
                   FROM view_full_occurrence_individual 
                   WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                      ",cultivated_$query,newworld_$query,natives_$query," 
                      AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 
                      AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0)",
                      observation_$query, 
                      "AND scrubbed_species_binomial IS NOT NULL ;")
    
    
  }
  
  
  return(.BIEN_sql(query, ...))
  
}

##############################

#'Extract species occurrence records by county.
#'
#'BIEN_occurrence_county extracts occurrences records for the specified county or counties.
#' @param country A single country or vector of countries.
#' @param state A state or vector of states (or other primary political divisions, e.g. provinces).
#' @param county A single county or a vector of counties (or other secondary political division, e.g. parish).
#' @param state.code A single state/province code, or a vector of states/province codes.
#' @param country.code A single country (or other primary administrative boundary) code or a vector of country codes equal in length to the vector of states/province codes.
#' @param county.code A single county (or other secondary administrative boundary) code or a vector of county codes equal in length to the vectors of states/province codes and country codes.
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @template occurrence
#' @note This function requires you supply either 1) a single country with one or more states, or 2) vectors of equal length for each political level.
#' @return Dataframe containing occurrence records for the specified states/provinces.
#' @examples \dontrun{
#' BIEN_occurrence_county("United States","Arizona","Pima")
#' country_vector<-c("United States","United States")
#' state_vector<-c("Arizona","Michigan")
#' county_vector<-c("Pima","Kent")
#' BIEN_occurrence_county(country=country_vector, state = state_vector, county = county_vector)}
#' @family occurrence functions
#' @export
BIEN_occurrence_county <- function(country = NULL,
                                   state = NULL,
                                   county = NULL,
                                   country.code = NULL,
                                   state.code = NULL,
                                   county.code = NULL,
                                   cultivated = FALSE,
                                   new.world = NULL,
                                   all.taxonomy = FALSE,
                                   native.status = FALSE,
                                   natives.only = TRUE,
                                   observation.type = FALSE,
                                   political.boundaries = FALSE,
                                   collection.info = FALSE,
                                   ...){

  .is_char(country)
  .is_char(state)
  .is_char(county)
  .is_char(country.code)
  .is_char(state.code)
  .is_char(county.code)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(observation.type)
  .is_log(political.boundaries)
  .is_log(collection.info)
  
  #set conditions for query
  cultivated_<-.cultivated_check(cultivated)  
  newworld_<-.newworld_check(new.world)
  taxonomy_<-.taxonomy_check(all.taxonomy)  
  native_<-.native_check(native.status)
  observation_<-.observation_check(observation.type)
  political_<-.political_check(political.boundaries)  
  natives_<-.natives_check(natives.only)
  collection_<-.collection_check(collection.info)
  
  if(is.null(country.code) & is.null(state.code) & is.null(county.code)){    
    
    #sql where
    if(length(country) ==1 & length(state) == 1){
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") 
                         AND state_province in (", paste(shQuote(state, type = "sh"),collapse = ', '), ") 
                         AND county in (", paste(shQuote(county, type = "sh"),collapse = ', '), ")
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country)==length(state) & length(country)==length(county)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i<- paste("(country = ", paste(shQuote(country[i], type = "sh"),collapse = ', '), " 
                              AND state_province = ", paste(shQuote(state[i], type = "sh"),collapse = ', '), "
                              AND county = ", paste(shQuote(county[i], type = "sh"),collapse = ', '), ")
                              ")
          
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country and/or state the function requires matching vectors of countries, states and counties.")  
        
      }  

    }#if length(country>1)
    
  }else{
    
    #sql where
    if(length(country.code) == 1 & length(state.code) == 1){
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), "))
                         AND county in (SELECT county_parish_ascii FROM county_parish WHERE admin2code in (", paste(shQuote(county.code, type = "sh"),collapse = ', '), "))
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country) == length(state) & length(country) == length(county)){
        
        sql_where <- "WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i <- paste("(country = (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                              AND state_province = (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), ")) 
                              AND county = (SELECT county_parish_ascii FROM county_parish WHERE admin2code in (", paste(shQuote(county.code, type = "sh"),collapse = ', '), "))" )
          
          if(i != 1){condition_i <- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country and/or state the function requires matching vectors of countries, states and counties.")  
        
      }  

    }#if length(country>1)

  }#if codes are not null  

  # set the query
  query <- paste("SELECT scrubbed_species_binomial" ,taxonomy_$select,political_$select , ",latitude, longitude,date_collected,datasource,
                    dataset,dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",
                    collection_$select,cultivated_$select,newworld_$select,native_$select,observation_$select,"
                  FROM view_full_occurrence_individual ",
                    sql_where,cultivated_$query,newworld_$query,natives_$query," 
                    AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 
                    AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) 
                    AND observation_type IN ('plot','specimen','literature','checklist')  
                    AND scrubbed_species_binomial IS NOT NULL ;")

  return(.BIEN_sql(query, ...))
  
  }


############################
#'Extract species occurrence records by a latitude/longitude bounding box.
#'
#'BIEN_occurrence_box extracts occurrences records falling within the specific area.
#' @param min.lat Minimum latitude
#' @param max.lat Maximum latitude
#' @param min.long Minimum longitude
#' @param max.long Maximum longitude
#' @param species Optional.  A single species or a vector of species.
#' @param genus Optional. A single genus or a vector of genera.
#' @template occurrence
#' @return Dataframe containing occurrence records for the specified area.
#' @note Specifying species and/or genera will limit records returned to that set of taxa.
#' @examples \dontrun{
#' output_test<-
#' BIEN_occurrence_box(min.lat = 32,max.lat = 33,min.long = -114,max.long = -113,
#' cultivated = TRUE, new.world = FALSE)}
#' @family occurrence functions
#' @export
BIEN_occurrence_box<-function(min.lat,
                              max.lat,
                              min.long,
                              max.long,
                              species = NULL,
                              genus = NULL,
                              cultivated = FALSE,
                              new.world = NULL,
                              all.taxonomy = FALSE,
                              native.status = FALSE,
                              natives.only = TRUE,
                              observation.type = FALSE,
                              political.boundaries = TRUE,
                              collection.info = FALSE,
                              ...){

  .is_num(min.lat)
  .is_num(max.lat)
  .is_num(min.long)
  .is_num(max.long)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(observation.type)
  .is_log(collection.info)
  .is_char(species)
  .is_char(genus)
  
  #set conditions for query
  cultivated_ <- .cultivated_check(cultivated)  
  newworld_ <- .newworld_check(new.world)
  taxonomy_ <- .taxonomy_check(all.taxonomy)  
  native_ <- .native_check(native.status)
  observation_ <- .observation_check(observation.type)
  political_ <- .political_check(political.boundaries)  
  natives_ <- .natives_check(natives.only)
  collection_ <- .collection_check(collection.info)
  species_ <- .species_check(species)
  genus_ <- .genus_check(genus)  
  
  
  # set the query
  query <- paste("SELECT scrubbed_species_binomial", taxonomy_$select,political_$select,native_$select,",latitude, longitude, 
                    date_collected,datasource,dataset,dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",
                    collection_$select,cultivated_$select,newworld_$select,observation_$select,"
                 FROM view_full_occurrence_individual 
                 WHERE latitude between " , paste(shQuote(min.lat, type = "sh"),collapse = ', '), "AND " , paste(shQuote(max.lat, type = "sh"),collapse = ', '),"
                  AND longitude between ", paste(shQuote(min.long, type = "sh"),collapse = ', '), "AND " , paste(shQuote(max.long, type = "sh"),collapse = ', '), 
                  cultivated_$query,newworld_$query,natives_$query, species_$query, genus_$query  ,  "
                  AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 AND (georef_protocol is NULL OR georef_protocol<>'county centroid') 
                  AND (is_centroid IS NULL OR is_centroid=0) AND observation_type IN ('plot','specimen','literature','checklist') 
                  AND scrubbed_species_binomial IS NOT NULL ;")
  
  return(.BIEN_sql(query, ...))
  
}

#####

#'Download range maps for given species.
#'
#'BIEN_ranges_species extracts range maps for the specified species.
#' @param species A single species or a vector of species.
#' @template ranges
#' @return Range maps for specified species.
#' @examples \dontrun{
#' library(sf)
#' library(maps) #a convenient source of maps
#' species_vector <- c("Abies_lasiocarpa","Abies_amabilis")
#' BIEN_ranges_species(species_vector)
#' BIEN_ranges_species(species_vector, match_names_only = TRUE)
#' temp_dir <- file.path(tempdir(), "BIEN_temp")#Set a working directory
#' BIEN_ranges_species(species = species_vector,
#'                     directory = temp_dir)#saves ranges to a temporary directory
#' BIEN_ranges_species("Abies_lasiocarpa")
#' BIEN_ranges_species("Abies_lasiocarpa",
#'                     directory = temp_dir)
#' 
#' #Reading files
#' 
#' Abies_poly <- st_read(dsn = temp_dir,
#'                       layer = "Abies_lasiocarpa")
#' 
#' #Plotting files
#' plot(Abies_poly[1])#plots the range, but doesn't mean much without any reference
#' map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
#' 
#' plot(Abies_poly[1],
#'      col = "forest green",
#'      add = TRUE) #adds the range of Abies lasiocarpa to the map
#' 
#' # Getting data from the files (currently only species names and a BIEN ID field)
#' Abies_poly$species#gives the species name associated with "Abies_poly"}#'
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @export
BIEN_ranges_species <- function(species,
                                directory = NULL,
                                matched = TRUE,
                                match_names_only = FALSE,
                                include.gid = FALSE,
                                ...){
  
  .is_char(species)
  .is_log(matched)
  .is_log(match_names_only)
  
  #make sure there are no spaces in the species names
  species <- gsub(" ","_",species)
  
  if(match_names_only == FALSE){
    
    #record original working directory,change to specified directory if given
    if(is.null(directory)){
      directory <- getwd()
    }
    
    
    # set the query
    query <- paste("SELECT ST_AsText(geom),species,gid FROM ranges WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") ;")
    
    # create query to retrieve
    df <- .BIEN_sql(query, ...)
    #df <- .BIEN_sql(query)
    
    if(length(df) == 0){
      
      message("No species matched")
      
    }else{
      
      for(l in 1:length(df$species)){
        
        sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                                 wkt = "st_astext",
                                 crs = "epsg:4326")
        
        #Make sure that the directory doesn't have a "/" at the end-this confuses rgdal.  Probably a more eloquent way to do this with regex...
        # if(unlist(strsplit(directory,""))[length(unlist(strsplit(directory,"")))]=="/"){
        #   directory<-paste(unlist(strsplit(directory,""))[-length(unlist(strsplit(directory,"")))],collapse = "")
        # }
        
        if(include.gid == TRUE){
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l],"_",df$gid[l],sep=""),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
          
        }else{
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l]),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
        }
        
        #save output
        
      }#for species in df loop
    }#else
    
    #list matched species
    if(matched == TRUE){
      found <- as.data.frame(cbind(species,matrix(nrow=length(species),ncol=1,data="No")))
      colnames(found) <- c("Species","Range_map_downloaded?")
      found$`Range_map_downloaded?` <- as.character(found$`Range_map_downloaded?`)
      found$`Range_map_downloaded?`[which(species%in%df$species)] <- "Yes"
      return(found)
    }#matched = true
  }#match names only if statement
  
  if(match_names_only == TRUE){
    
    rangeQuery <- paste("SELECT species FROM ranges WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") ;")
    
    # create query to retrieve
    df <- .BIEN_sql(rangeQuery, ...)
    #df <- .BIEN_sql(rangeQuery)
    
    if(length(df) == 0){
      
      message("No species matched")
      
    }else{
      found <- as.data.frame(cbind(species,matrix(nrow=length(species),ncol=1,data="No")))
      colnames(found) <- c("Species","Range_map_available?")
      found$`Range_map_available?` <- as.character(found$`Range_map_available?`)
      found$`Range_map_available?`[which(species%in%df$species)] <- "Yes"
      return(found)
      
    }
    
  } #matched_names_only == TRUE
}

####################################

#'Extract range data for large numbers of species
#'
#'BIEN_ranges_species_bulk downloads ranges for a large number of species using parallel processing.
#' @param species A vector of species or NULL (the default).  If NULL, all available ranges will be used.
#' @param directory The directory where range shapefiles will be stored.  If NULL, a temporary directory will be used.
#' @param batch_size The number of ranges to download at once.
#' @param return_directory Should the directory be returned? Default is TRUE
#' @param use_parallel Logical.  Should batches be downloaded in parallel?  If set to TRUE, AND if parallel and foreach are available, parallel processing of downloads will use n-1 clusters.
#' @return Optionally, the directory to which the files were saved.
#' @note This function may take a long time (hours) to run depending on the number of cores, download speed, etc.
#' @examples \dontrun{
#' #To download all BIEN ranges maps:
#' BIEN_ranges_species_bulk()
#' }
#' @family range functions
#' @export
#' @import foreach
#' @import doParallel
BIEN_ranges_species_bulk <- function(species = NULL,
                                     directory = NULL,
                                     batch_size = 1000,
                                     return_directory = TRUE,
                                     use_parallel = FALSE){
  
  #Set species list and directory if NULL
  
  if(is.null(species)){ species <- BIEN_ranges_list()$species }  
  
  if(is.null(directory)){directory <- file.path(tempdir(), "BIEN_temp")
  print(paste("Files will be saved to ",directory))}  
  
  if(!file.exists(directory)){
    dir.create(directory)
  }
  
  
  
  if(nzchar(system.file(package = "doParallel"))  & nzchar(system.file(package = "foreach")) & use_parallel){
    
    
    #Download range maps
    cl <- parallel::makePSOCKcluster(parallel::detectCores())
    
    doParallel::registerDoParallel(cl = cl,
                                   cores = parallel::detectCores() - 1)
    
    foreach::foreach(i = 1:ceiling(length(species)/batch_size  )) %dopar%
      
      BIEN_ranges_species(species = species[(((i-1)*batch_size)+1):(i*batch_size)],
                          directory = file.path(directory,i),
                          matched = FALSE)
    
    parallel::stopCluster(cl)
    rm(cl)
    
  }else{
    
    
    for(i in 1:ceiling(length(species)/batch_size  )){
      
      BIEN_ranges_species(species = species[(((i-1)*batch_size)+1):(i*batch_size)],
                          directory = file.path(directory,i),
                          matched = FALSE)
      
    }
    
  }
  
  
  
  
  if(return_directory){return(directory)}
  
}#end fx



####################################
#'Download range maps for given genus.
#'
#'BIEN_ranges_genus extracts range maps for the specified genera.
#' @param genus A single genus or a vector of genera.
#' @template ranges
#' @return Range maps for all available species within the specified genera.
#' @examples \dontrun{
#' library(maps)
#' library(sf)
#' 
#' genus_vector <- c("Abies","Acer")
#' 
#' temp_dir <- file.path(tempdir(), "BIEN_temp")#Set a working directory
#' 
#' BIEN_ranges_genus(genus_vector)
#' 
#' BIEN_ranges_genus(genus = genus_vector,
#'                   match_names_only = TRUE)
#' 
#' BIEN_ranges_genus(genus = genus_vector,
#'                   directory = temp_dir) #saves ranges to a specified working directory
#' 
#' BIEN_ranges_genus("Abies")
#' 
#' BIEN_ranges_genus(genus = "Abies",
#'                   directory = temp_dir)
#' 
#' #Reading files
#' 
#' Abies_poly <- read_sf(dsn = temp_dir,layer = "Abies_lasiocarpa")
#' 
#' #Plotting files
#' 
#' plot(Abies_poly[1]) #plots the range, but doesn't mean much without any reference
#' 
#' map('world', fill = TRUE, col = "grey") #plots a world map (WGS84 projection), in grey
#' 
#' plot(Abies_poly[1],
#'      col="forest green",
#'      add = TRUE) #adds the range of Abies lasiocarpa to the map
#' 
#' # Getting data from the files (currently only species names)
#' 
#' Abies_poly$species#gives the species name associated with "Abies_poly"
#' }
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @export
BIEN_ranges_genus <- function(genus,
                              directory = NULL,
                              matched = TRUE,
                              match_names_only = FALSE,
                              include.gid = FALSE,
                              ...){
  
  .is_char(genus)
  .is_log(matched)
  .is_log(match_names_only)
  .is_log(include.gid)
  
  #modify the genus list to make searching easier
    genus <- paste("(",genus,"_",")",sep = "")
  
  if(match_names_only == FALSE){
    #record original working directory,change to specified directory if given
    if(is.null(directory)){
      directory <- getwd()
    }
    
    
    
    # set the query
    query <- paste("SELECT ST_AsText(geom),species,gid FROM ranges WHERE species ~ '",paste(genus,collapse="|"),"' ;",sep="")
    
    # create query to retrieve
    df <- .BIEN_sql(query, ...)
    #df <- .BIEN_sql(query)
    
    if(length(df) == 0){
      message("No species matched")
    }else{
      
      for(l in 1:length(df$species)){
        
        sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                             wkt = "st_astext",
                             crs = "epsg:4326")
        
        #Make sure that the directory doesn't have a "/" at the end-this confuses rgdal.  Probably a more eloquent way to do this with regex...
        # if(unlist(strsplit(directory,""))[length(unlist(strsplit(directory,"")))]=="/"){
        #   directory<-paste(unlist(strsplit(directory,""))[-length(unlist(strsplit(directory,"")))],collapse = "")
        # }
        
        if(include.gid == TRUE){
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l],"_",df$gid[l],sep=""),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
          
        }else{
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l]),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
        }
        
        #save output
        
      }#for species in df loop
    }#else
    
    #list matched species
    if(matched == TRUE){
      
      found <- as.data.frame(df$species)
      return(found)
      
    }#matched = true
    
  }#match names only if statement
  
  if(match_names_only == TRUE){
    
    query <- paste("SELECT species FROM ranges WHERE species ~ '",paste(genus,collapse="|"),"' ;",sep="")
    
    # create query to retrieve
    df <- .BIEN_sql(query, ...)
    
    if(length(df) == 0){
      
      message("No species matched")
      
    }else{
      
      found<-as.data.frame(df$species)
      return(found)
      
    }
    
  } #matched_names_only == TRUE
  
}

#######################################
#'Download range maps that intersect a specified bounding box.
#'
#'BIEN_ranges_box extracts range maps for a specified bounding box.
#' @param min.lat Minimum latitude of the ranges included.
#' @param max.lat Maximum latitude of the ranges included.
#' @param min.long Minimum longitude of the ranges included.
#' @param max.long Maximum longitude of the ranges included.
#' @param crop.ranges Should the ranges be cropped to the focal area? Default is FALSE.
#' @template ranges_spatial
#' @return Range maps for all available species within the specified bounding box.
#' @examples \dontrun{
#' temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a working directory
#' BIEN_ranges_box(42,43,-85,-84,species.names.only = TRUE)
#' BIEN_ranges_box(42,43,-85,-84,directory = temp_dir)}
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @export
BIEN_ranges_box <- function(min.lat,
                            max.lat,
                            min.long,
                            max.long,
                            directory = NULL,
                            species.names.only = FALSE,
                            return.species.list = TRUE ,
                            crop.ranges = FALSE,
                            include.gid = FALSE,
                            ...){
  
  .is_num(min.lat)
  .is_num(max.lat)
  .is_num(min.long)
  .is_num(max.long)
  .is_log(include.gid)
  .is_log(return.species.list)
  .is_log(species.names.only)
  
  if(species.names.only == FALSE){
    
    #record original working directory,change to specified directory if given
    if(is.null(directory)){
      directory <- getwd()
    }    
    
    # set the query
    
    if(crop.ranges){
      
      query <- paste("SELECT ST_AsText(ST_intersection(geom,ST_MakeEnvelope(",min.long, ",",min.lat,",",max.long,",",max.lat,",4326))),species,gid FROM ranges WHERE st_intersects(ST_MakeEnvelope(",min.long, ",",min.lat,",",max.long,",",max.lat,",4326),geom)") 
      
    }else{
      
      query <- paste("SELECT ST_AsText(geom),species,gid FROM ranges WHERE st_intersects(ST_MakeEnvelope(",min.long, ",",min.lat,",",max.long,",",max.lat,",4326),geom)")
      
    }
    
    # create query to retrieve
    
    df <- .BIEN_sql(query, ...)
    #df <- .BIEN_sql(query)
    
    if(nrow(df) == 0){
      
      message("No species matched")
      
    }else{
      
      for(l in 1:length(df$species)){
        
        sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                             wkt = "st_astext",
                             crs = "epsg:4326")
        
        if(include.gid == TRUE){
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l],"_",df$gid[l],sep=""),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
          
        }else{
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l]),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
        }
        
        #save output
        
      }#for species in df loop
      
      if(return.species.list){
        return(df$species)  
      }#if return.species.list
      
    }#else
    
  }#species names only if statement
  
  if(species.names.only == TRUE){
    
    # create query to retrieve
    query<-paste("SELECT species FROM ranges WHERE st_intersects(ST_MakeEnvelope(",min.long, ",",min.lat,",",max.long,",",max.lat,",4326),geom)")  
    
    df <- .BIEN_sql(query, ...)
    
    if(length(df) == 0){
      
      message("No species found")
      
    }else{
      
      return(df)
      
    }
    
  } #species.names.only == TRUE
}


#######################################
#'Download range maps that intersect the range of a given species.
#'
#'BIEN_ranges_intersect_species extracts range maps for a specified bounding box.
#' @param species Focal species (or a vector of species) for which to extract intersecting ranges.
#' @param include.focal Should a range for the focal species be downloaded? Default is TRUE.
#' @template ranges_spatial
#' @return Range maps for all available species that intersect the range of the focal species.
#' @examples \dontrun{
#' temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a working directory
#' BIEN_ranges_intersect_species(species = "Carnegiea_gigantea",
#' directory = temp_dir,include.focal = TRUE)
#' species_vector<-c("Carnegiea_gigantea","Echinocereus coccineus")
#' BIEN_ranges_intersect_species(species = species_vector,species.names.only = TRUE)
#' }
#' @family range functions
#' @importFrom sf st_as_sf st_write
#' @export
BIEN_ranges_intersect_species <- function(species,
                                          directory = NULL,
                                          species.names.only = FALSE,
                                          include.focal = TRUE,
                                          return.species.list = TRUE,
                                          include.gid = FALSE,
                                          ...){
  
  .is_char(species)
  .is_log(species.names.only)
  .is_log(include.focal)
  .is_log(include.gid)
  
  #make sure there are no spaces in the species names
  species <- gsub(" ","_",species)
  
  #set query chunk to include focal species
  
  if(include.focal){
    focal.query <- ""  
  }else{
    focal.query <- "a.species != b.species AND" 
  }
  
  if(species.names.only == FALSE){
    
    #set directory for saving
    
    if(is.null(directory)){
      directory <- getwd()
    }  
    
    # set the query
    query <- paste("SELECT b.species AS focal_species, a.species 
                   AS intersecting_species,a.species,a.gid, ST_AsText(a.geom) 
                   AS geom 
                   FROM ranges AS a, (SELECT species, geom 
                                      FROM ranges 
                                      WHERE species in (",paste(shQuote(species, type = "sh"),collapse = ', '),")) b WHERE", focal.query," ST_Intersects(a.geom, b.geom) ;")  
    
    # create query to retrieve
    df <- .BIEN_sql(query, ...)
    #df <- .BIEN_sql(query)
    #df <- .BIEN_sql(query,limit = limit)
    
    if(length(df) == 0){
      
      message("No species matched")
      
    }else{
      
      for(l in 1:nrow(df)){
        
        sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                             wkt = "geom",
                             crs = "epsg:4326")
        
        if(include.gid == TRUE){
          
          suppressWarnings(
            st_write(obj = sp_range,
                     dsn = directory,
                     layer = paste(df$species[l],"_",df$gid[l],sep=""),
                     driver = "ESRI Shapefile",
                     append = FALSE,
                     quiet=TRUE))
          
          
        }else{
          
          suppressWarnings(
            st_write(obj = sp_range,
                     dsn = directory,
                     layer = paste(df$species[l]),
                     driver = "ESRI Shapefile",
                     append = FALSE,
                     quiet=TRUE))
          
        }
        
        #save output
        
      }#for species in df loop
      
      if(return.species.list){
        
        return(df$species)
        
      }#if return.species.list
      
    }#else
    
  }#species names only if statement
  
  if(species.names.only == TRUE){
    
    query <- paste("SELECT b.species AS focal_species, a.species AS intersecting_species FROM ranges AS a, (SELECT species, geom FROM ranges WHERE species in (",paste(shQuote(species, type = "sh"),collapse = ', '),")) b WHERE", focal.query," ST_Intersects(a.geom, b.geom) ;")  
    
    
    # create query to retrieve
    
    df <- .BIEN_sql(query, ...)
    
    if(length(df) == 0){
      
      message("No species found")
      
    }else{
      return(df)
      
    }
    
  } #species.names.only == TRUE
}

#######################################
#'Download range maps that intersect a user-supplied sf object.
#'
#'BIEN_ranges_sf extracts range maps that intersect a specified simple features (sf) object.
#' @param sf An object of class sf.
#' @param crop.ranges Should the ranges be cropped to the focal area? Default is FALSE.
#' @template ranges_spatial
#' @return All range maps that intersect the user-supplied sf object.
#' @examples \dontrun{
#' 
#' # Here we use a range map as our example polygon
#' 
#' BIEN_ranges_species("Carnegiea gigantea") #saves ranges to the current working directory
#' 
#' # Read in the polygon  with sf   
#' sf <- sf::st_read(dsn = ".",
#'                   layer = "Carnegiea_gigantea")
#' 
#' BIEN_ranges_sf(sf = sf,
#'                limit = 10) 
#'                # We use the limit argument to return only 10 range maps.
#'                # Omit the limit argument to get all ranges
#' 
#' #Note that this will save many shapefiles to the working directory.
#' }
#' @family range functions
#' @importFrom sf st_geometry st_as_text st_as_sf st_write
#' @export
BIEN_ranges_sf <- function(sf,
                           directory = NULL,
                           species.names.only = FALSE,
                           return.species.list = TRUE,
                           crop.ranges = FALSE,
                           include.gid = FALSE,
                           ...){
  
  .is_log(return.species.list)
  .is_log(species.names.only)
  .is_log(crop.ranges)
  .is_log(include.gid)
  
  wkt <- sf |>
    st_geometry() |>
    st_as_text()
  
  if(species.names.only == FALSE){
    
    #set directory for saving
    if(is.null(directory)){
      directory <- getwd()
    }  
    
    # set the query
    if(crop.ranges){
      
      query <- paste("SELECT ST_AsText(ST_intersection(geom,ST_GeographyFromText('SRID=4326;",paste(wkt),"'))),species,gid FROM ranges WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom) ;") 
      
    }else{
      
      query <- paste("SELECT ST_AsText(geom),species,gid FROM ranges WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom) ;")  
      
    }
    
    # create query to retrieve
    
    df <- .BIEN_sql(query, ...)
    
    
    if(length(df) == 0){
      
      message("No species matched")
      
    }else{
      
      for(l in 1:length(df$species)){
        
        sp_range <- st_as_sf(x = df[l,, drop = FALSE],
                             wkt = "st_astext",
                             crs = "epsg:4326")
        
        if(include.gid == TRUE){
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l],"_",df$gid[l],sep=""),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
          
        }else{
          
          st_write(obj = sp_range,
                   dsn = directory,
                   layer = paste(df$species[l]),
                   driver = "ESRI Shapefile",
                   append = FALSE,
                   quiet=TRUE)
          
        }
        
        #save output
        
      }#for species in df loop
    }#else
    
  }#species names only if statement
  
  if(species.names.only == TRUE){
    
    query <- paste("SELECT species FROM ranges WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom) ;")  
    
    # create query to retrieve
    df <- .BIEN_sql(query, ...)
    
    if(length(df) == 0){
      message("No species found")
    }else{
      return(df)
      
    }
    
  } #species.names.only == TRUE
}

#######################################
#'Load range maps for specified species.
#'
#'BIEN_ranges_load_species returns spatial data for the specified species.
#' @param species A single species or a vector of species.
#' @param ... Additional arguments passed to internal functions.
#' @return A sf containing range maps for the specified species.
#' @examples \dontrun{
#' library(maps)
#' species_vector<-c("Abies_lasiocarpa","Abies_amabilis")
#' abies_maps <- BIEN_ranges_load_species(species = species_vector)
#' xanthium_strumarium <- BIEN_ranges_load_species(species = "Xanthium strumarium")
#' 
#' #Plotting files
#' plot(abies_maps) # plots the sf, but doesn't mean much without any reference
#' map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
#' plot(xanthium_strumarium,col="forest green",add = TRUE) #adds the range of X. strumarium
#' plot(abies_maps[1,], add = TRUE, col ="light green")
#' }
#' @family range functions
#' @importFrom sf st_as_sf
#' @export
BIEN_ranges_load_species <- function(species,
                                     ...){
  
  .is_char(species)
  
  #make sure there are no spaces in the species names
  
  species <- gsub(" ","_",species)
  
  # set the query
  
  query <- paste("SELECT ST_AsText(geom) as geometry,species,gid FROM ranges WHERE species in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") ;")
  
  # create query to retrieve
  df <- .BIEN_sql(query, ...)
  #df <- .BIEN_sql(query)
  
  if(length(df) == 0){
    
    message("No species matched")
    return(invisible(NULL))
    
  }else{
    
    poly <- st_as_sf(x = df,
                     wkt = "geometry",
                     crs = "epsg:4326")
    
    return(poly) 
    
  }#else
  
  
}


###############################
#'List available range maps
#'
#'BIEN_ranges_list a data.frame containing listing all range maps currently available.
#' @param ... Additional arguments passed to internal functions.
#' @return A data.frame containing the available species and their associated GIDs.
#' @examples \dontrun{
#' available_maps<-BIEN_ranges_list()}
#' @family range functions
#' @family metadata functions
#' @export
BIEN_ranges_list <- function( ...){
  
  # set the query
  query <- paste("SELECT species,gid FROM ranges ;")
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
}

########################################

#'Extract range data and convert to smaller "skinny" format
#'
#'BIEN_ranges_shapefile_to_skinny converts ranges to a "skinny" format to save space.
#' @param directory The directory where range shapefiles will be stored.  If NULL, a temporary directory will be used.
#' @param raster A raster (which must have a CRS specified) to be used for rasterizing the ranges.
#' @param skinny_ranges_file A filename that will be used to write the skinny ranges will be written to (RDS format).  If NULL, this will not be written.
#' @return Matrix containing 2 columns: 1) Species name; and 2) the raster cell number it occurs within.
#' @examples \dontrun{
#' BIEN_ranges_shapefile_to_skinny(directory = BIEN_ranges_species_bulk(species = c("Acer rubrum")),
#' raster = terra::rast(crs = "+proj=laea +lat_0=15 +lon_0=-80 +x_0=0 +y_0=0 +datum=WGS84 
#' +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0",
#'            extent = terra::ext(c(-5261554,5038446,-7434988,7165012 )),
#'            resolution =c(100000,100000)
#'            )
#' )
#' }
#' @family range functions
#' @importFrom sf read_sf st_transform st_crs
#' @importFrom terra rasterize
#' @importFrom terra rast
#' @importFrom terra values
#' @export
BIEN_ranges_shapefile_to_skinny <- function(directory,
                                            raster,
                                            skinny_ranges_file = NULL){
  
  
  range_maps <- list.files(path = directory,
                           pattern = ".shp",
                           full.names = TRUE,
                           recursive = TRUE)
  
  skinny_occurrences <- NULL
  
  raster <- rast(raster)
  
  for(i in range_maps){
    
    #print(i)
    raster_i <- read_sf(i) |>
      st_transform(crs = st_crs(raster)) |>
      rasterize(y = raster)
    
    if(length(which(values(raster_i) > 0)) > 0){
      skinny_occurrences <- rbind(skinny_occurrences,
                                  cbind(read_sf(i)$species,
                                        which(values(raster_i) > 0)))
    }#end if statement
  }#end i loop
  
  
  
  #Save skinny occurrences if filename specified
  
  if(!is.null(skinny_ranges_file)){
    saveRDS(object = skinny_occurrences,
            file = skinny_ranges_file)  
  }
  
  #return skinny occurrences
  return(skinny_occurrences)
  
}#end fx

########################################

#'Build a richness raster from a skinny range file
#'
#'BIEN_ranges_skinny_ranges_to_richness_raster takes in "skinny" range data and converts it to a richness raster.
#' @param skinny_ranges A matrix output by the function "BIEN_ranges_skinny" or equivalent methods.
#' @param raster The raster that was used in building the skinny_ranges matrix.
#' @return Raster
#' @examples \dontrun{
#' 
#' 
#' template_raster <- terra::rast(
#'   crs = "+proj=laea +lat_0=15 +lon_0=-80 +x_0=0 +y_0=0 +datum=WGS84 
#'   +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0",
#'   ext = ext(c(-5261554, 5038446, -7434988, 7165012 )),
#'   resolution =  c(100000, 100000))
#' 
#' #Download ranges and convert to a "skinny" format
#' skinny_ranges <- BIEN_ranges_shapefile_to_skinny(
#' directory = BIEN_ranges_species_bulk(species = c("Acer rubrum"), 
#' raster = template_raster)
#' 
#' #Convert from skinny format to richness raster                                                 
#' richness_raster<- BIEN_ranges_skinny_ranges_to_richness_raster(
#'  skinny_ranges = skinny_ranges,raster = template_raster)
#'  
#'  plot(richness_raster)
#' }
#' @family range functions
#' @export
#' @importFrom terra values
BIEN_ranges_skinny_ranges_to_richness_raster <- function(skinny_ranges,
                                                         raster){
  
  #Create empty output raster
    output_raster <- raster
    terra::values(output_raster) <- NA #note that if this "terra::" is omitted, cran checks raise a note
  
  #iterate through all cells with at least one occurrence, record 
  
  output_raster[as.numeric(unique(skinny_ranges[,2]))] <-
    sapply(X = unique(skinny_ranges[,2]),
           FUN = function(x){ length(unique(skinny_ranges[which(skinny_ranges[,2] == x), 1]))} )
  
  #return output raster  
  
  return(output_raster)
  
  
}


########################################
########################################

# Traits section ----------------------------------------------------------


#'Download trait data for given species.
#'
#'BIEN_trait_species extracts trait data for the species specified.
#' @param species A single species or a vector of species.
#' @template trait
#' @return A dataframe of all available trait data for the given species.
#' @examples \dontrun{
#' BIEN_trait_species("Poa annua")
#' species_vector<-c("Poa annua","Juncus trifidus")
#' BIEN_trait_species(species_vector)}
#' @family trait functions
#' @export
BIEN_trait_species <- function(species,
                               all.taxonomy = FALSE,
                               political.boundaries = FALSE,
                               source.citation = FALSE,
                               ...){

  .is_char(species)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  
  # set the query
  taxonomy_ <- .taxonomy_check_traits(all.taxonomy)  
  political_ <- .political_check_traits(political.boundaries)  
  source_ <- .source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_species_binomial, trait_name, trait_value, unit,
                 method, latitude, longitude, elevation_m, url_source",
                 source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select,", access, id 
                 FROM agg_traits WHERE scrubbed_species_binomial in (",
                 paste(shQuote(species, type = "sh"),collapse = ', '), ") ;")
  
  return(.BIEN_sql(query, ...))
  
}

############################

#'Calculates species mean values for a given trait, using Genus or Family level data where Species level data is lacking.
#'
#'BIEN_trait_mean Estimates species mean values for a given trait, using Genus or Family level data where Species level data is absent.
#' @param species A single species or a vector of species.
#' @param trait A single trait.
#' @param ... Additional arguments passed to internal functions.
#' @note Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
#' @return A dataframe of estimated trait means and associated metadata for the given species.
#' @examples \dontrun{
#' BIEN_trait_mean(species=c("Poa annua","Juncus trifidus"),trait="leaf dry mass per leaf fresh mass") }
#' @family trait functions
#' @export
BIEN_trait_mean <- function(species,
                            trait,
                            ...){
  
  
  #first, get taxonomic info for the species
  .is_char(trait)
  .is_char(species)
  
  #make sure there is only one trait
  if( length(trait) > 1){stop("Multiple traits submitted. This function only handles one trait at a time.")}
  
  #make sure trait exists
  traits_available <- BIEN_trait_list(...)
  if(!trait %in% traits_available$trait_name){stop("Trait not found.")}
  
  
  # create query to retreive taxonomic info
  genera <- unlist(lapply(X = strsplit(species," "),
                          FUN = function(x){x[1]}))
  
  query <- paste("SELECT DISTINCT scrubbed_family,scrubbed_genus,
                 scrubbed_species_binomial FROM bien_taxonomy
                 WHERE scrubbed_species_binomial in (",
                 paste(shQuote(species, type = "sh"),collapse = ', '),
                 ") or scrubbed_genus in (",
                 paste(shQuote(genera, type = "sh"),collapse = ', '), ") ;")
  
  
  taxonomy_for_traits <- .BIEN_sql(query, ...)
  #taxonomy_for_traits <- .BIEN_sql(query)
  if(length(taxonomy_for_traits) == 0){
    stop("Taxonomic data missing, check species name(s)")}
  
  
  #then, query the various taxonomic levels to get trait data

  query <- paste("SELECT * FROM agg_traits WHERE trait_name in (",
                 paste(shQuote(trait, type = "sh"),collapse = ', '),
                 ") AND (scrubbed_family in (",
                 paste(shQuote(unique(taxonomy_for_traits$scrubbed_family),
                               type = "sh"),collapse = ', '),
                 ") or  scrubbed_genus in (",
                 paste(shQuote(unique(taxonomy_for_traits$scrubbed_genus) ,
                               type = "sh"),collapse = ', '), ")) ;")
  
  traits_df <- suppressWarnings(.BIEN_sql(query, ...)) #suppress warnings to avoid the geom message
  #traits_df <- suppressWarnings(.BIEN_sql(query))
  
  if(length(traits_df) == 0){stop("No matching trait data for these taxa.")}
  
  #finally, choose the best available trait data
  
  output_data <- NULL
  for(i in 1:length(species)){
    
    species_i_data<-list()
    species_i_data[[1]]<-cbind(traits_df$trait_value[which(traits_df$scrubbed_species_binomial==species[i])],traits_df$id[which(traits_df$scrubbed_species_binomial==species[i])]  )
    species_i_data[[2]]<-cbind(traits_df$trait_value[which(traits_df$scrubbed_genus==taxonomy_for_traits$scrubbed_genus[which(taxonomy_for_traits$scrubbed_species_binomial==species[i])])],traits_df$id[which(traits_df$scrubbed_genus==taxonomy_for_traits$scrubbed_genus[which(taxonomy_for_traits$scrubbed_species_binomial==species[i])])])
    if(length(species_i_data[[2]])==0){
      species_i_data[[2]]<-cbind(traits_df$trait_value[which(traits_df$scrubbed_genus==strsplit(species[i]," ")[[1]][1])],traits_df$id[which(traits_df$scrubbed_genus==strsplit(species[i]," ")[[1]][1])])
    }
    #species_i_data[[3]]<-traits_df$trait_value[which(traits_df$family==taxonomy_for_traits$scrubbed_family[i])]
    species_i_data[[3]]<-cbind(traits_df$trait_value[which(traits_df$scrubbed_family==taxonomy_for_traits$scrubbed_family[which(taxonomy_for_traits$scrubbed_species_binomial==species[i])])],traits_df$id[which(traits_df$scrubbed_family==taxonomy_for_traits$scrubbed_family[which(taxonomy_for_traits$scrubbed_species_binomial==species[i])])])
    if(length(species_i_data[[3]])==0){
      species_i_data[[3]]<-cbind(traits_df$trait_value[which(traits_df$scrubbed_family==unique(taxonomy_for_traits$scrubbed_family[which(taxonomy_for_traits$scrubbed_genus==strsplit(species[i]," ")[[1]][1])]))],traits_df$id[which(traits_df$scrubbed_family==unique(taxonomy_for_traits$scrubbed_family[which(taxonomy_for_traits$scrubbed_genus==strsplit(species[i]," ")[[1]][1])]))])
    }
    species_i_data[[4]]<-"NA"
    
    names(species_i_data)<-c("Species","Genus","Family","NA")
    
    species_i_data<-species_i_data[which(lengths(species_i_data)>0)]#prunes list to include only taxonomic levels with data
    
    #trait_mean<- species_i_data[1]
    
    if(length(species_i_data)>0){
      level_used<-names(species_i_data[1])
      if(species_i_data[[1]][1]=="NA"){sample_size<-0}else{sample_size<-nrow(species_i_data[[1]])}
      if(species_i_data[[1]][1]=="NA"){mean_value<-"NA"}else{mean_value<-mean(as.numeric(species_i_data[[1]][,1]), na.rm = TRUE)}
      if(species_i_data[[1]][1]=="NA"){ids <-"NA"}else{ids<-paste(as.numeric(species_i_data[[1]][,2]),collapse = ",")}
      unit<-unique(traits_df$unit)
      
      output_data<-rbind(output_data,cbind(species[i],mean_value,trait,unit,level_used,sample_size,ids))
    }#if data is available
    
    
    
  }#i loop
  
  colnames(output_data)[1] <- "species"
  output_data <- as.data.frame(output_data)
  return(output_data)
  
}

############################

#'Download all measurements of a specific trait(s).
#'
#'BIEN_trait_trait downloads all measurements of the trait(s) specified.
#' @param trait A single trait or a vector of traits.
#' @template trait
#' @note Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
#' @return A dataframe of all available trait data for the given trait(s).
#' @examples \dontrun{
#' BIEN_trait_trait("whole plant height")
#' trait_vector<-c("whole plant height", "leaf dry mass per leaf fresh mass")
#' BIEN_trait_trait(trait_vector)}
#' @family trait functions
#' @export
BIEN_trait_trait <- function(trait,
                             all.taxonomy = FALSE,
                             political.boundaries = FALSE,
                             source.citation = FALSE,...){

  .is_char(trait)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  
  # set the query
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)
  source_<-.source_check_traits(source.citation)
  
  
  query <- paste("SELECT 
                 scrubbed_species_binomial, trait_name, trait_value, unit,
                 method, latitude, longitude, elevation_m, url_source",
                 source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select," , access, id 
                 FROM agg_traits WHERE trait_name in (",
                 paste(shQuote(trait, type = "sh"),collapse = ', '), ") ;")

  return(.BIEN_sql(query, ...))

}
############################

#'Download trait data for given species and trait.
#'
#'BIEN_trait_traitbyspecies extracts entries that contain the specified species and trait(s).
#' @param species A single species or a vector of species.
#' @param trait A single trait or a vector of traits.
#' @template trait
#' @note Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
#' @return A dataframe of all data matching the specified trait(s) and species.
#' @examples \dontrun{
#' BIEN_trait_traitbyspecies(trait = "whole plant height", species = "Carex capitata")
#' trait_vector<-c("whole plant height", "leaf area")
#' species_vector<-c("Carex capitata","Betula nana")
#' BIEN_trait_traitbyspecies(trait=trait_vector,species=species_vector)}
#' @family trait functions
#' @export
BIEN_trait_traitbyspecies <- function(species,
                                      trait,
                                      all.taxonomy = FALSE,
                                      political.boundaries = FALSE,
                                      source.citation = FALSE,
                                      ...){
  .is_char(species)
  .is_char(trait)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  
  # set the query
  taxonomy_ <- .taxonomy_check_traits(all.taxonomy)  
  political_ <- .political_check_traits(political.boundaries)
  source_ <- .source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_species_binomial, trait_name, trait_value, unit,
                 method, latitude, longitude, elevation_m, url_source",
                 source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select,", access, id 
                 FROM agg_traits 
                 WHERE scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") 
                 AND trait_name in (", paste(shQuote(trait, type = "sh"),collapse = ', '), ") ;")
  
  return(.BIEN_sql(query, ...))
  
}
###########################

#'Download trait data for given genus/genera and trait(s).
#'
#'BIEN_trait_traitbygenus extracts entries that contain the specified genus/genera and trait(s).
#' @param genus A single genus or a vector of genera.
#' @param trait A single trait or a vector of traits.
#' @template trait
#' @note Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
#' @return A dataframe of all data matching the specified trait(s) and genus/genera.
#' @examples \dontrun{
#' BIEN_trait_traitbygenus(trait = "whole plant height", genus = "Carex")
#' trait_vector<-c("whole plant height", "leaf area")
#' genus_vector<-c("Carex","Betula")
#' BIEN_trait_traitbygenus(trait=trait_vector,genus=genus_vector)}
#' @family trait functions
#' @export
BIEN_trait_traitbygenus <- function(genus,
                                    trait,
                                    all.taxonomy = FALSE,
                                    political.boundaries = FALSE,
                                    source.citation = FALSE,
                                    ...){

  .is_char(genus)
  .is_char(trait)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  # set the query
  
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)
  source_<-.source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_genus, scrubbed_species_binomial, trait_name,
                 trait_value, unit, method, latitude, longitude, elevation_m,
                 url_source",source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select,", access, id 
                 FROM agg_traits 
                 WHERE scrubbed_genus in (",
                 paste(shQuote(genus, type = "sh"),collapse = ', '), ") 
                 AND trait_name in (",
                 paste(shQuote(trait, type = "sh"),collapse = ', '), ") ;")

  return(.BIEN_sql(query, ...))

}
###########################

#'Download trait data for given families and traits.
#'
#'BIEN_trait_traitbyfamily extracts entries that contain the specified families and trait(s).
#' @param family A single family or a vector of families.
#' @param trait A single trait or a vector of traits.
#' @template trait
#' @note Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
#' @return A dataframe of all data matching the specified trait(s) and family/families.
#' @examples \dontrun{
#' BIEN_trait_traitbyfamily(trait = "whole plant height", family = "Poaceae")
#' trait_vector <- c("whole plant height", "leaf fresh mass")
#' family_vector < -c("Orchidaceae","Poaceae")
#' BIEN_trait_traitbyfamily(trait = trait_vector, family = family_vector)}
#' @family trait functions
#' @export
BIEN_trait_traitbyfamily <- function(family,
                                     trait,
                                     all.taxonomy = FALSE,
                                     political.boundaries = FALSE,
                                     source.citation = FALSE,
                                     ...){

  .is_char(family)
  .is_char(trait)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  
  # set the query
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)
  source_<-.source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_family, scrubbed_genus, scrubbed_species_binomial,
                 trait_name, trait_value, unit, method, latitude, longitude,
                 elevation_m, url_source",source_$select ,", project_pi,
                 project_pi_contact",
                 political_$select, taxonomy_$select,", access, id
                 FROM agg_traits 
                 WHERE scrubbed_family in (",
                 paste(shQuote(family, type = "sh"),collapse = ', '), ") 
                 AND trait_name in (",
                 paste(shQuote(trait, type = "sh"),collapse = ', '), ") ;")

  return(.BIEN_sql(query, ...))

}

############################

#'Download trait data for given genera.
#'
#'BIEN_trait_genus extracts entries that contain the specified genera.
#' @param genus A single genus or a vector of genera.
#' @template trait
#' @return A dataframe of all data matching the specified genera.
#' @examples \dontrun{
#' BIEN_trait_genus("Acer")
#' genus_vector <- c("Acer","Abies")
#' BIEN_trait_genus(genus_vector)}
#' @family trait functions
#' @export
BIEN_trait_genus <- function(genus,
                             all.taxonomy = FALSE,
                             political.boundaries = FALSE,
                             source.citation = FALSE,
                             ...){

  .is_char(genus)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  
  # set the query
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)
  source_<-.source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_genus, scrubbed_species_binomial, trait_name,
                 trait_value, unit, method, latitude, longitude, elevation_m,
                 url_source",source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select,", access,id 
                 FROM agg_traits WHERE scrubbed_genus in (",
                 paste(shQuote(genus, type = "sh"),collapse = ', '), ") ;")
  
  return(.BIEN_sql(query, ...))

}


###########################

#'Download trait data for given families.
#'
#'BIEN_trait_family extracts all trait data for the specified families.
#' @param family A single family or a vector of families.
#' @template trait
#' @return A dataframe of all data matching the specified families.
#' @examples \dontrun{
#' BIEN_trait_family("Poaceae")
#' family_vector<-c("Poaceae","Orchidaceae")
#' BIEN_trait_family(family_vector)}
#' @family trait functions
#' @export
BIEN_trait_family <- function(family,
                              all.taxonomy = FALSE,
                              political.boundaries = FALSE,
                              source.citation = FALSE,
                              ...){

  .is_char(family)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)

  # set the query
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)
  source_<-.source_check_traits(source.citation)
  
  query <- paste("SELECT 
                 scrubbed_family, scrubbed_genus, scrubbed_species_binomial,
                 trait_name, trait_value, unit, method, latitude, longitude,
                 elevation_m, url_source",source_$select ,", project_pi,
                 project_pi_contact",
                 political_$select, taxonomy_$select,", access,id 
                 FROM agg_traits WHERE scrubbed_family in (",
                 paste(shQuote(family, type = "sh"),collapse = ', '), ") ;")
  
  return(.BIEN_sql(query, ...))
  
}

############################

#'List all available types of trait data
#'
#'BIEN_trait_list produces a dataframe of all available types of trait data.
#' @param ... Additional arguments passed to internal functions.
#' @return A dataframe containing all currently available types of trait data and details on measurement.
#' @examples \dontrun{
#' BIEN_trait_list()}
#' @family trait functions
#' @export
BIEN_trait_list <- function( ...){
  
  # set the query
  query <- paste("SELECT DISTINCT trait_name FROM agg_traits ;")
  
  return(.BIEN_sql(query, ...))
  
}


###################################

#'Download trait data for given country.
#'
#'BIEN_trait_species extracts trait data for the species country.
#' @param country A single country or a vector of countries.
#' @param trait.name Optional.  The trait or traits you want returned.  If left blank, all traits will be returned.
#' @template trait
#' @return A dataframe of all available trait data for the given country.
#' @examples \dontrun{
#' BIEN_trait_country("South Africa")
#' BIEN_trait_country(country="South Africa",trait="whole plant growth form")}
#' @family trait functions
#' @export
BIEN_trait_country <- function(country,
                               trait.name = NULL,
                               all.taxonomy = FALSE,
                               political.boundaries = TRUE,
                               source.citation = FALSE,
                               ...){

  .is_char(country)
  .is_log(all.taxonomy)
  .is_log(political.boundaries)
  .is_log(source.citation)
  .is_char(trait.name)
  
  # set the query
  taxonomy_<-.taxonomy_check_traits(all.taxonomy)  
  political_<-.political_check_traits(political.boundaries)  
  source_<-.source_check_traits(source.citation)
  
  if(!is.null(trait.name)){
    trait_select<-paste(" AND", "trait_name in (",
                        paste(shQuote(trait.name, type = "sh"),
                              collapse = ', '), ") ")  
  }else{trait_select <- ""}
  
  
  query <- paste("SELECT 
                 scrubbed_species_binomial, trait_name, trait_value, unit,
                 method, latitude, longitude, elevation_m, url_source",
                 source_$select ,", project_pi, project_pi_contact",
                 political_$select, taxonomy_$select,", access, id 
                 FROM agg_traits WHERE country in (",
                 paste(shQuote(country, type = "sh"),collapse = ', '), ")",
                 trait_select," ;")
  
  return(.BIEN_sql(query, ...))
  
}



#############################

#'Count the number of (geoValid) occurrence records for each species in BIEN
#'
#'BIEN_occurrence_records_per_species downloads a count of the number of geovalidated occurrence records for each species in the BIEN database.
#' @param species A single species, or vector of species.  If NULL, the default, it will return counts for all species.
#' @param ... Additional arguments passed to internal functions.
#' @return A dataframe listing the number of geovalidated occurrence records for each species in the BIEN database.
#' @examples \dontrun{
#' occurrence_counts<-BIEN_occurrence_records_per_species()}
#' @family occurrence functions
#' @export
BIEN_occurrence_records_per_species <- function(species = NULL,
                                                ...){
  
  if(is.null(species)){    
    # set the query
    query <- paste("SELECT DISTINCT scrubbed_species_binomial,count(*)
                   FROM view_full_occurrence_individual
                   WHERE is_geovalid = 1
                    AND latitude IS NOT NULL
                    AND LONGITUDE IS NOT NULL
                   GROUP BY scrubbed_species_binomial ;")
  }
  
  if(is.character(species)){
    query <- paste("SELECT scrubbed_species_binomial,count(*) 
                 FROM view_full_occurrence_individual 
                 WHERE scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") 
                  AND is_geovalid = 1 AND (georef_protocol is NULL OR georef_protocol<>'county centroid') 
                  AND (is_centroid IS NULL OR is_centroid=0) 
                  AND observation_type IN ('plot','specimen','literature','checklist')    
                 GROUP BY scrubbed_species_binomial ;")
  }
  
  return(.BIEN_sql(query, ...))
  
  }

###############################################
#'Count the number of trait observations for each species in the BIEN database
#'
#'BIEN_trait_traits_per_species downloads a count of the number of records for each trait for each species in the BIEN database.
#' @param species Optional species or vector of species.  If left blank, returns counts for all species.
#' @param ... Additional arguments passed to internal functions.
#' @return Returns a dataframe containing the number of trait records for each species in the BIEN database.
#' @examples \dontrun{
#' trait_observation_counts<-BIEN_trait_traits_per_species()}
#' @family trait functions
#' @export
BIEN_trait_traits_per_species <- function(species = NULL,
                                          ...){

  if(!is.null(species)){
    
    species_query<-paste("WHERE scrubbed_species_binomial in (",
                         paste(shQuote(species, type = "sh"),collapse = ', '),
                         ")")  
    
  }else{
    
    species_query <- ""  

  }
  
  # set the query
  query <- paste("SELECT DISTINCT scrubbed_species_binomial, trait_name,count(*) 
                 FROM agg_traits",
                 species_query,
                 "GROUP BY trait_name,scrubbed_species_binomial ;")
  
  return(.BIEN_sql(query, ...))
  
}

################################
#Plot queries

##############################
#'Download plot data from a given datasource.
#'
#'BIEN_plot_datasource downloads all plot data from a given datasource.
#' @param datasource A datasource. See \code{\link{BIEN_plot_list_datasource}} for options.
#' @template plot
#' @return A dataframe containing all data from the specified datasource.
#' @examples \dontrun{
#' BIEN_plot_datasource("SALVIAS")}
#' @family plot functions
#' @export
BIEN_plot_datasource <- function(datasource,
                                 cultivated = FALSE,
                                 new.world = NULL,
                                 all.taxonomy = FALSE,
                                 native.status = FALSE,
                                 natives.only = TRUE,
                                 political.boundaries = FALSE,
                                 collection.info = FALSE,
                                 all.metadata = FALSE,
                                 ...){

  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(datasource)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)

  #set conditions for query
  cultivated_ <- .cultivated_check_plot(cultivated)
  newworld_ <- .newworld_check_plot(new.world)
  taxonomy_ <- .taxonomy_check_plot(all.taxonomy)
  native_ <- .native_check_plot(native.status)
  natives_ <- .natives_check_plot(natives.only)
  political_ <- .political_check_plot(political.boundaries)
  collection_ <- .collection_check_plot(collection.info)
  md_ <- .md_check_plot(all.metadata)
  
  # set the query
  query <- paste("SELECT view_full_occurrence_individual.plot_name,view_full_occurrence_individual.subplot, view_full_occurrence_individual.elevation_m,
                 view_full_occurrence_individual.plot_area_ha,view_full_occurrence_individual.sampling_protocol,
                 view_full_occurrence_individual.recorded_by, view_full_occurrence_individual.scrubbed_species_binomial,
                 view_full_occurrence_individual.individual_count",taxonomy_$select,political_$select,native_$select," ,view_full_occurrence_individual.latitude, 
                 view_full_occurrence_individual.longitude,view_full_occurrence_individual.date_collected,view_full_occurrence_individual.datasource,
                 view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,view_full_occurrence_individual.custodial_institution_codes,
                 view_full_occurrence_individual.collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                  (SELECT * FROM view_full_occurrence_individual 
                   WHERE view_full_occurrence_individual.datasource in (", paste(shQuote(datasource, type = "sh"),collapse = ', '), ")",cultivated_$query,newworld_$query,natives_$query,  "
                    AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                    AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                    AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0)  
                    AND observation_type='plot'
                    AND scrubbed_species_binomial IS NOT NULL ) as view_full_occurrence_individual
                 JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                  ;")
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
  }


##################################
#'List available datasources.
#'
#'BIEN_plot_list_datasource list all plot datasources in the BIEN database.
#' @param ... Additional arguments passed to internal functions.
#' @return A vector of available datasources.
#' @examples \dontrun{
#' BIEN_plot_list_datasource()}
#' @family plot functions
#' @export
BIEN_plot_list_datasource <- function(...){
  
  query <- paste("SELECT DISTINCT plot_metadata.datasource FROM plot_metadata ;")
  return(.BIEN_sql(query, ...))
  
}

###############################
#'Download plot data from specified countries.
#'
#'BIEN_plot_country downloads all plot data from specified countries.
#' @param country A country or vector of countries.
#' @param country.code A single country code or a vector of country codes equal in length to the vector of states/province codes.
#' @template plot
#' @return A dataframe containing all data from the specified countries.
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @examples \dontrun{
#' BIEN_plot_country("Costa Rica")
#' BIEN_plot_country(c("Costa Rica","Panama"))}
#' @family plot functions
#' @export
BIEN_plot_country <- function(country = NULL,
                              country.code = NULL,
                              cultivated = FALSE,
                              new.world = NULL,
                              all.taxonomy = FALSE,
                              native.status = FALSE,
                              natives.only = TRUE,
                              political.boundaries = FALSE,
                              collection.info = FALSE,
                              all.metadata = FALSE,
                              ...){

  .is_char(country.code)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(country)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)
  if(is.null(country)& is.null(country.code))  {stop("Please supply either a country name or 2-digit ISO code")}
  
  #set conditions for query
    cultivated_ <- .cultivated_check_plot(cultivated)
    newworld_ <- .newworld_check_plot(new.world)
    taxonomy_ <- .taxonomy_check_plot(all.taxonomy)
    native_ <- .native_check_plot(native.status)
    natives_ <- .natives_check_plot(natives.only)
    collection_ <- .collection_check_plot(collection.info)
    md_ <- .md_check_plot(all.metadata)
  
  if(!political.boundaries){
    political_select <- "view_full_occurrence_individual.country,"
  }else{
    political_select <- "view_full_occurrence_individual.country,view_full_occurrence_individual.state_province,view_full_occurrence_individual.county,view_full_occurrence_individual.locality,"
  }
  
  # set the query
  if(is.null(country.code)){
    query <- paste("SELECT ",political_select," view_full_occurrence_individual.plot_name,view_full_occurrence_individual.subplot, view_full_occurrence_individual.elevation_m,
                      view_full_occurrence_individual.plot_area_ha, view_full_occurrence_individual.sampling_protocol,view_full_occurrence_individual.recorded_by, 
                      view_full_occurrence_individual.scrubbed_species_binomial,view_full_occurrence_individual.individual_count",taxonomy_$select,native_$select,", 
                      view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude, view_full_occurrence_individual.date_collected,
                      view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,
                      view_full_occurrence_individual.custodial_institution_codes,view_full_occurrence_individual.collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                   FROM 
                      (SELECT * FROM view_full_occurrence_individual 
                       WHERE view_full_occurrence_individual.country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ")",
                          cultivated_$query,newworld_$query,natives_$query,  "AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                          AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                          AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                          AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0) AND observation_type='plot' 
                          AND scrubbed_species_binomial IS NOT NULL) as view_full_occurrence_individual 
                   LEFT JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                    ;")
  }else{
    
    query <- paste("SELECT ",political_select," view_full_occurrence_individual.plot_name,view_full_occurrence_individual.subplot, view_full_occurrence_individual.elevation_m,
                      view_full_occurrence_individual.plot_area_ha, view_full_occurrence_individual.sampling_protocol,view_full_occurrence_individual.recorded_by, 
                      view_full_occurrence_individual.scrubbed_species_binomial,view_full_occurrence_individual.individual_count",taxonomy_$select,native_$select,", 
                      view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude, view_full_occurrence_individual.date_collected,
                      view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,
                      view_full_occurrence_individual.custodial_institution_codes,view_full_occurrence_individual.collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                    FROM 
                      (SELECT * FROM view_full_occurrence_individual WHERE view_full_occurrence_individual.country in 
                        (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), "))",
                        cultivated_$query,newworld_$query,natives_$query,  "
                        AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                        AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                        AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                        AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0) AND observation_type='plot'
                        AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL) as view_full_occurrence_individual 
                   LEFT JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                    ;")  
    
    
  }
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
}
###############################
#'Download plot data from specified states/provinces.
#'
#'BIEN_plot_state downloads all plot data from specified states/provinces.
#' @param country A single country.
#' @param state A state or vector of states (or other primary political divisions).
#' @param state.code A single state/province code, or a vector of states/province codes.
#' @param country.code A single country code or a vector of country codes equal in length to the vector of states/province codes.
#' @template plot
#' @note Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
#' @note This function requires you supply either 1) a single country with one or states, or 2) vectors of equal length for each political level.
#' @return A dataframe containing all data from the specified states.
#' @examples \dontrun{
#' BIEN_plot_state(country="United States", state="Colorado")
#' BIEN_plot_state(country="United States",state= c("Colorado","California"))}
#' @family plot functions
#' @export
BIEN_plot_state <- function(country = NULL,
                            state = NULL,
                            country.code = NULL,
                            state.code = NULL,
                            cultivated = FALSE,
                            new.world = NULL,
                            all.taxonomy = FALSE,
                            native.status = FALSE,
                            natives.only = TRUE,
                            political.boundaries = TRUE,
                            collection.info = FALSE,
                            all.metadata = FALSE,
                            ...){

  .is_char(country)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(state)
  .is_char(state.code)
  .is_char(country.code)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)
  
  #set conditions for query
  cultivated_<-.cultivated_check_plot(cultivated)
  newworld_<-.newworld_check_plot(new.world)
  taxonomy_<-.taxonomy_check_plot(all.taxonomy)
  native_<-.native_check_plot(native.status)
  natives_<-.natives_check_plot(natives.only)
  collection_<-.collection_check_plot(collection.info)
  md_<-.md_check_plot(all.metadata)
  
  if(!political.boundaries){
    political_select<-"view_full_occurrence_individual.country,view_full_occurrence_individual.state_province,"
  }else{
    political_select<-"view_full_occurrence_individual.country,view_full_occurrence_individual.state_province,view_full_occurrence_individual.county,view_full_occurrence_individual.locality,"
  }
  
  if(is.null(country.code) & is.null(state.code) ){  

    #state where
    if(length(country)==1){
      sql_where <- paste(" WHERE country in (", paste(shQuote(country, type = "sh"),collapse = ', '), ") 
                         AND state_province in (", paste(shQuote(state, type = "sh"),collapse = ', '), ") 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country)==length(state)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country)){
          
          condition_i<- paste("(country = ", paste(shQuote(country[i], type = "sh"),collapse = ', '), " AND state_province = ", paste(shQuote(state[i], type = "sh"),collapse = ', '), ")")
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  
        
      }  
  
    }#if length(country>1)
    
  }else{
    
    
    if(length(country.code)==1){
      sql_where <- paste(" WHERE country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code, type = "sh"),collapse = ', '), ")) 
                         AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code, type = "sh"),collapse = ', '), ")) 
                         AND scrubbed_species_binomial IS NOT NULL")
    }else{
      
      if(length(country.code)==length(state.code)){
        
        sql_where<-"WHERE ("
        
        for(i in 1:length(country.code)){
          
          condition_i<- paste("country in (SELECT country FROM country WHERE iso in (", paste(shQuote(country.code[i], type = "sh"),collapse = ', '), ")) 
                              AND state_province in (SELECT state_province_ascii FROM county_parish WHERE admin1code in (", paste(shQuote(state.code[i], type = "sh"),collapse = ', '), "))")
          if(i!=1){condition_i<- paste("OR ",condition_i)}#stick OR onto the condition where needed
          sql_where<-paste(sql_where,condition_i)
          
        }#for i  
        
        sql_where<-paste(sql_where,") AND scrubbed_species_binomial IS NOT NULL")  
        
      }else{
        stop("If supplying more than one country, the function requires a vector of countries corresponding to the vector of states")  
        
      }  
      
    }#if length(country>1)
    
  }    
  
  # set the query
  query <- paste("SELECT ",political_select," view_full_occurrence_individual.plot_name,subplot, view_full_occurrence_individual.elevation_m, 
                    view_full_occurrence_individual.plot_area_ha,view_full_occurrence_individual.sampling_protocol,recorded_by, scrubbed_species_binomial,individual_count",
                    taxonomy_$select,native_$select," ,view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude,view_full_occurrence_individual.date_collected,
                    view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,custodial_institution_codes,
                    collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM view_full_occurrence_individual ",
                      sql_where,cultivated_$query,newworld_$query,natives_$query,  "
                      AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                      AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                      AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0) 
                      AND observation_type='plot' 
                      AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL) as view_full_occurrence_individual 
                 JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                  ;")
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
  }
###############################

#'Download plot data from specified sf object.
#'
#'BIEN_plot_sf downloads all plot data falling within a supplied sf polygon.
#' @param sf An object of class sf.  Note that the projection must be WGS84.
#' @template plot
#' @return A dataframe containing all plot data from within the specified sf polygon.
#' @examples \dontrun{
#' library(sf)
#' 
#' BIEN_ranges_species("Carnegiea gigantea") #saves ranges to the current working directory
#' 
#' sf <- st_read(dsn = ".",
#'               layer = "Carnegiea_gigantea")
#' 
#' saguaro_plot_data <- BIEN_plot_sf(sf = sf)
#' }
#' @family plot functions
#' @importFrom sf st_geometry st_as_text
#' @export
BIEN_plot_sf <- function(sf,
                         cultivated = FALSE,
                         new.world = NULL,
                         all.taxonomy = FALSE,
                         native.status = FALSE,
                         natives.only = TRUE,
                         political.boundaries = TRUE,
                         collection.info = FALSE,
                         all.metadata = FALSE,
                         ...){
  
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)
  
  
  # Convert the sf to wkt (needed for sql query)
  
  wkt <- sf |>
    st_geometry() |>
    st_as_text()
  
  #set conditions for query
  
  cultivated_ <- .cultivated_check_plot(cultivated)
  newworld_ <- .newworld_check_plot(new.world)
  taxonomy_ <- .taxonomy_check_plot(all.taxonomy)
  native_ <- .native_check_plot(native.status)
  natives_ <- .natives_check_plot(natives.only)
  collection_ <- .collection_check_plot(collection.info)
  md_ <- .md_check_plot(all.metadata)
  
  if(!political.boundaries){
    
    political_select <- "view_full_occurrence_individual.country,"
    
  }else{
    
    political_select <- "view_full_occurrence_individual.country,view_full_occurrence_individual.state_province,view_full_occurrence_individual.county,view_full_occurrence_individual.locality,"
    
  }
  
  
  # set the query
  
  query <- paste("SELECT ",political_select," view_full_occurrence_individual.plot_name,subplot, view_full_occurrence_individual.elevation_m, 
                      view_full_occurrence_individual.plot_area_ha,view_full_occurrence_individual.sampling_protocol,recorded_by, scrubbed_species_binomial,individual_count",
                 taxonomy_$select,native_$select," ,view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude,view_full_occurrence_individual.date_collected,
                      view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,custodial_institution_codes,
                      collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                   FROM 
                      (SELECT * FROM view_full_occurrence_individual ",
                 "WHERE st_intersects(ST_GeographyFromText('SRID=4326;",paste(wkt),"'),geom) ",cultivated_$query,newworld_$query,natives_$query,  "
                          AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                          AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                          AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0) 
                          AND observation_type='plot' 
                          AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL ) as view_full_occurrence_individual 
                   JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                   ;")
  
  # create query to retrieve
  
  return(.BIEN_sql(query, ...))
  #return(.BIEN_sql(query))
  
}




###############################
#'List available sampling protocols.
#'
#'BIEN_plot_list_sampling_protocols list all available sampling protocols.
#' @param ... Additional arguments passed to internal functions.
#' @return A vector of available sampling protocols.
#' @examples \dontrun{
#' BIEN_plot_list_sampling_protocols()}
#' @family plot functions
#' @export
BIEN_plot_list_sampling_protocols <- function(...){
  
  query <- paste("SELECT DISTINCT sampling_protocol FROM plot_metadata ;")
  return(.BIEN_sql(query, ...))
  
}

################################
#'Download plot data using a specified sampling protocol.
#'
#'BIEN_plot_sampling_protocol downloads all plot data using a specified sampling protocol.
#' @param sampling_protocol A sampling protocol or vector of sampling protocols. See \code{\link{BIEN_plot_list_sampling_protocols}} for options.
#' @template plot
#' @return A dataframe containing all data from the specified datasource.
#' @examples \dontrun{
#' BIEN_plot_sampling_protocol("Point-intercept")}
#' @family plot functions
#' @export
BIEN_plot_sampling_protocol <- function (sampling_protocol,
                                         cultivated = FALSE,
                                         new.world = FALSE,
                                         all.taxonomy = FALSE,
                                         native.status = FALSE,
                                         natives.only = TRUE, 
                                         political.boundaries = FALSE,
                                         collection.info = FALSE,
                                         all.metadata = FALSE,
                                         ...){
  
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(sampling_protocol)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)
  
  cultivated_ <- .cultivated_check_plot(cultivated)
  newworld_ <- .newworld_check_plot(new.world)
  taxonomy_ <- .taxonomy_check_plot(all.taxonomy)
  native_ <- .native_check_plot(native.status)
  natives_ <- .natives_check_plot(natives.only)
  political_ <- .political_check_plot(political.boundaries)
  collection_ <- .collection_check_plot(collection.info)
  md_ <- .md_check_plot(all.metadata)
  
  query <- paste("SELECT view_full_occurrence_individual.plot_name,subplot, view_full_occurrence_individual.elevation_m, view_full_occurrence_individual.plot_area_ha, 
                    view_full_occurrence_individual.sampling_protocol,recorded_by, scrubbed_species_binomial,individual_count", 
                    taxonomy_$select, native_$select, political_$select,",
                    view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude,date_collected,view_full_occurrence_individual.datasource,
                    view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,custodial_institution_codes,collection_code,
                    view_full_occurrence_individual.datasource_id", collection_$select, cultivated_$select, newworld_$select, md_$select, " 
                 FROM 
                    (SELECT * FROM view_full_occurrence_individual 
                    WHERE view_full_occurrence_individual.sampling_protocol in (", paste(shQuote(sampling_protocol, type = "sh"), collapse = ", "), ")",
                        cultivated_$query, newworld_$query, natives_$query, 
                        "AND view_full_occurrence_individual.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                        AND (view_full_occurrence_individual.is_geovalid = 1 ) 
                        AND (view_full_occurrence_individual.georef_protocol is NULL OR view_full_occurrence_individual.georef_protocol<>'county centroid') 
                        AND (view_full_occurrence_individual.is_centroid IS NULL OR view_full_occurrence_individual.is_centroid=0) 
                        AND view_full_occurrence_individual.observation_type='plot' 
                        AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL) as view_full_occurrence_individual
                 JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id) ;")
  return(.BIEN_sql(query, ...))
}


#################################
#'Download plot data by plot name.
#'
#'BIEN_plot_name downloads all plot data for a set of plot names.
#' @param plot.name A plot name or vector of names. See BIEN_plot_metadata for more information on plots.
#' @template plot
#' @note Plot names can be looked up with \code{\link{BIEN_plot_metadata}}.
#' @return A dataframe containing all data from the specified plot(s).
#' @examples \dontrun{
#' BIEN_plot_name("SR-1")}
#' @family plot functions
#' @export
BIEN_plot_name <- function(plot.name,
                           cultivated = FALSE,
                           new.world = NULL,
                           all.taxonomy = FALSE,
                           native.status = FALSE,
                           natives.only = TRUE,
                           political.boundaries = FALSE,
                           collection.info = FALSE,
                           all.metadata = FALSE,
                           ...){

  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(plot.name)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)  
  .is_log(collection.info)
  .is_log(all.metadata)
  
  #set conditions for query
  cultivated_<-.cultivated_check_plot(cultivated)
  newworld_<-.newworld_check_plot(new.world)
  taxonomy_<-.taxonomy_check_plot(all.taxonomy)
  native_<-.native_check_plot(native.status)
  natives_<-.natives_check_plot(natives.only)
  political_<-.political_check_plot(political.boundaries)
  collection_<-.collection_check_plot(collection.info)
  md_<-.md_check_plot(all.metadata)
  
  # set the query
  query <- paste("SELECT view_full_occurrence_individual.plot_name,subplot, view_full_occurrence_individual.elevation_m, view_full_occurrence_individual.plot_area_ha,
                    view_full_occurrence_individual.sampling_protocol,view_full_occurrence_individual.recorded_by, view_full_occurrence_individual.scrubbed_species_binomial,
                    view_full_occurrence_individual.individual_count",taxonomy_$select,native_$select,political_$select,", 
                    view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude,view_full_occurrence_individual.date_collected,
                    view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,view_full_occurrence_individual.dataowner,
                    view_full_occurrence_individual.custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM view_full_occurrence_individual WHERE view_full_occurrence_individual.plot_name in (", paste(shQuote(plot.name, type = "sh"),collapse = ', '), ")",
                      cultivated_$query,newworld_$query,natives_$query,  "AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') AND is_geovalid = 1 
                      AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) AND observation_type='plot' 
                      AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL ) as view_full_occurrence_individual
                 LEFT JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                  ;")
  
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
  }

#####################
#'Download plot data by dataset.
#'
#'BIEN_plot_dataset downloads all plot data for a given dataset or datasets.
#' @param dataset A plot dataset or vector of datasets. See BIEN_plot_metadata for more information on plots.
#' @template plot
#' @return A dataframe containing all data from the specified dataset.
#' @note Datasets and related information can be looked up with \code{\link{BIEN_plot_metadata}}
#' @examples \dontrun{
#' BIEN_plot_dataset("Gentry Transect Dataset")}
#' @family plot functions
#' @export
BIEN_plot_dataset <- function(dataset,
                              cultivated = FALSE,
                              new.world = NULL,
                              all.taxonomy = FALSE,
                              native.status = FALSE,
                              natives.only = TRUE,
                              political.boundaries = FALSE,
                              collection.info = FALSE,
                              all.metadata = FALSE,
                              ...){

  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(dataset)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  .is_log(all.metadata)
  
  #set conditions for query
  cultivated_<-.cultivated_check_plot(cultivated)
  newworld_<-.newworld_check_plot(new.world)
  taxonomy_<-.taxonomy_check_plot(all.taxonomy)
  native_<-.native_check_plot(native.status)
  natives_<-.natives_check_plot(natives.only)
  political_<-.political_check_plot(political.boundaries)
  collection_<-.collection_check_plot(collection.info)
  md_<-.md_check_plot(all.metadata)
  
  # set the query
  query <- paste("SELECT view_full_occurrence_individual.plot_name,subplot, view_full_occurrence_individual.elevation_m, view_full_occurrence_individual.plot_area_ha,
                    view_full_occurrence_individual.sampling_protocol,recorded_by, scrubbed_species_binomial,individual_count",taxonomy_$select,native_$select,political_$select,", 
                    view_full_occurrence_individual.latitude, view_full_occurrence_individual.longitude,view_full_occurrence_individual.date_collected,
                    view_full_occurrence_individual.datasource,view_full_occurrence_individual.dataset,
                    view_full_occurrence_individual.dataowner,custodial_institution_codes,collection_code,view_full_occurrence_individual.datasource_id",
                    collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM view_full_occurrence_individual 
                    WHERE view_full_occurrence_individual.dataset in (", paste(shQuote(dataset, type = "sh"),collapse = ', '), ")",
                        cultivated_$query,newworld_$query,natives_$query,  " AND higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                        AND is_geovalid = 1 AND (georef_protocol is NULL OR georef_protocol<>'county centroid') AND (is_centroid IS NULL OR is_centroid=0) 
                        AND observation_type='plot' AND view_full_occurrence_individual.scrubbed_species_binomial IS NOT NULL ) as view_full_occurrence_individual
                 LEFT JOIN plot_metadata ON (view_full_occurrence_individual.plot_metadata_id=plot_metadata.plot_metadata_id)
                  ;")
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
  }


##############################
#'Download plot metadata
#'
#'BIEN_plot_metadata downloads the plot metadata table. 
#' @param ... Additional arguments passed to internal functions.
#' @return A dataframe containing plot metadata.
#' @examples \dontrun{
#' BIEN_plot_metadata()}
#' @family plot functions
#' @family metadata functions
#' @export
BIEN_plot_metadata <- function( ...){
  
  # set the query
  query <- "SELECT * FROM plot_metadata ;"
  
  # create query to retrieve
  return(.BIEN_sql(query, ...))
  
}



##############################

################################
#Taxonomy queries

#'Extract taxonomic information for species
#'
#'BIEN_taxonomy_species downloads a dataframe of all taxonomic information for given species.
#' @param species A single species or a vector of species.
#' @template taxonomy
#' @return Dataframe containing taxonomic information for the specified species.
#' @examples \dontrun{
#' BIEN_taxonomy_species("Cannabis sativa")
#' species_vector<-c("Acer nigrum","Cannabis sativa")
#' BIEN_taxonomy_species(species_vector)}
#' @family taxonomy functions
#' @export
BIEN_taxonomy_species <- function(species,
                                  ...){

  .is_char(species)
  
  #set base query components
    sql_select <-  paste('SELECT DISTINCT higher_plant_group, "class", superorder, "order", scrubbed_family,scrubbed_genus,scrubbed_species_binomial,scrubbed_author,scrubbed_taxonomic_status')
    sql_from <- paste(' FROM bien_taxonomy')
    sql_where <- paste(' WHERE scrubbed_species_binomial in (', paste(shQuote(species, type = "sh"),collapse = ', '), ') AND scrubbed_species_binomial IS NOT NULL')

  # form the final query
    query <- paste(sql_select, sql_from, sql_where,  " ;")
  
  # execute the query
  
    return(.BIEN_sql(query, ...))
  
}

#################
#'Extract taxonomic information for genera
#'
#'BIEN_taxonomy_genus downloads a dataframe of all taxonomic information for given genera.
#' @param genus A single genus or a vector of genera.
#' @template taxonomy
#' @return Dataframe containing taxonomic information for the specified genera.
#' @examples \dontrun{
#' BIEN_taxonomy_genus("Acer")
#' genus_vector<-c("Acer","Quercus")
#' BIEN_taxonomy_genus(genus_vector)}
#' @family taxonomy functions
#' @export
BIEN_taxonomy_genus <- function(genus,
                                ...){

  .is_char(genus)
  
  #set base query components
    sql_select <-  paste('SELECT DISTINCT higher_plant_group, "class", superorder, "order", scrubbed_family,scrubbed_genus,scrubbed_species_binomial,scrubbed_author,scrubbed_taxonomic_status')
    sql_from <- paste(" FROM bien_taxonomy")
    sql_where <- paste(" WHERE scrubbed_genus in (", paste(shQuote(genus, type = "sh"),collapse = ', '), ") AND scrubbed_species_binomial IS NOT NULL")

  # form the final query
    query <- paste(sql_select, sql_from, sql_where, " ;")
  
  # execute the query
    return(.BIEN_sql(query, ...))

}


###########
#'Extract taxonomic information for families
#'
#'BIEN_taxonomy_family downloads a dataframe of all taxonomic information for given families.
#' @param family A single family or a vector of families.
#' @template taxonomy
#' @return Dataframe containing taxonomic information for the specified families.
#' @examples \dontrun{
#' BIEN_taxonomy_family("Orchidaceae")
#' family_vector<-c("Orchidaceae","Poaceae")
#' BIEN_taxonomy_family(family_vector)}
#' @family taxonomy functions
#' @export
BIEN_taxonomy_family <- function(family,
                                 ...){

  .is_char(family)
  
  #set base query components
    sql_select <-  paste('SELECT DISTINCT higher_plant_group, "class", superorder, "order", scrubbed_family,scrubbed_genus,scrubbed_species_binomial,scrubbed_author,scrubbed_taxonomic_status')
    sql_from <- paste(" FROM bien_taxonomy")
    sql_where <- paste(" WHERE scrubbed_family in (", paste(shQuote(family, type = "sh"),collapse = ', '), ") AND scrubbed_species_binomial IS NOT NULL")

  # form the final query
    query <- paste(sql_select, sql_from, sql_where, " ;")
    #print(query)
  
  # execute the query
    return(.BIEN_sql(query, ...))
  
}

################################

###############################
#Phylogeny fxs

#'Download the complete BIEN phylogenies
#'
#'BIEN_phylogeny_complete downloads a specified number of the BIEN phylogeny replicates.
#' @param n_phylogenies The number of phylogenies to download.  Should be an integer between 1 and 100.  Default is 1.
#' @param seed Argument passed to set.seed.  Useful for replicating work with random phylogeny sets.
#' @param replicates The specific replicated phylogenies to return.  Should be a numeric vector of integers between 1 and 100.
#' @template phylogeny
#' @return A phylo or multiphylo object containing the specified phylogenies
#' @examples \dontrun{
#' phylos<-BIEN_phylogeny_complete(n_phylogenies = 10,seed = 1)
#' phylos<-BIEN_phylogeny_complete(replicates = c(1,2,99,100))}
#' @family phylogeny functions
#' @importFrom ape read.tree
#' @export
BIEN_phylogeny_complete<-function(n_phylogenies = 1,
                                  seed = NULL,
                                  replicates = NULL,
                                  ...){

  .is_num(n_phylogenies)  
  
  if(!is.null(replicates)){
    replicates <- replicates[which(replicates%in%1:100)]
    query <- paste("SELECT * FROM phylogeny WHERE phylogeny_version = 'BIEN_2016_complete' AND replicate in (", paste(shQuote(replicates, type = "sh"),collapse = ', '),")"  )
    
    df <- .BIEN_sql(query, ...)
    
    tree <- read.tree(text = df$phylogeny, tree.names = df$replicate)
    
    return(tree)
    
  }
  
  
  set.seed(seed)  
  
  if(n_phylogenies > 100){
    message("n_phylogenies must be an integer between 1 and 100.  Setting n_phylogenies to 100")  
    n_phylogenies <- 100  
    
  }
  
  if(n_phylogenies < 1){
    message("n_phylogenies must be an integer between 1 and 100.  Setting n_phylogenies to 1")  
    n_phylogenies <- 1  
    
  }
  
  phylo_sample <- sample(x = 1:100, size = n_phylogenies, replace = FALSE)
  
  
  query <- paste("SELECT * FROM phylogeny WHERE phylogeny_version = 'BIEN_2016_complete' AND replicate in (", paste(shQuote(phylo_sample, type = "sh"),collapse = ', '),")"  )
  
  df <- .BIEN_sql(query, ...)
  
  tree <- read.tree(text = df$phylogeny, tree.names = df$replicate)
  
  return(tree)
  
}

###############################
#'Download the conservative BIEN phylogeny
#'
#'BIEN_phylogeny_conservative downloads the conservative BIEN phylogeny, which only includes species with molecular data available.
#' @template phylogeny
#' @return A phylo object containing the BIEN conservative phylogeny
#' @examples \dontrun{
#' BIEN_phylo<-BIEN_phylogeny_conservative()}
#' @family phylogeny functions
#' @importFrom ape read.tree
#' @export
BIEN_phylogeny_conservative <- function(...){
  
  query <- paste("SELECT * FROM phylogeny WHERE phylogeny_version = 'BIEN_2016_conservative' ;"  )
  
  df <- .BIEN_sql(query, ...)
  
  tree <- read.tree(text = df$phylogeny,tree.names = df$replicate)
  
  return(tree)
  
}

#################################


#'Label nodes on a phylogeny
#'
#'BIEN_phylogeny_label_nodes will label the nodes on a phylogeny based on either the BIEN taxonomy or user-supplied taxa.
#' @param phylogeny A single phylogeny.
#' @param family Should family-level nodes be labeled?  Default is TRUE.
#' @param genus Should genus-level nodes be labeled?  Default is FALSE.  Overwrites family-level nodes where a family contains a single genera.
#' @param other_taxa A dataframe containing two columns: 1) the taxa to be labelled; 2) the species associated with each taxon.
#' @template phylogeny
#' @return Input phylogeny with labeled nodes.
#' @examples \dontrun{
#' phylogeny<-BIEN_phylogeny_conservative()
#'
#'phylogeny<-drop.tip(phy = phylogeny,tip = 101:length(phylogeny$tip.label))
#'plot.phylo(x = phylogeny,show.tip.label = FALSE)
#'
#'fam_nodes<-BIEN_phylogeny_label_nodes(phylogeny = phylogeny,family = TRUE)
#'plot.phylo(x = fam_nodes,show.tip.label = FALSE, show.node.label = TRUE)
#'
#'gen_nodes<-BIEN_phylogeny_label_nodes(phylogeny = phylogeny, family = FALSE, genus = TRUE)
#'plot.phylo(x = gen_nodes, show.tip.label = FALSE, show.node.label = TRUE)
#'
#'other_taxa <- as.data.frame(matrix(nrow = 10,ncol = 2))
#'colnames(other_taxa)<-c("taxon","species")
#'other_taxa$taxon[1:5]<-"A" #Randomly assign a few species to taxon A
#'other_taxa$taxon[6:10]<-"B" #Randomly assign a few species to taxon B
#'tax_nodes <- 
#'  BIEN_phylogeny_label_nodes(phylogeny = phylogeny,
#'                             family = FALSE, genus = FALSE, other_taxa = other_taxa)
#'plot.phylo(x = tax_nodes,show.tip.label = FALSE,show.node.label = TRUE)}
#' @family phylogeny functions
#' @importFrom ape getMRCA
#' @export
BIEN_phylogeny_label_nodes <- function(phylogeny,
                                       family = TRUE,
                                       genus = FALSE,
                                       other_taxa = NULL,
                                       ...){
  
  if(is.null(phylogeny$node.label)){
    phylogeny$node.label[1:phylogeny$Nnode] <- NA
  }
  
  taxonomy <- BIEN_taxonomy_species(species = gsub(pattern = "_",replacement = " ",x = phylogeny$tip.label))
  
  if(family == TRUE){
    for(i in 1:length(unique(taxonomy$scrubbed_family))){
      
      fam_i <- unique(taxonomy$scrubbed_family)[i]  
      spp_i <- taxonomy$scrubbed_species_binomial[which(taxonomy$scrubbed_family == fam_i)]
      mrca_i <- getMRCA(phy = phylogeny,
                             tip = which(phylogeny$tip.label %in% gsub(pattern = " ",replacement = "_", x = spp_i   ) )) 
      phylogeny$node.label[mrca_i-length(phylogeny$tip.label)] <- fam_i  
      
    }}
  
  if(genus == TRUE){
    for(i in 1:length(unique(taxonomy$scrubbed_genus))){
      
      gen_i <- unique(taxonomy$scrubbed_genus)[i]  
      spp_i <- taxonomy$scrubbed_species_binomial[which(taxonomy$scrubbed_genus == gen_i)]
      mrca_i <- getMRCA(phy = phylogeny,
                             tip = which(phylogeny$tip.label %in% gsub(pattern = " ",replacement = "_", x = spp_i   ) )) 
      phylogeny$node.label[mrca_i-length(phylogeny$tip.label)]<-gen_i  
      
    }}
  
  
  if(!is.null(other_taxa)){
    for(i in 1:length(unique(other_taxa[,1]))){
      
      tax_i <- unique(other_taxa[,1])[i]  
      spp_i <- other_taxa[,2][which(other_taxa[,1]==tax_i)]
      mrca_i <- getMRCA(phy = phylogeny,
                             tip = which(phylogeny$tip.label %in% gsub(pattern = " ",replacement = "_", x = spp_i   ) )) 
      phylogeny$node.label[mrca_i-length(phylogeny$tip.label)]<-tax_i  
      
    }}
  
  
  return(phylogeny)  
  
  
  
}#end fx


#################################

#################################
#'Download the current BIEN database version and release date
#'
#'BIEN_metadata_database_version downloads the current version number and release date for the BIEN database.
#' @param ... Additional arguments passed to internal functions.
#' @return A data frame containing the current version number and release date for the BIEN database.
#' @family metadata functions
#' @examples \dontrun{
#' BIEN_metadata_database_version()}
#' @export
BIEN_metadata_database_version <- function(...){
  
  query <- "SELECT db_version, db_release_date FROM bien_metadata a JOIN (SELECT MAX(bien_metadata_id) as max_id FROM bien_metadata) AS b ON a.bien_metadata_id=b.max_id ;"
  .BIEN_sql(query, ...)
  
}


################################

################################
#'Check for differing records between old and new dataframes.
#'
#'BIEN_metadata_match_data compares old and new dataframes, and can check whether they are identical or be used to select rows that are unique to the old or new versions.
#' @param old A dataframe that is to be compared to a (typically) newer dataframe.
#' @param new A dataframe that is to be compared to a (typically) older dataframe.
#' @param return What information should be returned?  Current options are: "identical" (Logical, are the two dataframes identical?), "additions" (numeric, which rows are new?), "deletions" (numeric, which rows are no longer present?), "logical" (logical, which elements of the old dataframe are in the new one?).
#' @return Logical of varying length (depending on choice of "return" parameter)
#' @note Since comparisons are done by row (except when using return="identical"), this function may fail to flag additions or deletions if they are exact duplicates of existing rows.
#' @family metadata functions
#' @examples \dontrun{
#' new<-BIEN_occurrence_species("Acer nigrum")
#' old<-new[-1:-4,]#simulate having an older dataset by removing four rows
#' BIEN_metadata_match_data(old,new,return="identical")
#' BIEN_metadata_match_data(old,new,return="additions")}
#' @export
BIEN_metadata_match_data <- function(old,
                                     new,
                                     return = "identical"){
  
  if(return %in% c("identical","logical","additions","deletions")){
    
    old <- apply(old,MARGIN = 1,FUN = toString)  
    new <- apply(new,MARGIN = 1,FUN = toString)
    elements <- is.element(new,old)    
    
    if(return == "logical"){
      elements <- is.element(new,old)
      return(elements)  
    }#returns TRUE where elements are in the old set, false where they are not
    
    if(return == "additions"){
      elements <- is.element(new,old)
      return(which(elements == FALSE))  
    }#returns index of new elements
    
    if(return == "deletions"){
      elements <- is.element(old,new)
      return(which(elements == FALSE))  
    }#returns index of deleted elements
    
    if(return == "identical"){
      return(identical(old,new))
    }#returns true if identical, false otherwise
  }else{message("Please specify either 'identical','logical','additions' or 'deletions' for the value of the return argument")}
  
}

################################


#'Generate citations for data extracted from BIEN.
#'
#'BIEN_metadata_citation guides a user through the proper documentation for data downloaded from the BIEN database.
#' @param dataframe A data.frame of occurrence data downloaded from the BIEN R package.
#' @param trait.dataframe A data.frame of trait data downloaded from the BIEN R package.
#' @param trait.mean.dataframe A data.frame of species mean trait data from the function BIEN_trait_mean.
#' @param bibtex_file Output file for writing bibtex citations.
#' @param acknowledgement_file Output file for writing acknowledgements.
#' @param ... Additional arguments passed to internal functions.
#' @return A list object containing information needed for data attribution.  Full information for herbaria is available at http://sweetgum.nybg.org/science/ih/
#' @examples \dontrun{
#' BIEN_metadata_citation()#If you are referencing the phylogeny or range maps.
#' Xanthium_data<-BIEN_occurrence_species("Xanthium strumarium")
#' citations<-BIEN_metadata_citation(dataframe=Xanthium_data)#If you are referencing occurrence data}
#' @family metadata functions
#' @export
BIEN_metadata_citation <- function(dataframe = NULL,
                                   trait.dataframe = NULL,
                                   trait.mean.dataframe = NULL,
                                   bibtex_file = NULL,
                                   acknowledgement_file = NULL,
                                   ...){
  
  
  BIEN_cite <- '@ARTICLE{Enquist_undated-aw, title  = "Botanical big data shows that plant diversity in the New World is driven by climatic-linked differences in evolutionary rates and 
  biotic exclusion", author = "Enquist, B J and Sandel, B and Boyle, B and Svenning, J-C and McGill, B J and Donoghue, J C and Hinchliff, C E and Jorgensen, P M and Kraft, N J B and Marcuse-Kubitza, A and Merow, C and Morueta-Holme, N and Peet, R K and Schildhauer, M and Spencer, N and Regetz, J and Simova, I and Smith, S A and Thiers, B and Violle, C and Wiser, S K and Andelman, S and Casler, N and Condit, R and Dolins, S and Guaderrama, D and Maitner, B S and Narro, M L and Ott, J E and Phillips, O and Sloat, L L and ter   Steege, H"}'
  BIEN_cite <- gsub(pattern = "\n", replacement = "", BIEN_cite)
  
  R_package_cite <- '@article{doi:10.1111/2041-210X.12861,
  author = {Maitner Brian S. and Boyle Brad and Casler Nathan and Condit Rick and Donoghue John and Duran Sandra M. and Guaderrama Daniel and Hinchliff Cody E. and Jorgensen Peter M. and Kraft Nathan J.B. and McGill Brian and Merow Cory and Morueta-Holme Naia and Peet Robert K. and Sandel Brody and Schildhauer Mark and Smith Stephen A. and Svenning Jens-Christian and Thiers Barbara and Violle Cyrille and Wiser Susan and Enquist Brian J.},
  title = {The bien r package: A tool to access the Botanical Information and Ecology Network (BIEN) database},
  journal = {Methods in Ecology and Evolution},
  volume = {9},
  number = {2},
  pages = {373-379},
  keywords = {biodiversity, community plot, ecoinformatics, functional traits, herbarium records, occurrence, phylogeny, plants, presence, R, range maps},
  doi = {10.1111/2041-210X.12861},
  url = {https://besjournals.onlinelibrary.wiley.com/doi/abs/10.1111/2041-210X.12861},
  eprint = {https://besjournals.onlinelibrary.wiley.com/doi/pdf/10.1111/2041-210X.12861},
  abstract = {Abstract There is an urgent need for large-scale botanical data to improve our understanding of community assembly, coexistence, biogeography, evolution, and many other fundamental biological processes. Understanding these processes is critical for predicting and handling human-biodiversity interactions and global change dynamics such as food and energy security, ecosystem services, climate change, and species invasions. The Botanical Information and Ecology Network (BIEN) database comprises an unprecedented wealth of cleaned and standardised botanical data, containing roughly 81 million occurrence records from c. 375,000 species, c. 915,000 trait observations across 28 traits from c. 93,000 species, and co-occurrence records from 110,000 ecological plots globally, as well as 100,000 range maps and 100 replicated phylogenies (each containing 81,274 species) for New World species. Here, we describe an r package that provides easy access to these data. The bien r package allows users to access the multiple types of data in the BIEN database. Functions in this package query the BIEN database by turning user inputs into optimised PostgreSQL functions. Function names follow a convention designed to make it easy to understand what each function does. We have also developed a protocol for providing customised citations and herbarium acknowledgements for data downloaded through the bien r package. The development of the BIEN database represents a significant achievement in biological data integration, cleaning and standardization. Likewise, the bien r package represents an important tool for open science that makes the BIEN database freely and easily accessible to everyone.}
}' 
  
  R_package_cite <- gsub(pattern = "\n",replacement = "",R_package_cite)
  
  
  
  
  
  if(!is.null(trait.dataframe)){
    trait.query<-paste("SELECT DISTINCT citation_bibtex,source_citation,source, url_source, access, project_pi, project_pi_contact FROM agg_traits 
                       WHERE id in (", paste(shQuote(as.integer(trait.dataframe$id), type = "sh"),collapse = ', '),") ;")
    trait.sources<-.BIEN_sql(trait.query, ...)}
  
  
  if(!is.null(trait.mean.dataframe)){
    
    ids<-paste(trait.mean.dataframe$ids,collapse = ",")
    ids<-unique(unlist(strsplit(x = ids,split = ",")))
    ids<-ids[which(ids!="NA")]
    
    trait.mean.query<-paste("SELECT DISTINCT citation_bibtex,source_citation,source, url_source, access, project_pi, project_pi_contact FROM agg_traits 
                            WHERE id in (", paste(shQuote(as.integer(ids), type = "sh"),collapse = ', '),") ;")
    
    trait.mean.sources <- .BIEN_sql(trait.mean.query, ...)
    #trait.mean.sources <- .BIEN_sql(trait.mean.query)
  }
  
  
  if(!is.null(trait.dataframe) & !is.null(trait.mean.dataframe)){
    trait.sources<- rbind(trait.sources,trait.mean.sources)
    trait.sources<-unique(trait.sources)
    
  }
  
  if(is.null(trait.dataframe) & !is.null(trait.mean.dataframe)){
    trait.sources <- trait.mean.sources
    
  }
  
  
  
  #########
  ##########
  #If an occurrence dataframe is supplied:
  
  if(!is.null(dataframe)){  
    
    datasources<-unique(dataframe$datasource_id[!is.na(dataframe$datasource_id)])
    
    query<-paste("WITH a AS (SELECT * FROM datasource where datasource_id in (", paste(shQuote(datasources, type = "sh"),collapse = ', '),")) 
                 SELECT * FROM datasource where datasource_id in (SELECT proximate_provider_datasource_id FROM a) OR datasource_id in (SELECT datasource_id FROM a) ;")
    
    sources<-.BIEN_sql(query, ...)
    
    citation<-list()
    citation[[1]]<-general<-"Public BIEN data are licensed via a CC-BY license.  Please see BIENdata.org for more information.
    The references in this list should be added to any publication using these data.  This is most easily done by specifying a bibtex_file and importing the bibtex formatted references into a reference manager.
    The acknowledgements in this list should be pasted into the acknowledgements of any resulting publications.
    Be sure to check for a 'data owners to contact' section in this list, as any authors listed there need to be contacted prior to publishing with their data."
    citation[[1]]<-gsub(pattern = "\n",replacement = "",citation[[1]])
    
    
    #Cleaning up the bibtex so that it loads properly into reference managers.  Better too many new lines than not enough...for some reason...
    
    dl_cites<-unique(sources$source_citation[which(!is.na(sources$source_citation))])
    if(!is.null(trait.dataframe)){dl_cites<-c(dl_cites,trait.sources$citation_bibtex)}
    dl_cites<-gsub(dl_cites,pattern = '"@',replacement = '@')
    dl_cites<-gsub(dl_cites,pattern = '" @',replacement = '@')
    dl_cites<-unique(dl_cites[which(!is.na(dl_cites))])
    
    citation[[2]]<-c(BIEN_cite,R_package_cite,dl_cites)
    citation[[2]]<-gsub(citation[[2]],pattern = "author", replacement = "\nauthor")
    citation[[2]]<-gsub(citation[[2]],pattern = "title", replacement = "\ntitle")
    citation[[2]]<-gsub(citation[[2]],pattern = "year", replacement = "\nyear")
    citation[[2]]<-gsub(citation[[2]],pattern = "organization", replacement = "\norganization")
    citation[[2]]<-gsub(citation[[2]],pattern = "address", replacement = "\naddress")
    citation[[2]]<-gsub(citation[[2]],pattern = "url", replacement = "\nurl")
    citation[[2]]<-gsub(citation[[2]],pattern = "journal", replacement = "\njournal")
    citation[[2]]<-gsub(citation[[2]],pattern = "note", replacement = "\nnote")
    citation[[2]]<-iconv(citation[[2]],to="ASCII//TRANSLIT")
    citation[[2]]<-gsub(citation[[2]],pattern = '\n}\"', replacement = '\n}')
    citation[[2]]<-gsub(citation[[2]],pattern = '\"\\\nurl', replacement = '\"\\url', fixed = TRUE)
    
    if(length(unique(sources$source_name[which(sources$is_herbarium==1)]))>0){
      citation[[3]]<-paste("We acknowledge the herbaria that contributed data to this work: ",
                           paste(unique(sources$source_name[which(sources$is_herbarium==1)])|>sort(),
                                 collapse = ", "),".",collapse = "",sep="")
    }
    if(length(unique(sources$source_name[which(sources$is_herbarium==1)]))==0){
      citation[[3]]<-data.frame()
    }
    
    citation[[4]]<-sources[which(sources$access_conditions=="contact authors"),]
    citation[[4]]<-citation[[4]][c('primary_contact_fullname','primary_contact_email','access_conditions','source_fullname','source_citation')]
    
    if(!is.null(trait.dataframe)){
      if(length(which(trait.sources$access=='public (notify the PIs)'))>0){
        
      ack_trait_sources<-trait.sources[which(trait.sources$access=='public (notify the PIs)'),]
      ack_trait_sources<-ack_trait_sources[c('project_pi','project_pi_contact','access','source_citation','citation_bibtex')]
      colnames(ack_trait_sources)<-c('primary_contact_fullname','primary_contact_email','access_conditions','source_fullname','source_citation')
      citation[[4]]<-rbind(citation[[4]],ack_trait_sources)
      }
    }
    
    
    names(citation)<-c("general information","references","acknowledgements","data owners to contact")
    
    
    #Write acknowledgements    
    if(nrow(citation[[4]])==0){citation[[4]]<-NULL}
    
    if(!is.null(acknowledgement_file)){
      
      if(length(unique(sources$source_name[which(sources$is_herbarium==1)]))>0){
        writeLines(text = citation$acknowledgements,con = acknowledgement_file)}else{
          message("No herbarium records found, not generating an herbarium acknowledgement file.")
          
        }  
      
    }
    
    #Write author contact warning and info        
    
    if("contact authors"%in%sources$access_conditions & is.null(trait.dataframe)){
      affected_datasource_id<-sources$datasource_id[which(sources$access_conditions=='contact authors')]
      n_affected_records<-length(which(dataframe$datasource_id%in%affected_datasource_id))
      pct_affected_records<-round(x =( n_affected_records/(length(which(!dataframe$datasource_id%in%affected_datasource_id))+n_affected_records))*100,digits = 2)
      
      n_affected_sources<-nrow(citation$`data owners to contact`)
      pct_affected_sources<-round(x = (n_affected_sources/nrow(sources))*100,digits = 2)
      
      message(paste("NOTE: You have references that require you to contact the data owners before publication.  This applies to ",
                    n_affected_records, " records (",pct_affected_records,"%) from ",n_affected_sources," sources (",pct_affected_sources,"%).",sep=""))
      
    }#if need to contact authors of a study
    
    if("contact authors"%in%sources$access_conditions & !is.null(trait.dataframe)){
      affected_datasource_id<-sources$datasource_id[which(sources$access_conditions=='contact authors')]
      #using author to identify datasource here.  Not perfect, but should generally work
      affected_trait__datasource_id<-trait.sources$project_pi_contact[which(trait.sources$access=='public (notify the PIs)')]
      
      n_affected_records<-length(which(dataframe$datasource_id%in%affected_datasource_id))+length(which(trait.dataframe$access%in%'public (notify the PIs)'))
      
      pct_affected_records<-round(x =( n_affected_records/(nrow(dataframe)+nrow(trait.dataframe) ))*100,digits = 2)
      
      n_affected_sources<-nrow(citation$`data owners to contact`)
      
      pct_affected_sources<-round(x = (n_affected_sources/(nrow(sources)+nrow(trait.sources)))*100,digits = 2)
      
      message(paste("NOTE: You have references that require you to contact the data owners before publication.  This applies to ",
                    n_affected_records, " records (",pct_affected_records,"%) from ",n_affected_sources," sources (",pct_affected_sources,"%).",sep=""))
      
    }#if need to contact authors of a study
    
  }  #if a dataframe is supplied
  
  ##########  
  
  #########  
  #If no dataframe or trait dataframe supplied  
  
  
  if(is.null(dataframe) & is.null(trait.dataframe)){
    
    citation<-list()
    citation[[1]]<-general<-"Public BIEN data are licensed via a CC-BY license.  Please see BIENdata.org for more information.
    The references in this list should be added to any publication using these data.  This is most easily done by specifying a bibtex_file and importing the bibtex formatted references into a reference manager.
    The acknowledgements in this list should be pasted into the acknowledgements of any resulting publications.
    Be sure to check for a 'data owners to contact' section in this list, as any authors listed there need to be contacted prior to publishing with their data."
    citation[[1]]<-gsub(pattern = "\n",replacement = "",citation[[1]])
    
    
    #Cleaning up the bibtex so that it loads properly into reference managers.  Better too many new lines than not enough...for some reason...
    citation[[2]]<-c(BIEN_cite,R_package_cite)
    citation[[2]]<-gsub(citation[[2]],pattern = "author", replacement = "\nauthor")
    citation[[2]]<-gsub(citation[[2]],pattern = "title", replacement = "\ntitle")
    citation[[2]]<-gsub(citation[[2]],pattern = "year", replacement = "\nyear")
    citation[[2]]<-gsub(citation[[2]],pattern = "organization", replacement = "\norganization")
    citation[[2]]<-gsub(citation[[2]],pattern = "address", replacement = "\naddress")
    citation[[2]]<-gsub(citation[[2]],pattern = "url", replacement = "\nurl")
    citation[[2]]<-gsub(citation[[2]],pattern = "journal", replacement = "\njournal")
    citation[[2]]<-gsub(citation[[2]],pattern = "note", replacement = "\nnote")
    citation[[2]]<-iconv(citation[[2]],to="ASCII//TRANSLIT")
    names(citation)<-c("general information","references")
    
  }#if dataframe is null
  
  #######
  #####  
  
  
  
  if((!is.null(trait.dataframe) |!is.null(trait.mean.dataframe)) & is.null(dataframe)){  
    citation<-list()
    citation[[1]]<-general<-"Public BIEN data are licensed via a CC-BY license.  Please see BIENdata.org for more information.
    The references in this list should be added to any publication using these data.  This is most easily done by specifying a bibtex_file and importing the bibtex formatted references into a reference manager.
    The acknowledgements in this list should be pasted into the acknowledgements of any resulting publications.
    Be sure to check for a 'data owners to contact' section in this list, as any authors listed there need to be contacted prior to publishing with their data."
    citation[[1]]<-gsub(pattern = "\n",replacement = "",citation[[1]])
    
    
    #Cleaning up the bibtex so that it loads properly into reference managers.  Better too many new lines than not enough...for some reason...
    if(!is.null(trait.dataframe)| !is.null(trait.mean.dataframe)){dl_cites<-c(trait.sources$citation_bibtex)}
    dl_cites<-gsub(dl_cites,pattern = '"@',replacement = '@')
    dl_cites<-gsub(dl_cites,pattern = '" @',replacement = '@')
    dl_cites<-unique(dl_cites[which(!is.na(dl_cites))])
    
    citation[[2]]<-c(BIEN_cite,R_package_cite,dl_cites)
    citation[[2]]<-gsub(citation[[2]],pattern = "author", replacement = "\nauthor")
    citation[[2]]<-gsub(citation[[2]],pattern = "title", replacement = "\ntitle")
    citation[[2]]<-gsub(citation[[2]],pattern = "year", replacement = "\nyear")
    citation[[2]]<-gsub(citation[[2]],pattern = "organization", replacement = "\norganization")
    citation[[2]]<-gsub(citation[[2]],pattern = "address", replacement = "\naddress")
    citation[[2]]<-gsub(citation[[2]],pattern = "url", replacement = "\nurl")
    citation[[2]]<-gsub(citation[[2]],pattern = "journal", replacement = "\njournal")
    citation[[2]]<-gsub(citation[[2]],pattern = "note", replacement = "\nnote")
    citation[[2]]<-iconv(citation[[2]],to="ASCII//TRANSLIT")
    citation[[2]]<-gsub(citation[[2]],pattern = '\n}\"', replacement = '\n}')
    citation[[2]]<-gsub(citation[[2]],pattern = '\"\\\nurl', replacement = '\"\\url', fixed = TRUE)
    citation[[3]]<-data.frame()
    ack_trait_sources<-trait.sources[which(trait.sources$access=='public (notify the PIs)'),]
    ack_trait_sources<-ack_trait_sources[c('project_pi','project_pi_contact','access','source_citation','citation_bibtex')]
    citation[[4]]<-ack_trait_sources
    colnames(citation[[4]])<-c('primary_contact_fullname','primary_contact_email','access_conditions','source_fullname','source_citation')
    
    
    names(citation)<-c("general information","references","acknowledgements","data owners to contact")
    
    
    #Write acknowledgements    
    
    #add code here if we decide to do trait acknowledgements
    
    
    if(nrow(citation[[4]])==0){citation[[4]]<-NULL}
    
    #Write author contact warning and info        
    
    if('public (notify the PIs)'%in%trait.sources$access){
      affected_trait__datasource_id<-trait.sources$project_pi_contact[which(trait.sources$access=='public (notify the PIs)')]
      
      n_affected_records<-length(which(trait.dataframe$access%in%'public (notify the PIs)'))
      
      pct_affected_records<-round(x =( n_affected_records/(nrow(trait.dataframe) ))*100,digits = 2)
      
      n_affected_sources<-nrow(citation$`data owners to contact`)
      
      pct_affected_sources<-round(x = (n_affected_sources/(nrow(trait.sources)))*100,digits = 2)
      
      message(paste("NOTE: You have references that require you to contact the data owners before publication.  This applies to ",
                    n_affected_records, " records (",pct_affected_records,"%) from ",n_affected_sources," sources (",pct_affected_sources,"%).",sep=""))
      
    }#if need to contact authors of a study
    
  }  #if only a trait dataframe is supplied  
  
  
  
  
  #######    
  ######  
  #Write bibtex output  
  if(!is.null(bibtex_file)){
    
    writeLines(text = citation[[2]],con=bibtex_file)  
    
    
  }
  
  #Return the citation list  
  
  return(citation)    
  
  }

#####################


#'List political divisions and associated geonames codes.
#'
#'BIEN_metadata_list_political_names downloads country, state, and county names and associated codes used by BIEN.
#' @param ... Additional arguments passed to internal functions.
#' @return A dataframe containing political division names and their associated codes.
#' @note Political names and codes follow http://www.geonames.org/
#' @examples \dontrun{
#' BIEN_metadata_list_political_names()}
#' @family metadata functions
#' @export
BIEN_metadata_list_political_names <- function(...){
  
  query<-'SELECT country,country_iso, state_province, state_province_ascii,state_province_code AS "state_code",
  county_parish,county_parish_ascii,county_parish_code AS "county_code" FROM county_parish ;'
  
  .BIEN_sql(query, ...)
  
}

####################

#'Download data dictionaries
#'
#'BIEN_metadata_data_dictionaries downloads the data dictionaries for the BIEN database.
#' @param ... Additional arguments passed to internal functions.
#' @return A list containing data.frames containing different data dictionaries.
#' "Tables" contains information on the contents of tables.
#' "Columns" contains information on the contents of columns within tables.
#' "Values" contains information on some of the values that are used to populated columns, particularly where these may be unclear.
#' "rbien" contains information on some of the values returned by the BIEN R package.
#' @family metadata functions
#' @examples \dontrun{
#' BIEN_metadata_data_dictionaries()}
#' @keywords internal
BIEN_metadata_data_dictionaries <- function(...){
  
  tables <- .BIEN_sql("SELECT * from data_dictionary_tables",...)
  columns <- .BIEN_sql("SELECT * from data_dictionary_columns",...)
  values <- .BIEN_sql("SELECT * from data_dictionary_values",...)
  rbien <- .BIEN_sql("SELECT * from data_dictionary_rbien",...)
  
  
  
  data_dictionary <- list(tables =tables,
                          columns = columns,
                          values = values,
                          rbien = rbien)
  
  
  return(data_dictionary)
  
}



################################

############################
#Stem functions

#'Extract stem data for specified species from BIEN
#'
#'BIEN_stem_species downloads occurrence records for specific species from the BIEN database.
#' @param species A single species, or a vector of species.  Genus and species should be separated by a space. Genus should be capitalized.
#' @template stem
#' @return Dataframe containing stem data for the specified species.
#' @note Setting either "cultivated" or "native.status" to TRUE will significantly slow the speed of a query.
#' @examples \dontrun{
#' BIEN_stem_species("Abies amabilis")
#' species_vector<-c("Abies amabilis", "Acer nigrum")
#' BIEN_stem_species(species_vector)
#' BIEN_stem_species(species_vector,all.taxonomy = TRUE)}
#' @family stem functions
#' @export
BIEN_stem_species <- function(species,
                              cultivated = FALSE,
                              new.world = NULL,
                              all.taxonomy = FALSE,
                              native.status = FALSE,
                              natives.only = TRUE,
                              political.boundaries = FALSE,
                              collection.info = FALSE,
                              all.metadata = FALSE,
                              ...){

  .is_log(all.metadata)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(species)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  
  #set conditions for query
    cultivated_ <- .cultivated_check_stem(cultivated)
    newworld_ <- .newworld_check_stem(new.world)
    taxonomy_ <- .taxonomy_check_stem(all.taxonomy)
    native_ <- .native_check_stem(native.status)
    natives_ <- .natives_check_stem(natives.only)
    political_ <- .political_check_stem(political.boundaries)
    collection_ <- .collection_check_stem(collection.info)
    vfoi_ <- .vfoi_check_stem(native.status,cultivated,natives.only,collection.info)
    md_ <- .md_check_stem(all.metadata)
  
  # set the query
    query <- paste("SELECT analytical_stem.scrubbed_species_binomial",taxonomy_$select,native_$select,political_$select," ,analytical_stem.latitude, 
                        analytical_stem.longitude,analytical_stem.date_collected, analytical_stem.relative_x_m, analytical_stem.relative_y_m, 
                        analytical_stem.taxonobservation_id,analytical_stem.stem_code, analytical_stem.stem_dbh_cm, analytical_stem.stem_height_m, 
                        plot_metadata.dataset,plot_metadata.datasource,plot_metadata.dataowner,analytical_stem.custodial_institution_codes,
                        analytical_stem.collection_code,analytical_stem.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                   FROM 
                      (SELECT * FROM analytical_stem WHERE scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ")) AS analytical_stem 
                      JOIN plot_metadata ON 
                      (analytical_stem.plot_metadata_id= plot_metadata.plot_metadata_id)",
                      vfoi_$join ," 
                   WHERE analytical_stem.scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ")",
                      cultivated_$query,newworld_$query,natives_$query,  "AND analytical_stem.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                      AND (analytical_stem.is_geovalid = 1) AND (analytical_stem.georef_protocol is NULL OR analytical_stem.georef_protocol<>'county centroid') 
                      AND (analytical_stem.is_centroid IS NULL OR analytical_stem.is_centroid=0) ;")
  
  return(.BIEN_sql(query, ...))
  
  }


#######################

#'Extract stem data for specified families from BIEN
#'
#'BIEN_stem_family downloads occurrence records for specific families from the BIEN database.
#' @param family A single family, or a vector of families. Families should be capitalized.
#' @template stem
#' @return Dataframe containing stem data for the specified families.
#' @note Setting either "cultivated" or "native.status" to TRUE will significantly slow the speed of a query.
#' @examples \dontrun{
#' BIEN_stem_family(family = "Marantaceae")
#' family_vector<-c("Marantaceae", "Buxaceae")
#' BIEN_stem_family(family = family_vector)
#' BIEN_stem_family(family = family_vector, all.taxonomy = TRUE, native.status = TRUE)}
#' @family stem functions
#' @export
BIEN_stem_family <- function(family,
                             cultivated = FALSE,
                             new.world = NULL,
                             all.taxonomy = FALSE,
                             native.status = FALSE,
                             natives.only = TRUE,
                             political.boundaries = FALSE,
                             collection.info = FALSE,
                             all.metadata = FALSE,
                             ...){

  .is_log(all.metadata)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(family)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)

  #set conditions for query
  cultivated_<-.cultivated_check_stem(cultivated)
  newworld_<-.newworld_check_stem(new.world)
  taxonomy_<-.taxonomy_check_stem(all.taxonomy)
  native_<-.native_check_stem(native.status)
  natives_<-.natives_check_stem(natives.only)
  political_<-.political_check_stem(political.boundaries)
  collection_<-.collection_check_stem(collection.info)
  vfoi_<-.vfoi_check_stem(native.status,cultivated,natives.only,collection.info)
  md_<-.md_check_stem(all.metadata)
  
  # set the query
  query <- paste("SELECT analytical_stem.scrubbed_family, analytical_stem.scrubbed_genus,analytical_stem.scrubbed_species_binomial",taxonomy_$select,native_$select,
                      political_$select,", analytical_stem.latitude, analytical_stem.longitude,analytical_stem.date_collected,analytical_stem.relative_x_m, 
                      analytical_stem.relative_y_m, analytical_stem.taxonobservation_id, analytical_stem.stem_code, analytical_stem.stem_dbh_cm, analytical_stem.stem_height_m, 
                      plot_metadata.dataset,plot_metadata.datasource,plot_metadata.dataowner,analytical_stem.custodial_institution_codes,
                      analytical_stem.collection_code,analytical_stem.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM analytical_stem WHERE scrubbed_family in (", paste(shQuote(family, type = "sh"),collapse = ', '), ")) AS analytical_stem 
                 JOIN plot_metadata ON 
                 (analytical_stem.plot_metadata_id= plot_metadata.plot_metadata_id)",
                 vfoi_$join ," 
                 WHERE analytical_stem.scrubbed_family in (", paste(shQuote(family, type = "sh"),collapse = ', '), ")",
                    cultivated_$query,newworld_$query,natives_$query,  "AND analytical_stem.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                    AND (analytical_stem.is_geovalid = 1) AND (analytical_stem.georef_protocol is NULL OR analytical_stem.georef_protocol<>'county centroid') 
                    AND (analytical_stem.is_centroid IS NULL OR analytical_stem.is_centroid=0) ;")
  
  return(.BIEN_sql(query, ...))
  
  }

#######################################

#'Extract stem data for specified genera from BIEN
#'
#'BIEN_stem_genus downloads occurrence records for specific genera from the BIEN database.
#' @param genus A single genus, or a vector of genera. Genera should be capitalized.
#' @template stem
#' @return Dataframe containing stem data for the specified genera.
#' @note Setting either "cultivated" or "native.status" to TRUE will significantly slow the speed of a query.
#' @examples \dontrun{
#' BIEN_stem_genus(genus = "Tovomita")
#' genus_vector<-c("Tovomita", "Myrcia")
#' BIEN_stem_genus(genus = genus_vector)
#' BIEN_stem_genus(genus = genus_vector, all.taxonomy = TRUE)}
#' @family stem functions
#' @export
BIEN_stem_genus <- function(genus,
                           cultivated = FALSE,
                            new.world = NULL,
                            all.taxonomy = FALSE,
                            native.status = FALSE,
                            natives.only = TRUE,
                            political.boundaries = FALSE,
                            collection.info = FALSE,
                            all.metadata = FALSE,
                            ...){

  .is_log(all.metadata)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(genus)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  
  #set conditions for query
    cultivated_ <- .cultivated_check_stem(cultivated)
    newworld_ <- .newworld_check_stem(new.world)
    taxonomy_ <- .taxonomy_check_stem(all.taxonomy)
    native_ <- .native_check_stem(native.status)
    natives_ <- .natives_check_stem(natives.only)
    political_ <- .political_check_stem(political.boundaries)
    collection_ <- .collection_check_stem(collection.info)
    vfoi_ <- .vfoi_check_stem(native.status,cultivated,natives.only,collection.info)
    md_ <- .md_check_stem(all.metadata)
  
  # set the query
  query <- paste("SELECT analytical_stem.scrubbed_genus,analytical_stem.scrubbed_species_binomial",taxonomy_$select,native_$select,political_$select," , 
                      analytical_stem.latitude, analytical_stem.longitude,analytical_stem.date_collected, analytical_stem.relative_x_m, analytical_stem.relative_y_m, 
                      analytical_stem.taxonobservation_id, analytical_stem.stem_code, analytical_stem.stem_dbh_cm, analytical_stem.stem_height_m, plot_metadata.dataset,
                      plot_metadata.datasource,plot_metadata.dataowner, analytical_stem.custodial_institution_codes, analytical_stem.collection_code, 
                      analytical_stem.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM analytical_stem WHERE scrubbed_genus in (", paste(shQuote(genus, type = "sh"),collapse = ', '), ")) AS analytical_stem 
                 JOIN plot_metadata ON 
                    (analytical_stem.plot_metadata_id= plot_metadata.plot_metadata_id)",
                 vfoi_$join ," 
                 WHERE analytical_stem.scrubbed_genus in (", paste(shQuote(genus, type = "sh"),collapse = ', '), ")",
                    cultivated_$query,newworld_$query,natives_$query, "AND analytical_stem.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                    AND (analytical_stem.is_geovalid = 1) AND (analytical_stem.georef_protocol is NULL OR analytical_stem.georef_protocol<>'county centroid') 
                    AND (analytical_stem.is_centroid IS NULL OR analytical_stem.is_centroid=0) ;")
  
  return(.BIEN_sql(query, ...))
  
  }

###########

#'Extract stem data for a given datasource from BIEN
#'
#'BIEN_stem_datasource downloads occurrence records for specific datasources from the BIEN database.
#' @param datasource A single datasource, or a vector of datasources.
#' @template stem
#' @return Dataframe containing stem data for the specified datasource.
#' @note Setting either "cultivated" or "native.status" to TRUE will significantly slow the speed of a query.
#' @note #' @note For a list of available datasources, use \code{\link{BIEN_plot_list_datasource}}.
#' @examples \dontrun{
#' BIEN_stem_datasource(datasource = "SALVIAS")}
#' @family stem functions
#' @export
BIEN_stem_datasource <- function(datasource,
                                 cultivated = FALSE,
                                 new.world = NULL,
                                 all.taxonomy = FALSE,
                                 native.status = FALSE,
                                 natives.only = TRUE,
                                 political.boundaries = FALSE,
                                 collection.info = FALSE,
                                 all.metadata = FALSE,
                                 ...){

  .is_log(all.metadata)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(datasource)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
    
  #set conditions for query
    cultivated_<-.cultivated_check_stem(cultivated)
    newworld_<-.newworld_check_stem(new.world)
    taxonomy_<-.taxonomy_check_stem(all.taxonomy)
    native_<-.native_check_stem(native.status)
    natives_<-.natives_check_stem(natives.only)
    political_<-.political_check_stem(political.boundaries)
    collection_<-.collection_check_stem(collection.info)
    vfoi_<-.vfoi_check_stem(native.status,cultivated,natives.only,collection.info)
    md_<-.md_check_stem(all.metadata)
  
  # set the query
  query <- paste("SELECT analytical_stem.plot_name,analytical_stem.subplot, analytical_stem.elevation_m, analytical_stem.plot_area_ha,analytical_stem.sampling_protocol,
                      analytical_stem.recorded_by,analytical_stem.scrubbed_species_binomial",taxonomy_$select,native_$select,political_$select," ,analytical_stem.latitude, 
                      analytical_stem.longitude,analytical_stem.date_collected,analytical_stem.relative_x_m, analytical_stem.relative_y_m, analytical_stem.taxonobservation_id, 
                      analytical_stem.stem_code, analytical_stem.stem_dbh_cm, analytical_stem.stem_height_m, plot_metadata.dataset,plot_metadata.datasource,plot_metadata.dataowner,
                      analytical_stem.custodial_institution_codes, analytical_stem.collection_code,analytical_stem.datasource_id",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM analytical_stem WHERE datasource in (", paste(shQuote(datasource, type = "sh"),collapse = ', '), ")) AS analytical_stem 
                 JOIN plot_metadata ON 
                    (analytical_stem.plot_metadata_id= plot_metadata.plot_metadata_id)",
                 vfoi_$join ," 
                 WHERE analytical_stem.datasource in (", paste(shQuote(datasource, type = "sh"),collapse = ', '), ")",
                    cultivated_$query,newworld_$query,native_$query,  "AND analytical_stem.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                    AND (analytical_stem.is_geovalid = 1) AND (analytical_stem.georef_protocol is NULL OR analytical_stem.georef_protocol<>'county centroid') 
                    AND (analytical_stem.is_centroid IS NULL OR analytical_stem.is_centroid=0) ;")
  
  return(.BIEN_sql(query, ...))
  
  }


##########################


#'Download stem data using a specified sampling protocol.
#'
#'BIEN_stem_sampling_protocol downloads plot-based stem data using a specified sampling protocol.
#' @param sampling_protocol A sampling protocol or vector of sampling protocols. See \code{\link{BIEN_plot_list_sampling_protocols}} for options.
#' @template stem
#' @return A dataframe containing all data from the specified sampling protocol.
#' @examples \dontrun{
#' BIEN_stem_sampling_protocol("Point-intercept")}
#' @family stem functions
#' @export
BIEN_stem_sampling_protocol <- function(sampling_protocol,
                                        cultivated = FALSE,
                                        new.world = NULL,
                                        all.taxonomy = FALSE,
                                        native.status = FALSE,
                                        natives.only = TRUE,
                                        political.boundaries = FALSE,
                                        collection.info = FALSE,
                                        all.metadata = FALSE,
                                        ...){

  .is_log(all.metadata)
  .is_log(cultivated)
  .is_log_or_null(new.world)
  .is_log(all.taxonomy)
  .is_char(sampling_protocol)
  .is_log(native.status)
  .is_log(natives.only)
  .is_log(political.boundaries)
  .is_log(collection.info)
  
  #set conditions for query
    cultivated_<-.cultivated_check_stem(cultivated)
    newworld_<-.newworld_check_stem(new.world)
    taxonomy_<-.taxonomy_check_stem(all.taxonomy)
    native_<-.native_check_stem(native.status)
    natives_<-.natives_check_stem(natives.only)
    political_<-.political_check_stem(political.boundaries)
    collection_<-.collection_check_stem(collection.info)
    vfoi_<-.vfoi_check_stem(native.status,cultivated,natives.only,collection.info)
    md_<-.md_check_stem(all.metadata)
  
  # set the query
  query <- paste("SELECT analytical_stem.scrubbed_species_binomial",taxonomy_$select,native_$select,political_$select," ,analytical_stem.latitude, analytical_stem.longitude,analytical_stem.date_collected,
                    analytical_stem.relative_x_m, analytical_stem.relative_y_m, analytical_stem.taxonobservation_id,analytical_stem.stem_code, analytical_stem.stem_dbh_cm, analytical_stem.stem_height_m, 
                    plot_metadata.dataset,plot_metadata.datasource,plot_metadata.dataowner,analytical_stem.custodial_institution_codes,
                    analytical_stem.collection_code,analytical_stem.datasource_id,view_full_occurrence_individual.plot_name,view_full_occurrence_individual.subplot, 
                    view_full_occurrence_individual.elevation_m, view_full_occurrence_individual.plot_area_ha, 
                    view_full_occurrence_individual.sampling_protocol,view_full_occurrence_individual.recorded_by, view_full_occurrence_individual.individual_count",collection_$select,cultivated_$select,newworld_$select,md_$select,"
                 FROM 
                    (SELECT * FROM analytical_stem WHERE sampling_protocol in (", paste(shQuote(sampling_protocol, type = "sh"),collapse = ', '), ")) AS analytical_stem 
                 JOIN plot_metadata ON 
                    (analytical_stem.plot_metadata_id= plot_metadata.plot_metadata_id)",
                 vfoi_$join ," 
                 WHERE analytical_stem.sampling_protocol in (", paste(shQuote(sampling_protocol, type = "sh"),collapse = ', '), ")",
                    cultivated_$query,newworld_$query,natives_$query,  "AND analytical_stem.higher_plant_group NOT IN ('Algae','Bacteria','Fungi') 
                    AND (analytical_stem.is_geovalid = 1) AND (analytical_stem.georef_protocol is NULL OR analytical_stem.georef_protocol<>'county centroid') 
                    AND (analytical_stem.is_centroid IS NULL OR analytical_stem.is_centroid=0) ;")
  
  return(.BIEN_sql(query, ...))
  
}

#######################################

#Tests for BIEN package

#Normal testing
schema <- "public"

#List tests

test_that("List functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  expect_bien_class(BIEN_list_all(schema = schema), "data.frame")
  
  expect_bien_class(BIEN_list_country("Bahamas",
                                      schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_list_county(country = "United States",
                                     state = "Michigan",
                                     county = "Kent",
                                     cultivated = TRUE,
                                     new.world = FALSE,
                                     schema=schema),
                    "data.frame")
  
  expect_bien_class(BIEN_list_state(country = "United States",
                                    state = "Rhode Island",
                                    schema = schema),
                    "data.frame")

})

#Metadata tests
test_that("Metadata functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  expect_bien_class(BIEN_metadata_database_version(schema = schema),
                    "data.frame")
  
})

test_that("Metadata_citation function returns a list",{
  
  skip_if_bien_unavailable()
  
  expect_bien_class(BIEN_metadata_citation(),
                    "list")
  
})


#Occurrence tests
# test_that("postGIS Occurrence functions return a dataframe",{
#   
#   skip_if_offline()
#   
#   skip_on_cran()
#   
#   expect_that(BIEN_occurrence_box(min.lat = 32.9,
#                                   max.lat = 33,
#                                   min.long = -114,
#                                   max.long = -113.9,
#                                   cultivated = TRUE,
#                                   new.world = FALSE,
#                                   all.taxonomy = T,
#                                   native.status = T,
#                                   natives.only = T,
#                                   observation.type = T,
#                                   political.boundaries = T,
#                                   schema=schema),
#               is_a("data.frame"))
# 
# 
# })



#Occurrence tests
test_that("Occurrence functions return a dataframe",{
  
 skip_if_bien_unavailable()
  
 skip_on_cran()
  

 expect_bien_class(BIEN_occurrence_country("Bahamas",
                                           schema = schema,
                                           collection.info = T,
                                           limit = 1),
                   "data.frame")
 
 expect_bien_class(BIEN_occurrence_county(country = "United States",
                                          state = "Arizona",
                                          county = "Pima",
                                          limit = 1,
                                          schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_occurrence_family(family = "Cactaceae",
                                          limit = 1,
                                          schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_occurrence_genus(genus = "Xanthium",
                                         schema = schema,
                                         limit = 10),
                   "data.frame")
 
 expect_bien_class(BIEN_occurrence_species(species = "Xanthium strumarium",
                                           cultivated = T,
                                           new.world = F,
                                           all.taxonomy = T,
                                           native.status = T,
                                           observation.type = T,
                                           political.boundaries = T,
                                           limit = 1,
                                           schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_occurrence_state(country = "United States",
                                         state = "Rhode Island",
                                         schema = schema,
                                         limit = 10),
                   "data.frame")
 
 # expect_that(BIEN_occurrence_sf( BIEN_ranges_load_species(species = "Aa argyrolepis"),
 #                                              limit = 10),
 #             is_a("data.frame"))
 
 # expect_that(BIEN_occurrence_records_per_species(),
 #             is_a("data.frame"))
 
 })



#Phylogeny
test_that("Phylogeny functions return a phylogeny",{
  
  skip_if_bien_unavailable()
  
  expect_bien_class(BIEN_phylogeny_complete(n_phylogenies = 2,
                                            schema = schema),
                    "multiPhylo")
  
  expect_bien_class(BIEN_phylogeny_conservative(schema = schema),
                    "phylo")
  
})

# Plot
test_that("Plot functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  skip_on_cran()
  
 expect_bien_class(BIEN_plot_country("Puerto Rico",
                                     cultivated = T,
                                     new.world = F,
                                     all.taxonomy = T,
                                     native.status = T,
                                     political.boundaries = T,
                                     all.metadata = T,
                                     schema = schema,
                                     limit = 1),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_dataset(dataset = "SALVIAS",
                                     schema = schema,
                                     limit = 1),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_datasource("test",
                                        schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_list_datasource(schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_list_sampling_protocols(schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_metadata(schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_name("test",
                                  schema = schema,
                                  collection.info = T),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_sampling_protocol(sampling_protocol = "test",
                                               natives.only = T,
                                               schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_plot_state(country = "United States",
                                   state = "Michigan",
                                   limit = 1,
                                   schema = schema),
                   "data.frame")
 
 })

# Ranges

test_that("Ranges functions return an sf",{
  
  skip_if_bien_unavailable()
  
  expect_bien_class(BIEN_ranges_load_species(species = "Abies amabilis",
                                             schema = schema),
                    "sf")
  
})

# Stem

test_that("Stem functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  skip_on_cran()
  
 expect_bien_class(BIEN_stem_species(species = "Abies amabilis",
                                     natives.only = T,
                                     native.status = T,
                                     collection.info = T,
                                     schema = schema),
                   "data.frame")
 
 expect_bien_class(BIEN_stem_genus(genus = "Abies",
                                   natives.only = T,
                                   native.status = T,
                                   schema = schema,
                                   limit = 1),
                   "data.frame")
 
 expect_bien_class(BIEN_stem_family(family = "Selaginellaceae",
                                    natives.only = T,
                                    native.status = T,
                                    schema = schema),
                   "data.frame")

})

# Taxonomy
test_that("Taxonomy functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  
  expect_bien_class(BIEN_taxonomy_family("Cactaceae",
                                         schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_taxonomy_genus("Xanthium",
                                        schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_taxonomy_species("Xanthium strumarium",
                                          schema = schema),
                    "data.frame")
  
})

#trait

test_that("Trait functions return a dataframe",{
  
  skip_if_bien_unavailable()
  
  
  expect_bien_class(BIEN_trait_family("Cactaceae",
                                      schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_genus("Xanthium",
                                     schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_list(schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_mean(species = "Xanthium strumarium",
                                    trait = "leaf dry mass per leaf fresh mass",
                                    schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_species("Xanthium strumarium",
                                       all.taxonomy = TRUE,
                                       political.boundaries = TRUE,
                                       schema = schema),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_trait("leaf dry mass per leaf fresh mass",
                                     schema = schema,
                                     limit = 1),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_traitbyfamily(trait = "leaf dry mass per leaf fresh mass",
                                             family = "Asteraceae",
                                             schema = schema,
                                             limit = 1),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_traitbygenus(trait = "leaf dry mass per leaf fresh mass",
                                            genus = "Xanthium",
                                            schema = schema,
                                            limit = 1),
                    "data.frame")
  
  expect_bien_class(BIEN_trait_traitbyspecies(trait = "leaf dry mass per leaf fresh mass",
                                              species = "Xanthium strumarium",
                                              schema = schema,
                                              limit = 1),
                    "data.frame")
  
  #expect_that(BIEN_trait_traits_per_species(schema=schema) ,is_a("data.frame"))
  
})

#Internal functions

######################

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.cultivated_check<-function(cultivated){

  if(!cultivated){

    query <- "AND (is_cultivated_observation = 0 OR is_cultivated_observation IS NULL) AND is_location_cultivated IS NULL"
    select <- ""

  }else{

    query <- ""
    select <- ",is_cultivated_observation,is_cultivated_in_region, is_location_cultivated"

  }
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query","select")
  
  return(output)  
  
}


##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.newworld_check <- function(new.world){
  
  if(is.null(new.world)){
    query <- ""
    select <- ""
    output <- as.data.frame(cbind(query,select),stringsAsFactors = FALSE)  
    colnames(output) <- c("query", "select")
    return(output)  
    
  }
  
  if(!new.world){
    query <- "AND is_new_world = 0 "    
    select <- ", is_new_world"}
  
  if(new.world){
    query <- "AND is_new_world = 1 "    
    select <- ", is_new_world"}
  
  
  
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query", "select")
  
  return(output)  
  
}


##

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.geovalid_check<-function(only.geovalid){

  if(!only.geovalid){

    query <- ""
    select <- ",is_geovalid"

  }else{

    query <- "AND is_geovalid = 1"
    select <- ""

  }
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query", "select")
  
  return(output)  
  
}


##


#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.taxonomy_check <- function(all.taxonomy){

  if(!all.taxonomy){

    select <- ""

  }else{

    select <- ", verbatim_family,verbatim_scientific_name,family_matched,name_matched,name_matched_author,higher_plant_group,scrubbed_taxonomic_status,scrubbed_family,scrubbed_author"

  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
  
}  

##

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.native_check <- function(native.status){
  if(!native.status){
    select <- ""
  }else{
    select <- ",native_status, native_status_reason,native_status_sources,is_introduced,native_status_country,native_status_state_province,native_status_county_parish"
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}   

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.observation_check <- function(observation.type){
  if(!observation.type){
    query <- "AND observation_type IN ('plot','specimen','literature','checklist')"
    select <- ""
  }else{
    query <- ""
    select <- ",observation_type"
  }
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query","select")
  
  return(output)  
  
}   

##

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.political_check <- function(political.boundaries){
  if(!political.boundaries){
    select <- ""
  }else{
    select <- ", country,state_province,county,locality,elevation_m"
  }
  
  
  output <- as.data.frame(cbind(select),stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}   

##

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.natives_check <- function(natives.only){
  if(!natives.only){
    query <- ""
  }else{
    query <- "AND (is_introduced=0 OR is_introduced IS NULL) "
  }  
  
  output <- as.data.frame(cbind(query), stringsAsFactors = FALSE)  
  colnames(output) <- c("query")
  
  return(output)  
  
}   

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.collection_check <- function(collection.info){
  if(!collection.info){
    select <- ""
  }else{
    select <- ",catalog_number, recorded_by, record_number, date_collected, identified_by, date_identified, identification_remarks  "
  }  
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
} 


##########################
#Traits

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.political_check_traits <- function(political.boundaries){

if(political.boundaries){

  select <- ", region, country, state_province, locality_description"

}else{

  select <- ""  
  
}

  output <- as.data.frame(cbind(select),stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}   
  
##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.taxonomy_check_traits <- function(all.taxonomy){
  
if(all.taxonomy){

  select <- ", verbatim_family, verbatim_scientific_name, name_submitted, family_matched, name_matched, name_matched_author, 
    higher_plant_group, tnrs_warning, matched_taxonomic_status, scrubbed_taxonomic_status, scrubbed_family, scrubbed_genus, 
    scrubbed_specific_epithet, scrubbed_taxon_name_no_author, scrubbed_taxon_canonical, 
    scrubbed_author, scrubbed_taxon_name_with_author, scrubbed_species_binomial_with_morphospecies"
  
}else{

  select <- ""  
  
}

  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
} 

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.source_check_traits <- function(source.citation){
  
  if(source.citation){

    select <- ", source_citation"
    
  }else{

    select <- ""  
    
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
} 




#####################################
#Plots

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.cultivated_check_plot <- function(cultivated){
  if(!cultivated){
    query <- "AND (view_full_occurrence_individual.is_cultivated_observation = 0 OR view_full_occurrence_individual.is_cultivated_observation IS NULL) AND view_full_occurrence_individual.is_location_cultivated IS NULL"
    select <- ""
  }else{
    query <- ""
    select <- ",view_full_occurrence_individual.is_cultivated_observation,view_full_occurrence_individual.is_cultivated_in_region,view_full_occurrence_individual.is_location_cultivated"
  }
  output <- as.data.frame(cbind(query, select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query","select")
  
  return(output)  
  
}

##



##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.newworld_check_plot <- function(new.world){
  
  if(is.null(new.world)){

    query <- ""
    select <- ""
    output <- as.data.frame(cbind(query,select),stringsAsFactors = FALSE)  
    colnames(output) <- c("query","select")
    return(output)
    
  }
  
  if(!new.world){

    query <- "AND view_full_occurrence_individual.is_new_world = 0 "    
    select <- ",view_full_occurrence_individual.is_new_world"}
  
  if(new.world){
    query <- "AND view_full_occurrence_individual.is_new_world = 1 "    
    select <- ",view_full_occurrence_individual.is_new_world"}
  
  output <- as.data.frame(cbind(query,select),stringsAsFactors = FALSE)  
  colnames(output) <- c("query","select")
  
  return(output)  
  
}



##  

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.taxonomy_check_plot <- function(all.taxonomy){
  
  if(!all.taxonomy){

    select <- ""

  }else{

    select <- ",view_full_occurrence_individual.verbatim_family,view_full_occurrence_individual.verbatim_scientific_name,view_full_occurrence_individual.family_matched,view_full_occurrence_individual.name_matched,view_full_occurrence_individual.name_matched_author,view_full_occurrence_individual.higher_plant_group,view_full_occurrence_individual.scrubbed_taxonomic_status,view_full_occurrence_individual.scrubbed_family,view_full_occurrence_individual.scrubbed_author"
    
  }

  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}

##

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.native_check_plot <- function(native.status){
  
  if(!native.status){
    select <- ""
  }else{
    select <- ",view_full_occurrence_individual.native_status,view_full_occurrence_individual.native_status_reason,view_full_occurrence_individual.native_status_sources,view_full_occurrence_individual.is_introduced,view_full_occurrence_individual.native_status_country,view_full_occurrence_individual.native_status_state_province,view_full_occurrence_individual.native_status_county_parish"
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}

##  

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.natives_check_plot <- function(natives.only){  
  
  if(!natives.only){
    query <- ""
  }else{
    query <- "AND (view_full_occurrence_individual.is_introduced=0 OR view_full_occurrence_individual.is_introduced IS NULL) "
  }  
  
  output <- as.data.frame(cbind(query), stringsAsFactors = FALSE)  
  colnames(output) <- c("query")
  
  return(output)  
  
}  


##  


##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.political_check_plot <- function(political.boundaries){  
  
  if(!political.boundaries){
    select <- ""
  }else{
    select <- ",view_full_occurrence_individual.country,view_full_occurrence_individual.state_province,view_full_occurrence_individual.county,view_full_occurrence_individual.locality"
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}  

##

##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.collection_check_plot <- function(collection.info){    
  
  if(!collection.info){
    select <- ""
  }else{
    select <- ",view_full_occurrence_individual.catalog_number, view_full_occurrence_individual.recorded_by, view_full_occurrence_individual.record_number, view_full_occurrence_individual.date_collected, view_full_occurrence_individual.identified_by, view_full_occurrence_individual.date_identified, view_full_occurrence_individual.identification_remarks  "
  }  
  
  output <- as.data.frame(cbind(select),stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
}  


##
##
#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.md_check_plot <- function(all.metadata){    
  
  if(!all.metadata){
    select <- ""
  }else{
    select <- ",plot_metadata.coord_uncertainty_m,plot_metadata.methodology_reference,plot_metadata.methodology_description,growth_forms_included_all, growth_forms_included_trees, growth_forms_included_shrubs, growth_forms_included_lianas,
    growth_forms_included_herbs, growth_forms_included_epiphytes, growth_forms_included_notes, taxa_included_all, taxa_included_seed_plants, taxa_included_ferns_lycophytes,
    taxa_included_bryophytes,taxa_included_exclusions"
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  
  } 

#

#'Set query details
#'
#'Helper function to set query components.
#' @param species Single species or vector of species.
#' @keywords internal
.species_check <- function(species){    
  
  if(is.null(species)){
    query <- ""
  }else{
    query <- species_select<-paste(" AND", "scrubbed_species_binomial in (", paste(shQuote(species, type = "sh"),collapse = ', '), ") ")  
  }
  
  output <- as.data.frame(cbind(query), stringsAsFactors = FALSE)  
  colnames(output) <- c("query")
  
  return(output)  
  
} 

#

#'Set query details
#'
#'Helper function to set query components.
#' @param genus Single genus or vector of genera.
#' @keywords internal
.genus_check <- function(genus){    
  
  if(is.null(genus)){
    query <- ""
  }else{
    query <- species_select<-paste(" AND", "scrubbed_genus in (", paste(shQuote(genus, type = "sh"),collapse = ', '), ") ")  
  }
  
  output <- as.data.frame(cbind(query), stringsAsFactors = FALSE)  
  colnames(output) <- c("query")
  
  return(output)  
  
} 


########################################
#Stem


#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.cultivated_check_stem <- function(cultivated){    
  
  if(!cultivated){

    query <- "AND (analytical_stem.is_cultivated_observation = 0 OR analytical_stem.is_cultivated_observation IS NULL) AND analytical_stem.is_location_cultivated IS NULL"
    select <- ""

  }else{

    query <- ""
    select <- ",analytical_stem.is_cultivated_observation,view_full_occurrence_individual.is_cultivated_in_region,analytical_stem.is_location_cultivated"

  }
  
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query", "select")
  
  return(output)  
  
}   

#  

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.newworld_check_stem <- function(new.world){
  
  if(is.null(new.world)){
    query <- ""
    select <- ""
    output <- as.data.frame(cbind(query,select),stringsAsFactors = FALSE)  
    colnames(output) <- c("query","select")
    return(output)  
    
  }
  
  
  if(!new.world){
    query <- "AND analytical_stem.is_new_world = 0 "    
    select <- ",analytical_stem.is_new_world"}
  
  if(new.world){
    query <- "AND analytical_stem.is_new_world = 1 "    
    select <- ",analytical_stem.is_new_world"}
  
  output <- as.data.frame(cbind(query,select), stringsAsFactors = FALSE)  
  colnames(output) <- c("query","select")
  
  return(output)  
  
}





#

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.taxonomy_check_stem <- function(all.taxonomy){    
  
  if(!all.taxonomy){

    select<-""

  }else{

    select<-",analytical_stem.verbatim_family,analytical_stem.verbatim_scientific_name,analytical_stem.family_matched,analytical_stem.name_matched,analytical_stem.name_matched_author,analytical_stem.higher_plant_group,analytical_stem.scrubbed_taxonomic_status,analytical_stem.scrubbed_family,analytical_stem.scrubbed_author"

  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
}

#  

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.native_check_stem <- function(native.status){    
  if(!native.status){
    select <- ""
  }else{
    select <- ",analytical_stem.native_status,analytical_stem.native_status_reason,analytical_stem.native_status_sources,analytical_stem.is_introduced,analytical_stem.native_status_country,analytical_stem.native_status_state_province,analytical_stem.native_status_county_parish"
  }
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
}


#


#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.natives_check_stem <- function(natives.only){    
  
  if(!natives.only){
    query <- ""
  }else{
    query <- "AND (view_full_occurrence_individual.is_introduced=0 OR view_full_occurrence_individual.is_introduced IS NULL)"
  }  
  
  output <- as.data.frame(cbind(query),stringsAsFactors = FALSE)  
  colnames(output) <- c("query")
  
  return(output)  
}

#  

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.political_check_stem <- function(political.boundaries){    
  
  if(!political.boundaries){
    select <- ""
  }else{
    select <- ",analytical_stem.country,analytical_stem.state_province,analytical_stem.county,analytical_stem.locality"
  }
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
}

#


#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.collection_check_stem <- function(collection.info){   
  
  if(!collection.info){
    select <- ""
  }else{
    select <- ",view_full_occurrence_individual.catalog_number, view_full_occurrence_individual.recorded_by, view_full_occurrence_individual.record_number, view_full_occurrence_individual.date_collected, view_full_occurrence_individual.identified_by, view_full_occurrence_individual.date_identified, view_full_occurrence_individual.identification_remarks  "
  }  
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
}

#

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.vfoi_check_stem <- function(native.status, cultivated,natives.only, collection.info){ 
  
  if(native.status | cultivated |natives.only | collection.info){
    join <- " JOIN view_full_occurrence_individual ON (analytical_stem.taxonobservation_id  = view_full_occurrence_individual.taxonobservation_id)"}else{
      join <- ""  
    }
  
  output <- as.data.frame(cbind(join),stringsAsFactors = FALSE)  
  colnames(output) <- c("join")
  
  return(output)  
}

#  

#'Set query details
#'
#'Helper function to set query components.
#' @keywords internal
.md_check_stem <- function(all.metadata){   
  
  if(!all.metadata){
    select <- ""
  }else{
    select <- ",plot_metadata.coord_uncertainty_m,plot_metadata.methodology_reference,plot_metadata.methodology_description,growth_forms_included_all, growth_forms_included_trees, growth_forms_included_shrubs, growth_forms_included_lianas,
    growth_forms_included_herbs, growth_forms_included_epiphytes, growth_forms_included_notes, taxa_included_all, taxa_included_seed_plants, taxa_included_ferns_lycophytes,
    taxa_included_bryophytes,taxa_included_exclusions"
  }
  
  output <- as.data.frame(cbind(select), stringsAsFactors = FALSE)  
  colnames(output) <- c("select")
  
  return(output)  
  }

#########################################
#Value checkers
#########################################
##################################
#'Check that value is logical
#'
#'Helper function to check data format.
#' @keywords internal
#' @examples \dontrun{
#' is_log(TRUE)}
.is_log <- function(x) {
  if (!inherits(x, 'logical')) {
    stop(sys.call()[-1], " should be logical", call. = FALSE)
  }
}


##################################
#'Check that value is logical or null
#'
#'Helper function to check data format.
#' @keywords internal
#' @examples \dontrun{
#' is_log_or_null(new.world)}
.is_log_or_null <- function(x) {
  if (!inherits(x, c('logical','NULL'))) {
    stop(sys.call()[-1], " should be logical or NULL", call. = FALSE)
  }
}

###################################
#'Check that value is character
#'
#'Helper function to check data format.
#' @keywords internal
#' @examples \dontrun{
#' is_char(species)}
.is_char <- function(x) {
  if (!inherits(x ,c("character","NULL"))) {
    stop(sys.call()[-1]," should be character", call. = FALSE)
  }
}

###################################

#'Check that value is numeric
#'
#'Helper function to check data format.
#' @keywords internal
#' @examples \dontrun{
#' is_num(min.lat)}
.is_num <- function(x) {
  if (!inherits(x ,'numeric')) {
    stop(sys.call()[-1]," should be numeric", call. = FALSE)
  }
}

#################################
#################################

#'Run an SQL query on the BIEN database.
#'
#'.BIEN_sql is an internal function used to submit SQL queries.
#' @param query A PostgreSQL query.
#' @param view_full_occurrence_individual Alternative value to be substituted for "view_full_occurrence_individual" in queries when not NULL.
#' @param agg_traits Alternative value to be substituted for "agg_traits" in queries when not NULL.
#' @param species_by_political_division Alternative value to be substituted for "species_by_political_division" in queries when not NULL.
#' @param bien_species_all Alternative value to be substituted for "bien_species_all" in queries when not NULL.
#' @param ranges Alternative value to be substituted for "ranges" in queries when not NULL.
#' @param bien_taxonomy Alternative value to be substituted for "bien_taxonomy" in queries when not NULL.
#' @param phylogeny Alternative value to be substituted for "phylogeny" in queries when not NULL.
#' @param bien_metadata Alternative value to be substituted for "bien_metadata" in queries when not NULL.
#' @param plot_metadata Alternative value to be substituted for "plot_metadata" in queries when not NULL.
#' @param analytical_stem Alternative value to be substituted for "analytical_stem" in queries when not NULL.
#' @param datasource Alternative value to be substituted for "datasource" in queries when not NULL.
#' @param centroid Alternative value to be substituted for "centroid" in queries when not NULL.
#' @param limit A limit on the number of records to be returned.  Should be a single number or NULL (the default).
#' @param return.query Should  the query used be returned rather than executed?  Default is FALSE
#' @param schema An alternative schema to be accessed.  Used for testing purposes.
#' @param print.query Should  the query used be printed?  Default is FALSE
#' @param fetch.query If TRUE (the default) query is executed using dbFetch.  If FALSE, dbGetQuery is used.
#' @param record_limit The number of records to download at once. Defaults to 10000
#' @note Using fetch.query = TRUE provides better error handling, but fetch.query = FALSE results in a more useful (but uncatchable) error.
#' @import RPostgreSQL
#' @importFrom DBI dbDriver dbFetch
#' @return A dataframe returned by the query.
#' @keywords internal
#' @examples \dontrun{
#' .BIEN_sql("SELECT DISTINCT country, scrubbed_species_binomial FROM view_full_occurrence_individual 
#' WHERE country in ( 'United States' );")}
.BIEN_sql <- function(query,
                      view_full_occurrence_individual = NULL,
                      agg_traits = NULL,
                      species_by_political_division = NULL,
                      bien_species_all = NULL,
                      ranges = NULL,
                      bien_taxonomy = NULL,
                      phylogeny = NULL,
                      bien_metadata = NULL,
                      plot_metadata = NULL,
                      analytical_stem = NULL,
                      datasource = NULL,
                      centroid = NULL,
                      limit = NULL,
                      return.query = FALSE,
                      schema = NULL,
                      print.query = FALSE,
                      fetch.query = TRUE,
                      record_limit = 10000 ){
  
  .is_char(query)

  
  if(print.query){
    
    query <- gsub(pattern = "\n",replacement = "",query)
    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl = TRUE)
    print(query)
    
  }
  
  if(!is.null(schema)){
    
    view_full_occurrence_individual <- paste(schema,"view_full_occurrence_individual",sep = ".")
    agg_traits <- paste(schema,"agg_traits",sep = ".")
    species_by_political_division <- paste(schema,"species_by_political_division",sep = ".")
    bien_species_all <- paste(schema,"bien_species_all",sep = ".")
    ranges <- paste(schema,"ranges",sep = ".")
    bien_taxonomy <- paste(schema,"bien_taxonomy",sep = ".")
    phylogeny <- paste(schema,"phylogeny",sep = ".")
    bien_metadata <- paste(schema,"bien_metadata",sep = ".")
    plot_metadata <- paste(schema,"plot_metadata",sep = ".")
    analytical_stem <- paste(schema,"analytical_stem",sep = ".")
    datasource <- paste(schema,"datasource",sep = ".")
    centroid <- paste(schema,"centroid",sep = ".")

  }
  
  
  if(!is.null(view_full_occurrence_individual)){
    
    #query<-gsub(pattern = "\\<view_full_occurrence_individual\\>",replacement = view_full_occurrence_individual,x = query)}
    query <- gsub(pattern = "(?<!as |AS )(?<!\\S)view_full_occurrence_individual(?!\\S)",
                  replacement = view_full_occurrence_individual,
                  x = query,
                  perl = TRUE)
    
    }
  
  
  if(!is.null(plot_metadata)){
    
    #query<-gsub(pattern = "\\<view_full_occurrence_individual\\>",replacement = view_full_occurrence_individual,x = query)}
    query <- gsub(pattern = "(?<!as |AS )(?<!\\S)plot_metadata(?!\\S)",
                  replacement = plot_metadata,
                  x = query,
                  perl = TRUE)
    
    }
  
  if(!is.null(analytical_stem)){
    
    #query<-gsub(pattern = "\\<view_full_occurrence_individual\\>",replacement = view_full_occurrence_individual,x = query)}
    query <- gsub(pattern = "(?<!as |AS )(?<!\\S)analytical_stem(?!\\S)",
                  replacement = analytical_stem,
                  x = query,
                  perl = TRUE)
    
    }
  
  if(!is.null(agg_traits)){
    
    query <- gsub(pattern = "agg_traits",
                  replacement = agg_traits,
                  x = query)
    
    }
  
  if(!is.null(species_by_political_division)){
    
    query <- gsub(pattern = "species_by_political_division",
                  replacement = species_by_political_division,
                  x = query)
    
    }
  
  if(!is.null(bien_species_all)){
    
    query <- gsub(pattern = "bien_species_all",
                  replacement = bien_species_all,
                  x = query)
    }
  
  if(!is.null(ranges)){
    
    query <- gsub(pattern = "ranges",
                  replacement = ranges,
                  x = query)
    
    }
  
  if(!is.null(bien_taxonomy)){
    
    query <- gsub(pattern = "bien_taxonomy",
                  replacement = bien_taxonomy,
                  x = query)
    
    }
  
  if(!is.null(phylogeny)){
    
    query <- gsub(pattern = "\\<phylogeny\\>",
                  replacement = phylogeny,
                  x = query)
  
    }
  
  if(!is.null(bien_metadata)){
    
    query <- gsub(pattern = "\\<bien_metadata\\>",
                  replacement = bien_metadata,
                  x = query)
    
    }
  
  
  if(!is.null(datasource)){
    
    query <- gsub(pattern = "(?<=\\s)datasource(?=\\s)",
                  replacement = datasource,
                  x = query,
                  perl = TRUE)
    
    }  
  
  if(!is.null(centroid)){
    
    query <- gsub(pattern = "(?<=\\s)centroid(?=\\s)",
                  replacement = datasource,
                  x = query,
                  perl = TRUE)
    
    }  
  
  
  if(!is.null(limit)){
    
    query <- gsub(pattern = " ;",
                  replacement = paste(" LIMIT ",limit,";"),
                  x = query)
    
    }
 
  
  
  host <- 'vegbiendev.nceas.ucsb.edu'
  dbname <- 'public_vegbien'
  user <- 'public_bien'
  password <- 'bien_public'
  # Name the database type that will be used
  drv <- dbDriver('PostgreSQL')
  # establish connection with database
  
  if(return.query){
    query <- gsub(pattern = "\n", replacement = "", query)
    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl = TRUE)
    return(query)
  }
  
  
# Try to connect

    con <- tryCatch(expr = dbConnect(drv,
                                     host = host,
                                     dbname = dbname,
                                     user = user,
                                     password = password),
                    error = function(e){e}
    )
  
    if("error" %in% class(con)){
    
      message("\nUnable to connect to the BIEN database. The remote PostgreSQL service may be unavailable or refusing connections.\n",
              conditionMessage(con))
    
      return(invisible(NULL))
    
    }
  



# Execute the query
  
  if(fetch.query){

    # Send the query
    
          suppressWarnings(
            
            
            res <-  tryCatch(expr = dbSendQuery(conn = con,
                                                statement = query),
                            error = function(e){e}
            )
          )
        
        if("error" %in% class(res)){
          
          message("\n There was an error in the SQL query or the SQL query could not be sent (e.g., due to connection issues).")
          
          suppressWarnings(
            
            tryCatch(expr = dbClearResult(res = res),
                     error = function(e){e}
            )
          )   
          
          dbDisconnect(con)
          
          return(invisible(NULL))
          
        }
    
    
    
    
    
    
    # Fetch the query    
        # suppressWarnings(
        #   
        #   df <-  tryCatch(expr = dbFetch(res = res),
        #                   error = function(e){e}
        #                   )
        #   )
    
    
    
    df <- NULL
    batch <- 1
    
    while(!dbHasCompleted(res)){
      
      message("Getting page ", batch, " of records")
      
      df_x <-  tryCatch(expr = dbFetch(res = res,
                                       n = record_limit),
                        error = function(e){e}
      )
      
      #check for errors
      if("error" %in% class(df_x)){
        
        message("Error fetching page ", batch, " of records.")
        dbClearResult(res)
        return(invisible(NULL))
      }
      
      df_x |>
        rbind(df) -> df
      
      batch = batch+1
      
      
    }
    
  
    # Clear the query results
        
      suppressWarnings(
        
        tryCatch(expr = dbClearResult(res = res),
                        error = function(e){e}
        )
      )   
  
    
      
    
  }else{
    
   
    df <- dbGetQuery(con, statement = query); #dbGetQuery doesn't allow error catching, so we've stopped using it by default
    
     
  }

     
# Disconnect from the database
  dbDisconnect(con)
  
  
# Optionally print the query  
  if(print.query){
    
    query <- gsub(pattern = "\n",replacement = "", query)
    query <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", query, perl = TRUE)
    print(query)
    
  }
  
  #Check whether query worked
  
  if("error" %in% class(df)){
    
    message("\nThe query was properly executed, but there was a problem retreiving the query results. This may be due to internet connection issues.")
    
    return(invisible(NULL))
    
  }else{
  
    return(df)
      
  }
  
  
}

# Shared test helpers for BIEN integration tests

bien_connection_available <- local({
  available <- NULL
  
  function(){
    if(!is.null(available)){
      return(available)
    }
    
    available <<- !is.null(
      tryCatch(expr = BIEN:::.BIEN_sql("SELECT 1;"),
               error = function(e){NULL})
    )
    
    available
  }
})

skip_if_bien_unavailable <- function(){
  testthat::skip_if_offline()
  
  if(!bien_connection_available()){
    testthat::skip("BIEN database is unavailable or refusing connections.")
  }
}

expect_bien_class <- function(expr, class){
  result <- tryCatch(expr = eval.parent(substitute(expr)),
                     error = function(e){e})
  
  if(inherits(result, "error")){
    message <- conditionMessage(result)
    connection_error <- grepl(
      pattern = "could not connect|connection to server|remaining connection slots|connection refused|timed out|timeout|reset by peer|server closed the connection",
      x = message,
      ignore.case = TRUE
    )
    
    if(connection_error){
      testthat::skip(paste("BIEN database became unavailable during test:", message))
    }
    
    stop(result)
  }
  
  if(is.null(result)){
    testthat::skip("BIEN database became unavailable during test (NULL returned).")
  }
  
  testthat::expect_true(inherits(result, class))
  invisible(result)
}

#' BIEN: Tools for accessing the BIEN database.
#'
#' @description The Botanical Information and Ecology Network(BIEN) R package provides access to the BIEN database as well as useful tools for working with the BIEN data.
#' 
#' @section Getting started:
#' Type vignette("BIEN") to view the vignette, which contains useful information on the BIEN package.
#' 
#' @references Maitner BS, Boyle B, Casler N, et al. The BIEN R package: A tool to access the Botanical Information and Ecology Network (BIEN) Database. Methods Ecol Evol. 2018;9:373-379. https://doi.org/10.1111/2041-210X.12861
#' 
#' @name BIEN
#' @aliases BIEN-package
"_PACKAGE"
###################


.onAttach <- function(libname, pkgname) {
  packageStartupMessage('Type vignette("BIEN") or vignette("BIEN_tutorial") to get started')
  
  
  suppressWarnings(x <- try(readLines("https://raw.githubusercontent.com/bmaitner/RBIEN/master/NOTES", warn = FALSE), silent = TRUE))
  if(inherits(class(x),"character")){
    if(length(x) != 0){
      packageStartupMessage(x, appendLF = TRUE)  
      
    }  
    
  }
  
  
}


#######################

---
title: "BIEN tutorial"
author: "Brian Maitner"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BIEN tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  
---

```{r setup, include = FALSE}
library(BIEN)

bien_vignette_online <- !is.null(
  tryCatch(expr = BIEN:::.BIEN_sql("SELECT 1;"),
           error = function(e){NULL})
)

knitr::opts_chunk$set(echo = TRUE,
                      eval = bien_vignette_online)

if(!bien_vignette_online){
  message("BIEN database is unavailable or refusing connections; remote-query chunks were not evaluated while building this vignette.")
}
```

# Setup

```{r load-packages, message = F,warning = FALSE, results = 'hide'}
library(ape) #Package for working with phylogenies in R
library(maps) #Useful for making quick maps of occurrences
library(sf) # A package for spatial data
```

# An overview of the package
We try to make this package as easy and intuitive to use as possible, but it is still often easiest to start with our vignette.  Particularly useful are the "Function Names" and "Function Directory" sections.

```{r load-vignette, eval = FALSE}
vignette("BIEN")
```

The function names follow a consistent naming strategy, and mostly consist of 3 parts:

1. The prefix "BIEN_" 
2. The type of data being accessed, e.g. "occurrence_"
3. How you'll be querying the data.  For example, the suffix "state" refers to functions that return data for a specified state.

As a complete example, the function `BIEN_occurrence_species` returns occurrence records for a given species (or set of species).

# Function Families
Currently we have 9 function families in BIEN.  These are sets of functions that access a given type of data.

1. occurrence records (`BIEN_occurrence_...`)
2. range maps (`BIEN_ranges_...`)
3. plot data (`BIEN_plot_...`)
4. trait data (`BIEN_trait_...`)
5. taxonomic information (`BIEN_taxonomy_...`)
6. phylogenies (`BIEN_phylogeny_...`)
7. stem data (`BIEN_stem_...`)
8. species lists (`BIEN_list_...`)
9. metadata (`BIEN_metadata_...`)


We'll walk through each of the function families and take a look at some the options available within each.

# Occurrence records

These functions begin with the prefix `BIEN_occurrence_...` and allow you to query occurrences by either taxonomy or geography.  Functions include:

1. `BIEN_occurrence_country` Returns all occurrence records within a given country

2. `BIEN_occurrence_state` Returns all occurrences records within a given state/province

3.  `BIEN_occurrence_county` Returns all occurrences records within a given state/province

4. `BIEN_occurrence_family` Returns all occurrence records for a specified family

5. `BIEN_occurrence_genus` Returns all occurrence records for a specified genus

6. `BIEN_occurrence_species` Returns all occurrence records for a specified species


Each of these functions has a number of different arguments that modify your query, either refining your search criteria or returning more data for each record.  These arguments include:

1. `cultivated`  If `TRUE`, records known to be cultivated will be returned.

2. `new.world` If `TRUE`, records returned are limited to those in North and South America, where greater data cleaning and validation has been done. IF `FALSE`, records will be limited to the Old World.  If `NULL` (the default), global records will be returned.

* Note that the arguments cultivated and new.world may change the number of records returned.

3. `all.taxonomy` If `TRUE`, the query will return additional taxonomic data, including the uncorrected taxonomic information for those records.

4. `native.status` If `TRUE`, additional information will be returned regarding whether a species is native in a given region.

5. `natives.only` If `TRUE`, the default, information for occurrences flagged as introduced will not be returned.

6. `observation.type`  If `TRUE`, the query will return whether each record is from either a plot or a specimen.  This may be useful if a user believes one type of information may be more accurate.

7. `political.boundaries` If `TRUE`, the query will return information on which country, state, etc. that an occurrence is found within.

8. `collection.info` If `TRUE`, the quest will return additional information about the collection and identification of that specimen.


**Example 1: Occurrence records for a species**

Okay, enough reading.  Let's get some data.

Let's say we're interested in the species *Xanthium strumarium* and we'd like some occurrence data.  We'll use the function `BIEN_occurrence_species` to grab the occurrence data.

```{r xs-occurrences}

Xanthium_strumarium <- BIEN_occurrence_species(species = "Xanthium strumarium")

```

Take a moment and view the dataframe and take a look at the structure

```{r view-xs-occurrences}

str(Xanthium_strumarium)

head(Xanthium_strumarium)

```

The default data that is returned consists of the latitude, longitude and date collected, along with a set of attribution data.  The meaning of some of these columns is obvious (e.g. latitude, longitude), however others may be less clear.  The meanings of these columns and the information within is explained in more detail in our data dictionary, which can be accessed using the function `BIEN_metadata_data_dictionaries`. 

If we want more information on these occurrences, we just need to change the arguments:
```{r xs-occurrences-pt-2, eval = FALSE}

Xanthium_strumarium_full <- BIEN_occurrence_species(species = "Xanthium strumarium",
                                                    cultivated = TRUE,
                                                    all.taxonomy = TRUE,
                                                    native.status = TRUE,
                                                    observation.type = TRUE,
                                                    political.boundaries = TRUE)


```

We now have considerably more information.

If we want to take a quick look at where those occurrences are we could use:

```{r map-xs, eval = FALSE}
# Make a quick map to plot our points on

  map('world', fill = TRUE, col= "grey", bg = "light blue") 

# Plot the points from the full query in red

  points(cbind(Xanthium_strumarium_full$longitude,
               Xanthium_strumarium_full$latitude),
         col = "red",
         pch = 20,
         cex = 1) 

# Plot the points from the default query in blue

  points(cbind(Xanthium_strumarium$longitude,
               Xanthium_strumarium$latitude),
         col = "blue",
         pch = 20,
         cex = 1) 

```


**Example 2: Occurrence records for a country**

Since we may be interested in a particular geographic area, rather than a particular set of species, there are also options to easily extract data by political region as well.

We'll choose a relatively small region, the Bahamas, for our demonstration.  

```{r occs-bahamas, eval = FALSE}

Bahamas <- BIEN_occurrence_country(country =  "Bahamas")

#Let's see how many species we have

  length(unique(Bahamas$scrubbed_species_binomial))
  #About 400 species with valid occurrence records.

#Now, let's take a look at where those occurrences are:
  
  map(regions = "Bahamas" ,
      fill = TRUE ,
      col= "grey",
      bg = "light blue")
  
  points(cbind(Bahamas$longitude,Bahamas$latitude),
         col = "blue",
         pch = 20,
         cex = 1) 

#Looks like some islands are considerably better sampled than others.

```


# Range maps

These functions begin with the prefix `BIEN_ranges_...` and return (unsurprisingly) species ranges.  Most of these functions work by saving the downloaded ranges to a specified directory in shapefile format, rather than by loading them into the R environment.



Functions include:

1. `BIEN_ranges_species`  Downloads range maps for given species and save them to a specified directory.

2. `BIEN_ranges_genus`  Saves range maps for all species within a genus to a specified directory.

3. `BIEN_ranges_load_species`  This function returns the ranges for a set of species as a sf object.

4. `BIEN_range_sf` This function returns all ranges that intersect a user-specified sf object.

The range functions have different arguments than we have seen so far, including:

1. `directory` This is where the function will be saving the shapefiles you download

2.  `matched`  If `TRUE`, the function will return a dataframe listing which species ranges were downloaded and which weren't.

3.  `match_names_only` If `TRUE`, the function will check whether a map is available for each species without actually downloading it

4. `include.gid` If `TRUE`, the function will append a unique gid number to each range map's filename.  This argument is designed to allow forward compatibility when BIEN contains multiple range maps for each species.

**Example 3: Range maps and occurrence points**

If we have a species we're interested in, and would like to load the range map into the environment,  we can use the function `BIEN_ranges_load_species`.  Let's try this for *Xanthium strumarium*.

```{r xs-range}

Xanthium_strumarium_range <- BIEN_ranges_load_species(species = "Xanthium strumarium")

```

The range map is now in our global environment as an sf object.  Let's plot the map and see what it looks like.

```{r xs-range-map}

#First, let's add a base map so that our range has some context:

  map('world', fill = TRUE ,
      col= "grey",
      bg = "light blue",
      xlim = c(-180, -20),
      ylim = c(-60, 80))

#Now, we can add the range map:
  plot(Xanthium_strumarium_range[1],
       col = "green",
       add = TRUE)



```

Now, let's add those occurrence points from earlier to this map:

```{r xs-range-and-points}

map('world',
    fill = TRUE ,
    col = "grey",
    bg = "light blue",
    xlim = c(-180, -20),
    ylim = c(-60, 80))

plot(Xanthium_strumarium_range[1],
     col = "green",
     add = TRUE)

points(cbind(Xanthium_strumarium$longitude,Xanthium_strumarium$latitude),
       col = "blue",
       pch = 20,
       cex = 1)


```

# Plot data

These functions begin with the prefix "BIEN_plot_" and return ecological plot data.  Functions include:

1. `BIEN_plot_list_sampling_protocols` Returns the different plot sampling protocols found in the BIEN database.

2. `BIEN_plot_list_datasource` Returns the different datasources that are available in the BIEN database.  

* These first two functions are useful for identifying plots with comparable sampling methods.

3. `BIEN_plot_sampling_protocol` Downloads data for a specified sampling protocol

4. `BIEN_plot_datasource` Downloads data for a specific datasource

* These next two function are then useful for downloading datasets with consistent methodology.

5. `BIEN_plot_country`

6. `BIEN_plot_state`

7. `BIEN_plot_dataset` Downloads data for a given dataset (which is nested within a datasource)

8. `BIEN_plot_name` Downloads data for a specific plot name (these are nested within a given dataset)

Again we have some of the same arguments available for these queries that we saw for the occurrence functions.  We also have the new argument *`all.metadata`*, which causes the functions to return more metadata for each plot.


**Example 4: Plot data by plot name**

Let's take a look at the data for an individual plot.
```{r luq-occs, eval = FALSE}

LUQUILLO <- BIEN_plot_name(plot.name = "LUQUILLO")

head(LUQUILLO)
  
```

We can see that this is a 0.1 hectare transect where stems >= 2.5 cm diameter at breast height were included.  If we'd like more detail, we can use additional arguments:
```{r, eval = FALSE}

LUQUILLO_full <- BIEN_plot_name(plot.name = "LUQUILLO",
                                cultivated = TRUE,
                                all.taxonomy = TRUE,
                                native.status = TRUE,
                                political.boundaries = TRUE,
                                all.metadata = TRUE)

```

The dataframe `LUQUILLO_full` contains more useful information, including metadata on which taxa were included, which growth forms were included and information on whether species are known to be native or introduced.




# Trait data

These functions begin with the prefix `BIEN_trait_...` and access the BIEN trait database.   Note that the spelling of the trait names must be precise, so we recommend using the function `BIEN_trait_list` first.  Traits names are standardized to follow https:<area>//www<area>.top-thesaurus.org/ where available.  Trait units have been standardized for each trait.


Functions include:

1. `BIEN_trait_list`  Start with this.  It returns a dataframe of the traits available.

2. `BIEN_trait_family`  Returns a dataframe of all trait data for a given family (or families).

3. `BIEN_trait_genus`

4. `BIEN_trait_species`

5. `BIEN_trait_trait` Downloads all records of a specified trait (or traits).

6. `BIEN_trait_mean` Estimates species mean trait values using genus or family level means where species-level data is absent.

7. `BIEN_trait_traitbyfamily` Downloads data for a given family (or families) and trait(s).

8. `BIEN_trait_traitbygenus`

9. `BIEN_trait_traitbyspecies`


**Example 5: Accessing trait data**

If you're interested in accessing all traits for a taxon, say the genus *Salix*, just go ahead and use the corresponding function:

```{r salix-traits, eval = FALSE}

Salix_traits <- BIEN_trait_genus(genus = "Salix")

```

If instead we're interested in a particular trait, the first step is to check if that trait is present and verify the spelling using the function `BIEN_trait_list`.

```{r trait-list, eval = FALSE}

BIEN_trait_list()

```

If we're interested in leaf area, we see that this is indeed called "leaf area" in the database.  Now that we know the proper spelling, we can use the function `BIEN_trait_trait` to download all observations of that trait.

```{r leaf-area, eval = FALSE}

leaf_area <- BIEN_trait_trait(trait = "leaf area")

```

Note that the units have been standardized and that there is a full set of attribution data for each trait.

# Taxonomy Data

While there are existing packages that query taxonomic data (e.g. those included in the excellent taxize package), the BIEN taxonomy functions access the taxonomic information that underlies the BIEN database, ensuring consistency.

1. `BIEN_taxonomy_family` Downloads all taxonomic information for a given family.

2. `BIEN_taxonomy_genus`

3. `BIEN_taxonomy_species`

**Example 6: Taxonomic data**

Let's say we're interested in the genus *Asclepias*, and we'd like to get an idea of how many species there are in this genus and what higher taxa it falls within.

```{r taxonomy}

Asclepias_taxonomy <- BIEN_taxonomy_genus(genus = "Asclepias")

#We see that the genus Asclepias falls within the family Apocynaceae and the order Gentianales.

#You'll also notice that a given species may appear more than once (due to multiple circumscriptions, some of which may be illegitimate).

#If we'd just like to know all the speciess that aren't illegitimate:

Asclepias_species <- unique(Asclepias_taxonomy$scrubbed_species_binomial[Asclepias_taxonomy$scrubbed_taxonomic_status %in% c("accepted",  "no opinion")])


```



# Phylogenies
The BIEN database currently contains 101 phylogenies for new world plants.  This includes 100 replicated phylogenies that include a large fraction of New World plant species ("complete phylogenies") and 1 phylogeny containing only those New World plant species for which molecular data were available ("conservative phylogeny"). Currently, there are only 2 functions available:

1. `BIEN_phylogeny_complete`  This function will return a specified number of the replicated "complete" phylogenies.  Note that each phylogeny is several Mb in size, so downloading many may take a while on slow connections.

2. `BIEN_phylogeny_conservative` This function returns the conservative phylogeny.

Arguments: The function `BIEN_phylogeny_complete` has a few arguments that are worth explaining:

1. `n_phylogenies` This is the number of replicated phylogenies that you want to download (between 1 and 100)

2. `seed` This function sets the seed for the random number generator before randomly drawing the phylogenies to be downloaded.  This is useful for replicating analyses.

3. `replicates` This function allows you to specify WHICH of the 100 phylogenies to download, rather than having them selected randomly.


**Example 7: Phylogenies**

Let's say we want to download the conservative phylogeny.

```{r phylogeny, eval = FALSE}

phylo <- BIEN_phylogeny_conservative()

#Let's make sure it looks alright

  plot.phylo(x = phylo, show.tip.label =  FALSE)

#If we just want to see which species are included

  phylo_species <- phylo$tip.label


```


# Stem Data 

The BIEN database contains stem data associated with many of the plots.  This is typically either diameter at breast height or diameter at ground height.  At present, there is only one stem function (although expect more in the future):

1. `BIEN_stem_species` This function downloads all of the stem data for a given species (or set of species)
2. `BIEN_stem_genus`
3. `BIEN_stem_family`
4. `BIEN_stem_datasource` This function downloads all of the stem data for a given datasource.

Arguments:

The arguments for this function are the same that we have seen in the occurrence and plot functions.

**Example 8: Stem data**

If we'd like stem data for the species *Cupressus arizonica*

```{r stems, eval = FALSE}

Cupressus_arizonica_stems <- BIEN_stem_species("Cupressus arizonica")

```



# Species lists

These functions begin with the prefix `BIEN_list_` and allow you to quickly get a list of all the species in a geographic unit.  Functions include:

1. `BIEN_list_country` Returns all species found within a country.

2. `BIEN_list_state` Returns all species found within a given state/province or other 2nd level political division.

3.  `BIEN_list_county` Returns all species found within a given county/parish/or other 3rd level political division.

Some of the same arguments we saw in the occurrence functions appear here as well, including `cultivated` and `new.world`.


**Example 9: Species list for a country**

Let's return to our previous example.  What if we just need a list of the species in the Bahamas, rather than the specific details of each occurrence record?  We can instead use the function `BIEN_list_country` to download a list of species, which should be much faster than using `BIEN_occurrence_country` to get a species list.

```{r lists, eval = FALSE}

Bahamas_species_list <- BIEN_list_country(country = "Bahamas")

#Notice that we find many more species listed than we found occurrence records for.  What happened?  There are many records coming from the Bahamas that lack coordinates.  These records are used used in the "_list_" functions, but not the occurrence functions.

```

If we wanted to retrieve the results for multiple countries at once, that is simple as well.  We just need to supply a vector of countries.

```{r lists-multi-country, eval = FALSE}

country_vector <- c("Haiti","Dominican Republic")

Haiti_DR <- BIEN_list_country(country = country_vector)

```

We can also use political division codes (from geonames.org) instead of writing out the full country names.

```{r lists-pol-divs}
#To see all of the political division names, and associated codes, we can use this function:

political_names <- BIEN_metadata_list_political_names()

#Let's take a look at what the dataframe contains:

head(political_names)

#In addition to the standardized country, state (state_province_ascii) and county (county_parish_ascii) names, we have the associated codes that can be used in BIEN functions.
#Note that 'state' refers to any primary political division (e.g. province), and 'county' refers to any secondary political division (e.g. parish).

#Looking at the political_names dataframe, we see that the Dominican Republic has country code "DO", and Haiti has country code "HT"

Haiti_DR_from_codes <- BIEN_list_country(country.code = c("HT","DO"))


```


# Metadata
The BIEN metadata functions start with the prefix `BIEN_metadata_...` and provide useful metadata for the BIEN database.

1. `BIEN_metadata_database_version` Returns the current version number of the BIEN database and the release date.

2. `BIEN_metadata_match_data` Rudimentary function to check for changed records between old and current queries.

3. `BIEN_metadata_citation` Function to generate bibtex citations for use in reference managers.

4. `BIEN_metadata_list_political_names` Returns a dataframe containing political division names and associate codes.

**Example 10: Metadata**

To check what the current version of the BIEN database is (which we recommend reporting when using BIEN data):

```{r md-1}

BIEN_metadata_database_version()

```

**Example 11: Citations**

One of the more innovative features of the BIEN package is that it will generate custom attribution data for you based on what data you downloaded through the package.

Let's say we're interested in *Selaginella selaginoides*, and we'd like to download some occurrence data:

```{r selaginella-occs, eval = FALSE}

Selaginella_selaginoides_occurrences <- BIEN_occurrence_species("Selaginella selaginoides", new.world = NULL)

```

If we plan on using those data in a publication, we'll need proper attribution.  We can use `BIEN_metadata_citation` to do this for us:

```{r selaginella-md, eval = FALSE}

citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences)

```
citation_info is a list that contains 3 elements:
1. A bit of general information on how to use the list.
2. A set of bibtex formatted references.
3. Acknowledgement text.

To make things even easier on ourselves, we can use some of the additional functionality of the `BIEN_metadata_citation` function:

```{r md-2, eval = FALSE}

temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a temporary working directory


citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
                                     bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
                                      acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))


```
Now, we have a bibtex file, `selaginella_selaginoides.bib`, that can be loaded into a reference manager (e.g. Endnote, Paperpile, etc.), and a text file, `selaginella_selaginoides.txt`, containing text that can be pasted into the acknowledgements section of a publication.

What if we also have some trait data?  No problem there, the code handles that as well:

```{r md-3, eval = FALSE}
#First, let's get some trait data:
selaginella_selaginoides_traits <- BIEN_trait_species(species = "Selaginella selaginoides")

#Now, we just need to modify our previous bit of code to include the trait data as well:

temp_dir <- file.path(tempdir(), "BIEN_temp")

citation_info <- BIEN_metadata_citation(dataframe = Selaginella_selaginoides_occurrences,
                                      trait.dataframe = selaginella_selaginoides_traits,
                                      bibtex_file = file.path(temp_dir,"selaginella_selaginoides.bib"),
                                      acknowledgement_file = file.path(temp_dir,"selaginella_selaginoides.txt"))

```


The updated citation information will now contain references for both trait and occurrence records.


# Combining Queries
**Example 11: Putting it all together **
Coming soon!

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_list_county}
\alias{BIEN_list_county}
\title{Extract a species list by county.}
\usage{
BIEN_list_county(
  country = NULL,
  state = NULL,
  county = NULL,
  country.code = NULL,
  state.code = NULL,
  county.code = NULL,
  cultivated = FALSE,
  new.world = NULL,
  ...
)
}
\arguments{
\item{country}{A single country or vector of countries}

\item{state}{A state or vector of states (or other primary political divisions, e.g. provinces).}

\item{county}{A single county (or other secondary administrative boundary)or vector of counties.}

\item{country.code}{A single country (or other primary administrative boundary) code or a vector of country codes equal in length to the vector of states/province codes.}

\item{state.code}{A single state/province code, or a vector of states/province codes.}

\item{county.code}{A single county (or other secondary administrative boundary) code or a vector of county codes equal in length to the vectors of states/province codes and country codes.}

\item{cultivated}{Return information on cultivation status?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing species list(s) for the specified states/provinces.
}
\description{
BIEN_list_county produces a list of all species with geovalidated occurrences falling within specified county or counties.
}
\note{
Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.

We recommend using country, state, and county rather than codes, since county names have not been fully standardized.

This function requires you supply either 1) a single state and country with one or more counties, or 2) vectors of equal length for each political level.
}
\examples{
\dontrun{
BIEN_list_county("United States", "Michigan", "Kent")
BIEN_list_county(country = "United States", state = "Michigan", county = "Kent")
county_vector<-c("Kent","Kalamazoo")
BIEN_list_county(country = "United States", state = "Michigan", county = county_vector)}
}
\seealso{
Other list functions: 
\code{\link{BIEN_list_all}()},
\code{\link{BIEN_list_country}()},
\code{\link{BIEN_list_sf}()},
\code{\link{BIEN_list_state}()}
}
\concept{list functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_plot_metadata}
\alias{BIEN_plot_metadata}
\title{Download plot metadata}
\usage{
BIEN_plot_metadata(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe containing plot metadata.
}
\description{
BIEN_plot_metadata downloads the plot metadata table.
}
\examples{
\dontrun{
BIEN_plot_metadata()}
}
\seealso{
Other plot functions: 
\code{\link{BIEN_plot_country}()},
\code{\link{BIEN_plot_dataset}()},
\code{\link{BIEN_plot_datasource}()},
\code{\link{BIEN_plot_list_datasource}()},
\code{\link{BIEN_plot_list_sampling_protocols}()},
\code{\link{BIEN_plot_name}()},
\code{\link{BIEN_plot_sampling_protocol}()},
\code{\link{BIEN_plot_sf}()},
\code{\link{BIEN_plot_state}()}

Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_trait_traits_per_species}
\alias{BIEN_trait_traits_per_species}
\title{Count the number of trait observations for each species in the BIEN database}
\usage{
BIEN_trait_traits_per_species(species = NULL, ...)
}
\arguments{
\item{species}{Optional species or vector of species.  If left blank, returns counts for all species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Returns a dataframe containing the number of trait records for each species in the BIEN database.
}
\description{
BIEN_trait_traits_per_species downloads a count of the number of records for each trait for each species in the BIEN database.
}
\examples{
\dontrun{
trait_observation_counts<-BIEN_trait_traits_per_species()}
}
\seealso{
Other trait functions: 
\code{\link{BIEN_trait_country}()},
\code{\link{BIEN_trait_family}()},
\code{\link{BIEN_trait_genus}()},
\code{\link{BIEN_trait_list}()},
\code{\link{BIEN_trait_mean}()},
\code{\link{BIEN_trait_species}()},
\code{\link{BIEN_trait_trait}()},
\code{\link{BIEN_trait_traitbyfamily}()},
\code{\link{BIEN_trait_traitbygenus}()},
\code{\link{BIEN_trait_traitbyspecies}()}
}
\concept{trait functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_trait_list}
\alias{BIEN_trait_list}
\title{List all available types of trait data}
\usage{
BIEN_trait_list(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe containing all currently available types of trait data and details on measurement.
}
\description{
BIEN_trait_list produces a dataframe of all available types of trait data.
}
\examples{
\dontrun{
BIEN_trait_list()}
}
\seealso{
Other trait functions: 
\code{\link{BIEN_trait_country}()},
\code{\link{BIEN_trait_family}()},
\code{\link{BIEN_trait_genus}()},
\code{\link{BIEN_trait_mean}()},
\code{\link{BIEN_trait_species}()},
\code{\link{BIEN_trait_trait}()},
\code{\link{BIEN_trait_traitbyfamily}()},
\code{\link{BIEN_trait_traitbygenus}()},
\code{\link{BIEN_trait_traitbyspecies}()},
\code{\link{BIEN_trait_traits_per_species}()}
}
\concept{trait functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.vfoi_check_stem}
\alias{.vfoi_check_stem}
\title{Set query details}
\usage{
.vfoi_check_stem(native.status, cultivated, natives.only, collection.info)
}
\description{
Helper function to set query components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_plot_list_datasource}
\alias{BIEN_plot_list_datasource}
\title{List available datasources.}
\usage{
BIEN_plot_list_datasource(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A vector of available datasources.
}
\description{
BIEN_plot_list_datasource list all plot datasources in the BIEN database.
}
\examples{
\dontrun{
BIEN_plot_list_datasource()}
}
\seealso{
Other plot functions: 
\code{\link{BIEN_plot_country}()},
\code{\link{BIEN_plot_dataset}()},
\code{\link{BIEN_plot_datasource}()},
\code{\link{BIEN_plot_list_sampling_protocols}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_plot_name}()},
\code{\link{BIEN_plot_sampling_protocol}()},
\code{\link{BIEN_plot_sf}()},
\code{\link{BIEN_plot_state}()}
}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_plot_dataset}
\alias{BIEN_plot_dataset}
\title{Download plot data by dataset.}
\usage{
BIEN_plot_dataset(
  dataset,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  political.boundaries = FALSE,
  collection.info = FALSE,
  all.metadata = FALSE,
  ...
)
}
\arguments{
\item{dataset}{A plot dataset or vector of datasets. See BIEN_plot_metadata for more information on plots.}

\item{cultivated}{Return cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{all.metadata}{Should additional plot metadata be returned?  Default is FALSE.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe containing all data from the specified dataset.
}
\description{
BIEN_plot_dataset downloads all plot data for a given dataset or datasets.
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php

Datasets and related information can be looked up with \code{\link{BIEN_plot_metadata}}
}
\examples{
\dontrun{
BIEN_plot_dataset("Gentry Transect Dataset")}
}
\seealso{
Other plot functions: 
\code{\link{BIEN_plot_country}()},
\code{\link{BIEN_plot_datasource}()},
\code{\link{BIEN_plot_list_datasource}()},
\code{\link{BIEN_plot_list_sampling_protocols}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_plot_name}()},
\code{\link{BIEN_plot_sampling_protocol}()},
\code{\link{BIEN_plot_sf}()},
\code{\link{BIEN_plot_state}()}
}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_box}
\alias{BIEN_ranges_box}
\title{Download range maps that intersect a specified bounding box.}
\usage{
BIEN_ranges_box(
  min.lat,
  max.lat,
  min.long,
  max.long,
  directory = NULL,
  species.names.only = FALSE,
  return.species.list = TRUE,
  crop.ranges = FALSE,
  include.gid = FALSE,
  ...
)
}
\arguments{
\item{min.lat}{Minimum latitude of the ranges included.}

\item{max.lat}{Maximum latitude of the ranges included.}

\item{min.long}{Minimum longitude of the ranges included.}

\item{max.long}{Maximum longitude of the ranges included.}

\item{directory}{Directory that range maps should be saved in.  If none is specified, range maps will be saved in the current working directory.}

\item{species.names.only}{Return species names rather than spatial data? Default is FALSE.}

\item{return.species.list}{Should a species list be returned in addition to downloading range maps?  Default is FALSE}

\item{crop.ranges}{Should the ranges be cropped to the focal area? Default is FALSE.}

\item{include.gid}{Should the files returned have a unique GID appended to them? This is needed if downloading multiple maps for the same species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Range maps for all available species within the specified bounding box.
}
\description{
BIEN_ranges_box extracts range maps for a specified bounding box.
}
\note{
Details on the construction of BIEN range maps is available at https://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\dontrun{
temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a working directory
BIEN_ranges_box(42,43,-85,-84,species.names.only = TRUE)
BIEN_ranges_box(42,43,-85,-84,directory = temp_dir)}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_taxonomy_family}
\alias{BIEN_taxonomy_family}
\title{Extract taxonomic information for families}
\usage{
BIEN_taxonomy_family(family, ...)
}
\arguments{
\item{family}{A single family or a vector of families.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing taxonomic information for the specified families.
}
\description{
BIEN_taxonomy_family downloads a dataframe of all taxonomic information for given families.
}
\examples{
\dontrun{
BIEN_taxonomy_family("Orchidaceae")
family_vector<-c("Orchidaceae","Poaceae")
BIEN_taxonomy_family(family_vector)}
}
\seealso{
Other taxonomy functions: 
\code{\link{BIEN_taxonomy_genus}()},
\code{\link{BIEN_taxonomy_species}()}
}
\concept{taxonomy functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_metadata_match_data}
\alias{BIEN_metadata_match_data}
\title{Check for differing records between old and new dataframes.}
\usage{
BIEN_metadata_match_data(old, new, return = "identical")
}
\arguments{
\item{old}{A dataframe that is to be compared to a (typically) newer dataframe.}

\item{new}{A dataframe that is to be compared to a (typically) older dataframe.}

\item{return}{What information should be returned?  Current options are: "identical" (Logical, are the two dataframes identical?), "additions" (numeric, which rows are new?), "deletions" (numeric, which rows are no longer present?), "logical" (logical, which elements of the old dataframe are in the new one?).}
}
\value{
Logical of varying length (depending on choice of "return" parameter)
}
\description{
BIEN_metadata_match_data compares old and new dataframes, and can check whether they are identical or be used to select rows that are unique to the old or new versions.
}
\note{
Since comparisons are done by row (except when using return="identical"), this function may fail to flag additions or deletions if they are exact duplicates of existing rows.
}
\examples{
\dontrun{
new<-BIEN_occurrence_species("Acer nigrum")
old<-new[-1:-4,]#simulate having an older dataset by removing four rows
BIEN_metadata_match_data(old,new,return="identical")
BIEN_metadata_match_data(old,new,return="additions")}
}
\seealso{
Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_list_all}
\alias{BIEN_list_all}
\title{Extract a list of all species in the BIEN database.}
\usage{
BIEN_list_all(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing a list of all species in the BIEN database.
}
\description{
BIEN_list_all produces a list of all species in the BIEN database.
}
\examples{
\dontrun{
species_list<-BIEN_list_all()}
}
\seealso{
Other list functions: 
\code{\link{BIEN_list_country}()},
\code{\link{BIEN_list_county}()},
\code{\link{BIEN_list_sf}()},
\code{\link{BIEN_list_state}()}
}
\concept{list functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN_sql.R
\name{.BIEN_sql}
\alias{.BIEN_sql}
\title{Run an SQL query on the BIEN database.}
\usage{
.BIEN_sql(
  query,
  view_full_occurrence_individual = NULL,
  agg_traits = NULL,
  species_by_political_division = NULL,
  bien_species_all = NULL,
  ranges = NULL,
  bien_taxonomy = NULL,
  phylogeny = NULL,
  bien_metadata = NULL,
  plot_metadata = NULL,
  analytical_stem = NULL,
  datasource = NULL,
  centroid = NULL,
  limit = NULL,
  return.query = FALSE,
  schema = NULL,
  print.query = FALSE,
  fetch.query = TRUE,
  record_limit = 10000
)
}
\arguments{
\item{query}{A PostgreSQL query.}

\item{view_full_occurrence_individual}{Alternative value to be substituted for "view_full_occurrence_individual" in queries when not NULL.}

\item{agg_traits}{Alternative value to be substituted for "agg_traits" in queries when not NULL.}

\item{species_by_political_division}{Alternative value to be substituted for "species_by_political_division" in queries when not NULL.}

\item{bien_species_all}{Alternative value to be substituted for "bien_species_all" in queries when not NULL.}

\item{ranges}{Alternative value to be substituted for "ranges" in queries when not NULL.}

\item{bien_taxonomy}{Alternative value to be substituted for "bien_taxonomy" in queries when not NULL.}

\item{phylogeny}{Alternative value to be substituted for "phylogeny" in queries when not NULL.}

\item{bien_metadata}{Alternative value to be substituted for "bien_metadata" in queries when not NULL.}

\item{plot_metadata}{Alternative value to be substituted for "plot_metadata" in queries when not NULL.}

\item{analytical_stem}{Alternative value to be substituted for "analytical_stem" in queries when not NULL.}

\item{datasource}{Alternative value to be substituted for "datasource" in queries when not NULL.}

\item{centroid}{Alternative value to be substituted for "centroid" in queries when not NULL.}

\item{limit}{A limit on the number of records to be returned.  Should be a single number or NULL (the default).}

\item{return.query}{Should  the query used be returned rather than executed?  Default is FALSE}

\item{schema}{An alternative schema to be accessed.  Used for testing purposes.}

\item{print.query}{Should  the query used be printed?  Default is FALSE}

\item{fetch.query}{If TRUE (the default) query is executed using dbFetch.  If FALSE, dbGetQuery is used.}

\item{record_limit}{The number of records to download at once. Defaults to 10000}
}
\value{
A dataframe returned by the query.
}
\description{
.BIEN_sql is an internal function used to submit SQL queries.
}
\note{
Using fetch.query = TRUE provides better error handling, but fetch.query = FALSE results in a more useful (but uncatchable) error.
}
\examples{
\dontrun{
.BIEN_sql("SELECT DISTINCT country, scrubbed_species_binomial FROM view_full_occurrence_individual 
WHERE country in ( 'United States' );")}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_metadata_list_political_names}
\alias{BIEN_metadata_list_political_names}
\title{List political divisions and associated geonames codes.}
\usage{
BIEN_metadata_list_political_names(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe containing political division names and their associated codes.
}
\description{
BIEN_metadata_list_political_names downloads country, state, and county names and associated codes used by BIEN.
}
\note{
Political names and codes follow http://www.geonames.org/
}
\examples{
\dontrun{
BIEN_metadata_list_political_names()}
}
\seealso{
Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_phylogeny_label_nodes}
\alias{BIEN_phylogeny_label_nodes}
\title{Label nodes on a phylogeny}
\usage{
BIEN_phylogeny_label_nodes(
  phylogeny,
  family = TRUE,
  genus = FALSE,
  other_taxa = NULL,
  ...
)
}
\arguments{
\item{phylogeny}{A single phylogeny.}

\item{family}{Should family-level nodes be labeled?  Default is TRUE.}

\item{genus}{Should genus-level nodes be labeled?  Default is FALSE.  Overwrites family-level nodes where a family contains a single genera.}

\item{other_taxa}{A dataframe containing two columns: 1) the taxa to be labelled; 2) the species associated with each taxon.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Input phylogeny with labeled nodes.
}
\description{
BIEN_phylogeny_label_nodes will label the nodes on a phylogeny based on either the BIEN taxonomy or user-supplied taxa.
}
\note{
Information on the construction of the BIEN phylogenies is available online at https://bien.nceas.ucsb.edu/bien/biendata/bien-2/phylogeny/
}
\examples{
\dontrun{
phylogeny<-BIEN_phylogeny_conservative()

phylogeny<-drop.tip(phy = phylogeny,tip = 101:length(phylogeny$tip.label))
plot.phylo(x = phylogeny,show.tip.label = FALSE)

fam_nodes<-BIEN_phylogeny_label_nodes(phylogeny = phylogeny,family = TRUE)
plot.phylo(x = fam_nodes,show.tip.label = FALSE, show.node.label = TRUE)

gen_nodes<-BIEN_phylogeny_label_nodes(phylogeny = phylogeny, family = FALSE, genus = TRUE)
plot.phylo(x = gen_nodes, show.tip.label = FALSE, show.node.label = TRUE)

other_taxa <- as.data.frame(matrix(nrow = 10,ncol = 2))
colnames(other_taxa)<-c("taxon","species")
other_taxa$taxon[1:5]<-"A" #Randomly assign a few species to taxon A
other_taxa$taxon[6:10]<-"B" #Randomly assign a few species to taxon B
tax_nodes <- 
 BIEN_phylogeny_label_nodes(phylogeny = phylogeny,
                            family = FALSE, genus = FALSE, other_taxa = other_taxa)
plot.phylo(x = tax_nodes,show.tip.label = FALSE,show.node.label = TRUE)}
}
\seealso{
Other phylogeny functions: 
\code{\link{BIEN_phylogeny_complete}()},
\code{\link{BIEN_phylogeny_conservative}()}
}
\concept{phylogeny functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.native_check}
\alias{.native_check}
\title{Set query details}
\usage{
.native_check(native.status)
}
\description{
Helper function to set query components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_occurrence_records_per_species}
\alias{BIEN_occurrence_records_per_species}
\title{Count the number of (geoValid) occurrence records for each species in BIEN}
\usage{
BIEN_occurrence_records_per_species(species = NULL, ...)
}
\arguments{
\item{species}{A single species, or vector of species.  If NULL, the default, it will return counts for all species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe listing the number of geovalidated occurrence records for each species in the BIEN database.
}
\description{
BIEN_occurrence_records_per_species downloads a count of the number of geovalidated occurrence records for each species in the BIEN database.
}
\examples{
\dontrun{
occurrence_counts<-BIEN_occurrence_records_per_species()}
}
\seealso{
Other occurrence functions: 
\code{\link{BIEN_occurrence_box}()},
\code{\link{BIEN_occurrence_country}()},
\code{\link{BIEN_occurrence_county}()},
\code{\link{BIEN_occurrence_family}()},
\code{\link{BIEN_occurrence_genus}()},
\code{\link{BIEN_occurrence_sf}()},
\code{\link{BIEN_occurrence_species}()},
\code{\link{BIEN_occurrence_state}()}
}
\concept{occurrence functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_sf}
\alias{BIEN_ranges_sf}
\title{Download range maps that intersect a user-supplied sf object.}
\usage{
BIEN_ranges_sf(
  sf,
  directory = NULL,
  species.names.only = FALSE,
  return.species.list = TRUE,
  crop.ranges = FALSE,
  include.gid = FALSE,
  ...
)
}
\arguments{
\item{sf}{An object of class sf.}

\item{directory}{Directory that range maps should be saved in.  If none is specified, range maps will be saved in the current working directory.}

\item{species.names.only}{Return species names rather than spatial data? Default is FALSE.}

\item{return.species.list}{Should a species list be returned in addition to downloading range maps?  Default is FALSE}

\item{crop.ranges}{Should the ranges be cropped to the focal area? Default is FALSE.}

\item{include.gid}{Should the files returned have a unique GID appended to them? This is needed if downloading multiple maps for the same species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
All range maps that intersect the user-supplied sf object.
}
\description{
BIEN_ranges_sf extracts range maps that intersect a specified simple features (sf) object.
}
\note{
Details on the construction of BIEN range maps is available at https://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\dontrun{

# Here we use a range map as our example polygon

BIEN_ranges_species("Carnegiea gigantea") #saves ranges to the current working directory

# Read in the polygon  with sf   
sf <- sf::st_read(dsn = ".",
                  layer = "Carnegiea_gigantea")

BIEN_ranges_sf(sf = sf,
               limit = 10) 
               # We use the limit argument to return only 10 range maps.
               # Omit the limit argument to get all ranges

#Note that this will save many shapefiles to the working directory.
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_trait_traitbyfamily}
\alias{BIEN_trait_traitbyfamily}
\title{Download trait data for given families and traits.}
\usage{
BIEN_trait_traitbyfamily(
  family,
  trait,
  all.taxonomy = FALSE,
  political.boundaries = FALSE,
  source.citation = FALSE,
  ...
)
}
\arguments{
\item{family}{A single family or a vector of families.}

\item{trait}{A single trait or a vector of traits.}

\item{all.taxonomy}{Should full taxonomic information and TNRS output be returned?  Default is FALSE.}

\item{political.boundaries}{Should political boundary information (country, state, etc.) be returned?  Default is FALSE.}

\item{source.citation}{Should readable source information be downloaded for each record?  Note that \code{\link{BIEN_metadata_citation}} may be more useful.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe of all data matching the specified trait(s) and family/families.
}
\description{
BIEN_trait_traitbyfamily extracts entries that contain the specified families and trait(s).
}
\note{
Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
}
\examples{
\dontrun{
BIEN_trait_traitbyfamily(trait = "whole plant height", family = "Poaceae")
trait_vector <- c("whole plant height", "leaf fresh mass")
family_vector < -c("Orchidaceae","Poaceae")
BIEN_trait_traitbyfamily(trait = trait_vector, family = family_vector)}
}
\seealso{
Other trait functions: 
\code{\link{BIEN_trait_country}()},
\code{\link{BIEN_trait_family}()},
\code{\link{BIEN_trait_genus}()},
\code{\link{BIEN_trait_list}()},
\code{\link{BIEN_trait_mean}()},
\code{\link{BIEN_trait_species}()},
\code{\link{BIEN_trait_trait}()},
\code{\link{BIEN_trait_traitbygenus}()},
\code{\link{BIEN_trait_traitbyspecies}()},
\code{\link{BIEN_trait_traits_per_species}()}
}
\concept{trait functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_genus}
\alias{BIEN_ranges_genus}
\title{Download range maps for given genus.}
\usage{
BIEN_ranges_genus(
  genus,
  directory = NULL,
  matched = TRUE,
  match_names_only = FALSE,
  include.gid = FALSE,
  ...
)
}
\arguments{
\item{genus}{A single genus or a vector of genera.}

\item{directory}{Directory that range maps should be saved in.  If none is specified, range maps will be saved in the current working directory.}

\item{matched}{Return a list of taxa that were downloaded. Default is TRUE.}

\item{match_names_only}{Check for range maps for the taxa specified without downloading range maps. Default is FALSE.}

\item{include.gid}{Should the files returned have a unique GID appended to them? This is needed if downloading multiple maps for the same species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Range maps for all available species within the specified genera.
}
\description{
BIEN_ranges_genus extracts range maps for the specified genera.
}
\note{
Details on the construction of BIEN range maps is available at https://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\dontrun{
library(maps)
library(sf)

genus_vector <- c("Abies","Acer")

temp_dir <- file.path(tempdir(), "BIEN_temp")#Set a working directory

BIEN_ranges_genus(genus_vector)

BIEN_ranges_genus(genus = genus_vector,
                  match_names_only = TRUE)

BIEN_ranges_genus(genus = genus_vector,
                  directory = temp_dir) #saves ranges to a specified working directory

BIEN_ranges_genus("Abies")

BIEN_ranges_genus(genus = "Abies",
                  directory = temp_dir)

#Reading files

Abies_poly <- read_sf(dsn = temp_dir,layer = "Abies_lasiocarpa")

#Plotting files

plot(Abies_poly[1]) #plots the range, but doesn't mean much without any reference

map('world', fill = TRUE, col = "grey") #plots a world map (WGS84 projection), in grey

plot(Abies_poly[1],
     col="forest green",
     add = TRUE) #adds the range of Abies lasiocarpa to the map

# Getting data from the files (currently only species names)

Abies_poly$species#gives the species name associated with "Abies_poly"
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN_info.R
\docType{package}
\name{BIEN}
\alias{BIEN}
\alias{BIEN-package}
\title{BIEN: Tools for accessing the BIEN database.}
\description{
The Botanical Information and Ecology Network(BIEN) R package provides access to the BIEN database as well as useful tools for working with the BIEN data.
}
\section{Getting started}{

Type vignette("BIEN") to view the vignette, which contains useful information on the BIEN package.
}

\references{
Maitner BS, Boyle B, Casler N, et al. The BIEN R package: A tool to access the Botanical Information and Ecology Network (BIEN) Database. Methods Ecol Evol. 2018;9:373-379. https://doi.org/10.1111/2041-210X.12861
}
\author{
\strong{Maintainer}: Brian Maitner \email{bmaitner@gmail.com}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_taxonomy_species}
\alias{BIEN_taxonomy_species}
\title{Extract taxonomic information for species}
\usage{
BIEN_taxonomy_species(species, ...)
}
\arguments{
\item{species}{A single species or a vector of species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing taxonomic information for the specified species.
}
\description{
BIEN_taxonomy_species downloads a dataframe of all taxonomic information for given species.
}
\examples{
\dontrun{
BIEN_taxonomy_species("Cannabis sativa")
species_vector<-c("Acer nigrum","Cannabis sativa")
BIEN_taxonomy_species(species_vector)}
}
\seealso{
Other taxonomy functions: 
\code{\link{BIEN_taxonomy_family}()},
\code{\link{BIEN_taxonomy_genus}()}
}
\concept{taxonomy functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_list}
\alias{BIEN_ranges_list}
\title{List available range maps}
\usage{
BIEN_ranges_list(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A data.frame containing the available species and their associated GIDs.
}
\description{
BIEN_ranges_list a data.frame containing listing all range maps currently available.
}
\examples{
\dontrun{
available_maps<-BIEN_ranges_list()}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}

Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_plot_metadata}()}
}
\concept{metadata functions}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_phylogeny_conservative}
\alias{BIEN_phylogeny_conservative}
\title{Download the conservative BIEN phylogeny}
\usage{
BIEN_phylogeny_conservative(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A phylo object containing the BIEN conservative phylogeny
}
\description{
BIEN_phylogeny_conservative downloads the conservative BIEN phylogeny, which only includes species with molecular data available.
}
\note{
Information on the construction of the BIEN phylogenies is available online at https://bien.nceas.ucsb.edu/bien/biendata/bien-2/phylogeny/
}
\examples{
\dontrun{
BIEN_phylo<-BIEN_phylogeny_conservative()}
}
\seealso{
Other phylogeny functions: 
\code{\link{BIEN_phylogeny_complete}()},
\code{\link{BIEN_phylogeny_label_nodes}()}
}
\concept{phylogeny functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_skinny_ranges_to_richness_raster}
\alias{BIEN_ranges_skinny_ranges_to_richness_raster}
\title{Build a richness raster from a skinny range file}
\usage{
BIEN_ranges_skinny_ranges_to_richness_raster(skinny_ranges, raster)
}
\arguments{
\item{skinny_ranges}{A matrix output by the function "BIEN_ranges_skinny" or equivalent methods.}

\item{raster}{The raster that was used in building the skinny_ranges matrix.}
}
\value{
Raster
}
\description{
BIEN_ranges_skinny_ranges_to_richness_raster takes in "skinny" range data and converts it to a richness raster.
}
\examples{
\dontrun{


template_raster <- terra::rast(
  crs = "+proj=laea +lat_0=15 +lon_0=-80 +x_0=0 +y_0=0 +datum=WGS84 
  +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0",
  ext = ext(c(-5261554, 5038446, -7434988, 7165012 )),
  resolution =  c(100000, 100000))

#Download ranges and convert to a "skinny" format
skinny_ranges <- BIEN_ranges_shapefile_to_skinny(
directory = BIEN_ranges_species_bulk(species = c("Acer rubrum"), 
raster = template_raster)

#Convert from skinny format to richness raster                                                 
richness_raster<- BIEN_ranges_skinny_ranges_to_richness_raster(
 skinny_ranges = skinny_ranges,raster = template_raster)
 
 plot(richness_raster)
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_load_species}
\alias{BIEN_ranges_load_species}
\title{Load range maps for specified species.}
\usage{
BIEN_ranges_load_species(species, ...)
}
\arguments{
\item{species}{A single species or a vector of species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A sf containing range maps for the specified species.
}
\description{
BIEN_ranges_load_species returns spatial data for the specified species.
}
\examples{
\dontrun{
library(maps)
species_vector<-c("Abies_lasiocarpa","Abies_amabilis")
abies_maps <- BIEN_ranges_load_species(species = species_vector)
xanthium_strumarium <- BIEN_ranges_load_species(species = "Xanthium strumarium")

#Plotting files
plot(abies_maps) # plots the sf, but doesn't mean much without any reference
map('world', fill = TRUE, col = "grey")#plots a world map (WGS84 projection), in grey
plot(xanthium_strumarium,col="forest green",add = TRUE) #adds the range of X. strumarium
plot(abies_maps[1,], add = TRUE, col ="light green")
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.genus_check}
\alias{.genus_check}
\title{Set query details}
\usage{
.genus_check(genus)
}
\arguments{
\item{genus}{Single genus or vector of genera.}
}
\description{
Helper function to set query components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_trait_country}
\alias{BIEN_trait_country}
\title{Download trait data for given country.}
\usage{
BIEN_trait_country(
  country,
  trait.name = NULL,
  all.taxonomy = FALSE,
  political.boundaries = TRUE,
  source.citation = FALSE,
  ...
)
}
\arguments{
\item{country}{A single country or a vector of countries.}

\item{trait.name}{Optional.  The trait or traits you want returned.  If left blank, all traits will be returned.}

\item{all.taxonomy}{Should full taxonomic information and TNRS output be returned?  Default is FALSE.}

\item{political.boundaries}{Should political boundary information (country, state, etc.) be returned?  Default is FALSE.}

\item{source.citation}{Should readable source information be downloaded for each record?  Note that \code{\link{BIEN_metadata_citation}} may be more useful.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe of all available trait data for the given country.
}
\description{
BIEN_trait_species extracts trait data for the species country.
}
\examples{
\dontrun{
BIEN_trait_country("South Africa")
BIEN_trait_country(country="South Africa",trait="whole plant growth form")}
}
\seealso{
Other trait functions: 
\code{\link{BIEN_trait_family}()},
\code{\link{BIEN_trait_genus}()},
\code{\link{BIEN_trait_list}()},
\code{\link{BIEN_trait_mean}()},
\code{\link{BIEN_trait_species}()},
\code{\link{BIEN_trait_trait}()},
\code{\link{BIEN_trait_traitbyfamily}()},
\code{\link{BIEN_trait_traitbygenus}()},
\code{\link{BIEN_trait_traitbyspecies}()},
\code{\link{BIEN_trait_traits_per_species}()}
}
\concept{trait functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_intersect_species}
\alias{BIEN_ranges_intersect_species}
\title{Download range maps that intersect the range of a given species.}
\usage{
BIEN_ranges_intersect_species(
  species,
  directory = NULL,
  species.names.only = FALSE,
  include.focal = TRUE,
  return.species.list = TRUE,
  include.gid = FALSE,
  ...
)
}
\arguments{
\item{species}{Focal species (or a vector of species) for which to extract intersecting ranges.}

\item{directory}{Directory that range maps should be saved in.  If none is specified, range maps will be saved in the current working directory.}

\item{species.names.only}{Return species names rather than spatial data? Default is FALSE.}

\item{include.focal}{Should a range for the focal species be downloaded? Default is TRUE.}

\item{return.species.list}{Should a species list be returned in addition to downloading range maps?  Default is FALSE}

\item{include.gid}{Should the files returned have a unique GID appended to them? This is needed if downloading multiple maps for the same species.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Range maps for all available species that intersect the range of the focal species.
}
\description{
BIEN_ranges_intersect_species extracts range maps for a specified bounding box.
}
\note{
Details on the construction of BIEN range maps is available at https://bien.nceas.ucsb.edu/bien/biendata/bien-3/
}
\examples{
\dontrun{
temp_dir <- file.path(tempdir(), "BIEN_temp") #Set a working directory
BIEN_ranges_intersect_species(species = "Carnegiea_gigantea",
directory = temp_dir,include.focal = TRUE)
species_vector<-c("Carnegiea_gigantea","Echinocereus coccineus")
BIEN_ranges_intersect_species(species = species_vector,species.names.only = TRUE)
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_stem_datasource}
\alias{BIEN_stem_datasource}
\title{Extract stem data for a given datasource from BIEN}
\usage{
BIEN_stem_datasource(
  datasource,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  political.boundaries = FALSE,
  collection.info = FALSE,
  all.metadata = FALSE,
  ...
)
}
\arguments{
\item{datasource}{A single datasource, or a vector of datasources.}

\item{cultivated}{Return known cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{all.metadata}{Should additional plot metadata be returned?  Default is FALSE.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing stem data for the specified datasource.
}
\description{
BIEN_stem_datasource downloads occurrence records for specific datasources from the BIEN database.
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php

Setting either "cultivated" or "native.status" to TRUE will significantly slow the speed of a query.

#' @note For a list of available datasources, use \code{\link{BIEN_plot_list_datasource}}.
}
\examples{
\dontrun{
BIEN_stem_datasource(datasource = "SALVIAS")}
}
\seealso{
Other stem functions: 
\code{\link{BIEN_stem_family}()},
\code{\link{BIEN_stem_genus}()},
\code{\link{BIEN_stem_sampling_protocol}()},
\code{\link{BIEN_stem_species}()}
}
\concept{stem functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_occurrence_sf}
\alias{BIEN_occurrence_sf}
\title{Extract occurrence data for specified sf polygon}
\usage{
BIEN_occurrence_sf(
  sf,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  observation.type = FALSE,
  political.boundaries = FALSE,
  collection.info = FALSE,
  only.geovalid = TRUE,
  ...
)
}
\arguments{
\item{sf}{An object of class sf. Note that the projection must be WGS84.}

\item{cultivated}{Return known cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{observation.type}{Return information on type of observation (i.e. specimen vs. plot)?  The default value is FALSE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{only.geovalid}{Should the returned records be limited to those with validated coordinates?  Default is TRUE}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing occurrence records falling within the polygon.
}
\description{
BIEN_occurrence_sf downloads occurrence records falling within a user-specified sf polygon
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php
}
\examples{
\dontrun{
library(sf)  

# first, we download an example shapefile to use (a species range)

BIEN_ranges_species("Carnegiea gigantea")#saves range to the current working directory

# load the range map as an sf object

sf <- st_read(dsn = ".",layer = "Carnegiea_gigantea")

# get the occurrences that occur within the polygon.

species_occurrences <- BIEN_occurrence_sf(sf = sf)
}
}
\seealso{
Other occurrence functions: 
\code{\link{BIEN_occurrence_box}()},
\code{\link{BIEN_occurrence_country}()},
\code{\link{BIEN_occurrence_county}()},
\code{\link{BIEN_occurrence_family}()},
\code{\link{BIEN_occurrence_genus}()},
\code{\link{BIEN_occurrence_records_per_species}()},
\code{\link{BIEN_occurrence_species}()},
\code{\link{BIEN_occurrence_state}()}
}
\concept{occurrence functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_phylogeny_complete}
\alias{BIEN_phylogeny_complete}
\title{Download the complete BIEN phylogenies}
\usage{
BIEN_phylogeny_complete(n_phylogenies = 1, seed = NULL, replicates = NULL, ...)
}
\arguments{
\item{n_phylogenies}{The number of phylogenies to download.  Should be an integer between 1 and 100.  Default is 1.}

\item{seed}{Argument passed to set.seed.  Useful for replicating work with random phylogeny sets.}

\item{replicates}{The specific replicated phylogenies to return.  Should be a numeric vector of integers between 1 and 100.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A phylo or multiphylo object containing the specified phylogenies
}
\description{
BIEN_phylogeny_complete downloads a specified number of the BIEN phylogeny replicates.
}
\note{
Information on the construction of the BIEN phylogenies is available online at https://bien.nceas.ucsb.edu/bien/biendata/bien-2/phylogeny/
}
\examples{
\dontrun{
phylos<-BIEN_phylogeny_complete(n_phylogenies = 10,seed = 1)
phylos<-BIEN_phylogeny_complete(replicates = c(1,2,99,100))}
}
\seealso{
Other phylogeny functions: 
\code{\link{BIEN_phylogeny_conservative}()},
\code{\link{BIEN_phylogeny_label_nodes}()}
}
\concept{phylogeny functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.taxonomy_check_plot}
\alias{.taxonomy_check_plot}
\title{Set query details}
\usage{
.taxonomy_check_plot(all.taxonomy)
}
\description{
Helper function to set query components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_list_country}
\alias{BIEN_list_country}
\title{Extract species list by country}
\usage{
BIEN_list_country(
  country = NULL,
  country.code = NULL,
  cultivated = FALSE,
  new.world = NULL,
  ...
)
}
\arguments{
\item{country}{A single country or a vector of countries.}

\item{country.code}{A single country code or a vector of country codes equal in length to the vector of states/province codes.}

\item{cultivated}{Return information on cultivation status?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing species list(s) for the specified country or countries.
}
\description{
BIEN_list_country downloads a list of all species within a country or countries from the BIEN database.
}
\note{
Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
}
\examples{
\dontrun{
BIEN_list_country("Canada")
country_vector<-c("Canada","United States")
BIEN_list_country(country_vector)}
}
\seealso{
Other list functions: 
\code{\link{BIEN_list_all}()},
\code{\link{BIEN_list_county}()},
\code{\link{BIEN_list_sf}()},
\code{\link{BIEN_list_state}()}
}
\concept{list functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{.md_check_stem}
\alias{.md_check_stem}
\title{Set query details}
\usage{
.md_check_stem(all.metadata)
}
\description{
Helper function to set query components.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_trait_mean}
\alias{BIEN_trait_mean}
\title{Calculates species mean values for a given trait, using Genus or Family level data where Species level data is lacking.}
\usage{
BIEN_trait_mean(species, trait, ...)
}
\arguments{
\item{species}{A single species or a vector of species.}

\item{trait}{A single trait.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe of estimated trait means and associated metadata for the given species.
}
\description{
BIEN_trait_mean Estimates species mean values for a given trait, using Genus or Family level data where Species level data is absent.
}
\note{
Trait spelling needs to be exact and case-sensitive, see \code{\link{BIEN_trait_list}} for a list of traits.
}
\examples{
\dontrun{
BIEN_trait_mean(species=c("Poa annua","Juncus trifidus"),trait="leaf dry mass per leaf fresh mass") }
}
\seealso{
Other trait functions: 
\code{\link{BIEN_trait_country}()},
\code{\link{BIEN_trait_family}()},
\code{\link{BIEN_trait_genus}()},
\code{\link{BIEN_trait_list}()},
\code{\link{BIEN_trait_species}()},
\code{\link{BIEN_trait_trait}()},
\code{\link{BIEN_trait_traitbyfamily}()},
\code{\link{BIEN_trait_traitbygenus}()},
\code{\link{BIEN_trait_traitbyspecies}()},
\code{\link{BIEN_trait_traits_per_species}()}
}
\concept{trait functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_occurrence_country}
\alias{BIEN_occurrence_country}
\title{Extract species occurrence records by country.}
\usage{
BIEN_occurrence_country(
  country = NULL,
  country.code = NULL,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  observation.type = FALSE,
  political.boundaries = FALSE,
  collection.info = FALSE,
  only.geovalid = TRUE,
  ...
)
}
\arguments{
\item{country}{A single country or a vector of country.}

\item{country.code}{A single country code or a vector of country codes equal in length to the vector of states/province codes.}

\item{cultivated}{Return known cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{observation.type}{Return information on type of observation (i.e. specimen vs. plot)?  The default value is FALSE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{only.geovalid}{Should the returned records be limited to those with validated coordinates?  Default is TRUE}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing occurrence records for the specified country.
}
\description{
BIEN_occurrence_country extracts occurrences records for the specified country/countries.
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php

Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
}
\examples{
\dontrun{
BIEN_occurrence_country("Cuba")
country_vector<-c("Cuba","Bahamas")
BIEN_occurrence_country(country_vector)}
}
\seealso{
Other occurrence functions: 
\code{\link{BIEN_occurrence_box}()},
\code{\link{BIEN_occurrence_county}()},
\code{\link{BIEN_occurrence_family}()},
\code{\link{BIEN_occurrence_genus}()},
\code{\link{BIEN_occurrence_records_per_species}()},
\code{\link{BIEN_occurrence_sf}()},
\code{\link{BIEN_occurrence_species}()},
\code{\link{BIEN_occurrence_state}()}
}
\concept{occurrence functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_shapefile_to_skinny}
\alias{BIEN_ranges_shapefile_to_skinny}
\title{Extract range data and convert to smaller "skinny" format}
\usage{
BIEN_ranges_shapefile_to_skinny(directory, raster, skinny_ranges_file = NULL)
}
\arguments{
\item{directory}{The directory where range shapefiles will be stored.  If NULL, a temporary directory will be used.}

\item{raster}{A raster (which must have a CRS specified) to be used for rasterizing the ranges.}

\item{skinny_ranges_file}{A filename that will be used to write the skinny ranges will be written to (RDS format).  If NULL, this will not be written.}
}
\value{
Matrix containing 2 columns: 1) Species name; and 2) the raster cell number it occurs within.
}
\description{
BIEN_ranges_shapefile_to_skinny converts ranges to a "skinny" format to save space.
}
\examples{
\dontrun{
BIEN_ranges_shapefile_to_skinny(directory = BIEN_ranges_species_bulk(species = c("Acer rubrum")),
raster = terra::rast(crs = "+proj=laea +lat_0=15 +lon_0=-80 +x_0=0 +y_0=0 +datum=WGS84 
+units=m +no_defs +ellps=WGS84 +towgs84=0,0,0",
           extent = terra::ext(c(-5261554,5038446,-7434988,7165012 )),
           resolution =c(100000,100000)
           )
)
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()},
\code{\link{BIEN_ranges_species_bulk}()}
}
\concept{range functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_plot_state}
\alias{BIEN_plot_state}
\title{Download plot data from specified states/provinces.}
\usage{
BIEN_plot_state(
  country = NULL,
  state = NULL,
  country.code = NULL,
  state.code = NULL,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  political.boundaries = TRUE,
  collection.info = FALSE,
  all.metadata = FALSE,
  ...
)
}
\arguments{
\item{country}{A single country.}

\item{state}{A state or vector of states (or other primary political divisions).}

\item{country.code}{A single country code or a vector of country codes equal in length to the vector of states/province codes.}

\item{state.code}{A single state/province code, or a vector of states/province codes.}

\item{cultivated}{Return cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{all.metadata}{Should additional plot metadata be returned?  Default is FALSE.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A dataframe containing all data from the specified states.
}
\description{
BIEN_plot_state downloads all plot data from specified states/provinces.
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php

Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.

This function requires you supply either 1) a single country with one or states, or 2) vectors of equal length for each political level.
}
\examples{
\dontrun{
BIEN_plot_state(country="United States", state="Colorado")
BIEN_plot_state(country="United States",state= c("Colorado","California"))}
}
\seealso{
Other plot functions: 
\code{\link{BIEN_plot_country}()},
\code{\link{BIEN_plot_dataset}()},
\code{\link{BIEN_plot_datasource}()},
\code{\link{BIEN_plot_list_datasource}()},
\code{\link{BIEN_plot_list_sampling_protocols}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_plot_name}()},
\code{\link{BIEN_plot_sampling_protocol}()},
\code{\link{BIEN_plot_sf}()}
}
\concept{plot functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_occurrence_box}
\alias{BIEN_occurrence_box}
\title{Extract species occurrence records by a latitude/longitude bounding box.}
\usage{
BIEN_occurrence_box(
  min.lat,
  max.lat,
  min.long,
  max.long,
  species = NULL,
  genus = NULL,
  cultivated = FALSE,
  new.world = NULL,
  all.taxonomy = FALSE,
  native.status = FALSE,
  natives.only = TRUE,
  observation.type = FALSE,
  political.boundaries = TRUE,
  collection.info = FALSE,
  ...
)
}
\arguments{
\item{min.lat}{Minimum latitude}

\item{max.lat}{Maximum latitude}

\item{min.long}{Minimum longitude}

\item{max.long}{Maximum longitude}

\item{species}{Optional.  A single species or a vector of species.}

\item{genus}{Optional. A single genus or a vector of genera.}

\item{cultivated}{Return known cultivated records as well?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{all.taxonomy}{Return all taxonomic information?  This includes the raw data as well as the "scrubbed" data.}

\item{native.status}{Return information on introduction status?  The default value is FALSE. A value of TRUE also returns additional information on introduction status.}

\item{natives.only}{Exclude detected introduced species?  Default is TRUE.}

\item{observation.type}{Return information on type of observation (i.e. specimen vs. plot)?  The default value is FALSE.}

\item{political.boundaries}{Return information on political boundaries for an observation? The default value is FALSE.}

\item{collection.info}{Return additional information about collection and identification? The default value is FALSE.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing occurrence records for the specified area.
}
\description{
BIEN_occurrence_box extracts occurrences records falling within the specific area.
}
\note{
US FIA coordinates have been fuzzed and swapped, for more details see: https://www.fia.fs.fed.us/tools-data/spatial/Policy/index.php

Specifying species and/or genera will limit records returned to that set of taxa.
}
\examples{
\dontrun{
output_test<-
BIEN_occurrence_box(min.lat = 32,max.lat = 33,min.long = -114,max.long = -113,
cultivated = TRUE, new.world = FALSE)}
}
\seealso{
Other occurrence functions: 
\code{\link{BIEN_occurrence_country}()},
\code{\link{BIEN_occurrence_county}()},
\code{\link{BIEN_occurrence_family}()},
\code{\link{BIEN_occurrence_genus}()},
\code{\link{BIEN_occurrence_records_per_species}()},
\code{\link{BIEN_occurrence_sf}()},
\code{\link{BIEN_occurrence_species}()},
\code{\link{BIEN_occurrence_state}()}
}
\concept{occurrence functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_list_sf}
\alias{BIEN_list_sf}
\title{Extract a list of species within a given sf polygon.}
\usage{
BIEN_list_sf(sf, cultivated = FALSE, new.world = NULL, ...)
}
\arguments{
\item{sf}{An object of class ff.  Note that the object must be in WGS84.}

\item{cultivated}{Return information on cultivation status?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing a list of all species with occurrences in the supplied sf object.
}
\description{
BIEN_list_sf produces a list of all species with occurrence records falling within a user-supplied sf object.
}
\examples{
\dontrun{
library(sf)

BIEN_ranges_species("Carnegiea gigantea") # saves ranges to the current working directory

sf <- st_read(dsn = ".",
              layer = "Carnegiea_gigantea")

species_list <- BIEN_list_sf(sf = sf)
}
}
\seealso{
Other list functions: 
\code{\link{BIEN_list_all}()},
\code{\link{BIEN_list_country}()},
\code{\link{BIEN_list_county}()},
\code{\link{BIEN_list_state}()}
}
\concept{list functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_metadata_database_version}
\alias{BIEN_metadata_database_version}
\title{Download the current BIEN database version and release date}
\usage{
BIEN_metadata_database_version(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A data frame containing the current version number and release date for the BIEN database.
}
\description{
BIEN_metadata_database_version downloads the current version number and release date for the BIEN database.
}
\examples{
\dontrun{
BIEN_metadata_database_version()}
}
\seealso{
Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_metadata_data_dictionaries}
\alias{BIEN_metadata_data_dictionaries}
\title{Download data dictionaries}
\usage{
BIEN_metadata_data_dictionaries(...)
}
\arguments{
\item{...}{Additional arguments passed to internal functions.}
}
\value{
A list containing data.frames containing different data dictionaries.
"Tables" contains information on the contents of tables.
"Columns" contains information on the contents of columns within tables.
"Values" contains information on some of the values that are used to populated columns, particularly where these may be unclear.
"rbien" contains information on some of the values returned by the BIEN R package.
}
\description{
BIEN_metadata_data_dictionaries downloads the data dictionaries for the BIEN database.
}
\examples{
\dontrun{
BIEN_metadata_data_dictionaries()}
}
\seealso{
Other metadata functions: 
\code{\link{BIEN_metadata_citation}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_list_state}
\alias{BIEN_list_state}
\title{Extract a species list by state/province}
\usage{
BIEN_list_state(
  country = NULL,
  country.code = NULL,
  state = NULL,
  state.code = NULL,
  cultivated = FALSE,
  new.world = NULL,
  ...
)
}
\arguments{
\item{country}{A single country or a vector of countries equal in length to the vector of states/provinces.}

\item{country.code}{A single country code or a vector of country codes equal in length to the vector of states/province codes.}

\item{state}{A state or vector of states (or other primary political divisions, e.g. provinces).}

\item{state.code}{A single state/province code, or a vector of states/province codes.}

\item{cultivated}{Return information on cultivation status?  Default is FALSE.}

\item{new.world}{NULL (The default) returns global records, TRUE returns only New World, and FALSE only Old World.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing species list(s) for the specified states/provinces.
}
\description{
BIEN_list_state produces a list of all species with geovalidated occurrences falling within specified state(s) or province(s).
}
\note{
Political division (or political division code) spelling needs to be exact and case-sensitive, see \code{\link{BIEN_metadata_list_political_names}} for a list of political divisions and associated codes.
}
\examples{
\dontrun{
BIEN_list_state("United States","Michigan")
state_vector<-c("Michigan","Arizona")
BIEN_list_state(country="United States", state= state_vector)}
}
\seealso{
Other list functions: 
\code{\link{BIEN_list_all}()},
\code{\link{BIEN_list_country}()},
\code{\link{BIEN_list_county}()},
\code{\link{BIEN_list_sf}()}
}
\concept{list functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_taxonomy_genus}
\alias{BIEN_taxonomy_genus}
\title{Extract taxonomic information for genera}
\usage{
BIEN_taxonomy_genus(genus, ...)
}
\arguments{
\item{genus}{A single genus or a vector of genera.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
Dataframe containing taxonomic information for the specified genera.
}
\description{
BIEN_taxonomy_genus downloads a dataframe of all taxonomic information for given genera.
}
\examples{
\dontrun{
BIEN_taxonomy_genus("Acer")
genus_vector<-c("Acer","Quercus")
BIEN_taxonomy_genus(genus_vector)}
}
\seealso{
Other taxonomy functions: 
\code{\link{BIEN_taxonomy_family}()},
\code{\link{BIEN_taxonomy_species}()}
}
\concept{taxonomy functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_metadata_citation}
\alias{BIEN_metadata_citation}
\title{Generate citations for data extracted from BIEN.}
\usage{
BIEN_metadata_citation(
  dataframe = NULL,
  trait.dataframe = NULL,
  trait.mean.dataframe = NULL,
  bibtex_file = NULL,
  acknowledgement_file = NULL,
  ...
)
}
\arguments{
\item{dataframe}{A data.frame of occurrence data downloaded from the BIEN R package.}

\item{trait.dataframe}{A data.frame of trait data downloaded from the BIEN R package.}

\item{trait.mean.dataframe}{A data.frame of species mean trait data from the function BIEN_trait_mean.}

\item{bibtex_file}{Output file for writing bibtex citations.}

\item{acknowledgement_file}{Output file for writing acknowledgements.}

\item{...}{Additional arguments passed to internal functions.}
}
\value{
A list object containing information needed for data attribution.  Full information for herbaria is available at http://sweetgum.nybg.org/science/ih/
}
\description{
BIEN_metadata_citation guides a user through the proper documentation for data downloaded from the BIEN database.
}
\examples{
\dontrun{
BIEN_metadata_citation()#If you are referencing the phylogeny or range maps.
Xanthium_data<-BIEN_occurrence_species("Xanthium strumarium")
citations<-BIEN_metadata_citation(dataframe=Xanthium_data)#If you are referencing occurrence data}
}
\seealso{
Other metadata functions: 
\code{\link{BIEN_metadata_data_dictionaries}()},
\code{\link{BIEN_metadata_database_version}()},
\code{\link{BIEN_metadata_list_political_names}()},
\code{\link{BIEN_metadata_match_data}()},
\code{\link{BIEN_plot_metadata}()},
\code{\link{BIEN_ranges_list}()}
}
\concept{metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BIEN.R
\name{BIEN_ranges_species_bulk}
\alias{BIEN_ranges_species_bulk}
\title{Extract range data for large numbers of species}
\usage{
BIEN_ranges_species_bulk(
  species = NULL,
  directory = NULL,
  batch_size = 1000,
  return_directory = TRUE,
  use_parallel = FALSE
)
}
\arguments{
\item{species}{A vector of species or NULL (the default).  If NULL, all available ranges will be used.}

\item{directory}{The directory where range shapefiles will be stored.  If NULL, a temporary directory will be used.}

\item{batch_size}{The number of ranges to download at once.}

\item{return_directory}{Should the directory be returned? Default is TRUE}

\item{use_parallel}{Logical.  Should batches be downloaded in parallel?  If set to TRUE, AND if parallel and foreach are available, parallel processing of downloads will use n-1 clusters.}
}
\value{
Optionally, the directory to which the files were saved.
}
\description{
BIEN_ranges_species_bulk downloads ranges for a large number of species using parallel processing.
}
\note{
This function may take a long time (hours) to run depending on the number of cores, download speed, etc.
}
\examples{
\dontrun{
#To download all BIEN ranges maps:
BIEN_ranges_species_bulk()
}
}
\seealso{
Other range functions: 
\code{\link{BIEN_ranges_box}()},
\code{\link{BIEN_ranges_genus}()},
\code{\link{BIEN_ranges_intersect_species}()},
\code{\link{BIEN_ranges_list}()},
\code{\link{BIEN_ranges_load_species}()},
\code{\link{BIEN_ranges_sf}()},
\code{\link{BIEN_ranges_shapefile_to_skinny}()},
\code{\link{BIEN_ranges_skinny_ranges_to_richness_raster}()},
\code{\link{BIEN_ranges_species}()}
}
\concept{range functions}
